@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 2000-2024 ircd-hybrid development team
+ *  Copyright (c) 2000-2025 ircd-hybrid development team
  *  Copyright (c) 1996-2023 The Squid Software Foundation and contributors
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -596,3 +596,68 @@ list_get_shorter(list_t *list1, list_t *list2)
 
   return list2;
 }
+
+/**
+ * @brief Converts a double-linked list to an array.
+ *
+ * This function converts all the nodes of a double-linked list into an array of void pointers.
+ * The array will contain pointers to the data of each node, in the order they appear in the list.
+ * The caller is responsible for freeing the allocated array.
+ *
+ * @param list Pointer to the double-linked list.
+ * @return Pointer to the newly allocated array, or NULL if the list is empty.
+ */
+void **
+list_to_array(const list_t *list)
+{
+  if (list_is_empty(list))
+    return NULL;
+
+  void **array = io_calloc(list->length * sizeof(void *));
+
+  unsigned int index = 0;
+  list_node_t *node = list->head;
+  while (node)
+  {
+    array[index++] = node->data;
+    node = node->next;
+  }
+
+  return array;
+}
+
+/**
+ * @brief Peeks at the data of the head node of a double-linked list without removing the node.
+ *
+ * This function returns a pointer to the data stored in the head node (the first node)
+ * of the specified double-linked list. If the list is empty, it returns NULL.
+ *
+ * @param list Pointer to the double-linked list to peek at.
+ * @return Pointer to the data of the head node, or NULL if the list is empty.
+ */
+void *
+list_peek_head(const list_t *list)
+{
+  if (list_is_empty(list))
+    return NULL;
+
+  return list->head->data;
+}
+
+/**
+ * @brief Peeks at the data of the tail node of a double-linked list without removing the node.
+ *
+ * This function returns a pointer to the data stored in the tail node (the last node)
+ * of the specified double-linked list. If the list is empty, it returns NULL.
+ *
+ * @param list Pointer to the double-linked list to peek at.
+ * @return Pointer to the data of the tail node, or NULL if the list is empty.
+ */
+void *
+list_peek_tail(const list_t *list)
+{
+  if (list_is_empty(list))
+    return NULL;
+
+  return list->tail->data;
+}
