@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@
 #include <stddef.h>
 #include <stdbool.h>
 
-#include "irc_string.h"
+#include "io_string.h"
 
 /**
  * @brief Matches a string against a wildcard pattern with support for '*' and '?' wildcards.
