@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 2000-2024 ircd-hybrid development team
+ *  Copyright (c) 2000-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@
 #include "ltdl.h"
 #include "stdinc.h"
 #include "io.h"
-#include "irc_string.h"
+#include "io_string.h"
 #include "list.h"
 #include "log.h"
 #include "memory.h"
