@@ -42,42 +42,35 @@
 #include <netinet/in.h> /* BSD, Linux: for inet_addr */
 #include <arpa/inet.h> /* BSD, Linux, Solaris: for inet_addr */
 
-#include "patricia.h"
 #include "memory.h"
+#include "patricia.h"
 
-/* { from prefix.c */
+#define prefix_to_uint8(prefix) ((const uint8_t *)&(prefix)->add.sin)
 
-/* prefix_tochar
- * convert prefix information to bytes
- */
-static unsigned char *
-prefix_tochar(prefix_t *prefix)
+static bool
+comp_with_mask(const uint8_t *addr, const uint8_t *dest, unsigned int mask)
 {
-  if (prefix == NULL)
-    return NULL;
-  return (unsigned char *)&prefix->add.sin;
-}
+  /* Mask length of zero matches everything. */
+  if (mask == 0)
+    return true;
 
-static int
-comp_with_mask(void *addr, void *dest, unsigned int mask)
-{
-  if ( /* mask/8 == 0 || */ memcmp(addr, dest, mask / 8) == 0)
+  if (memcmp(addr, dest, mask / 8) == 0)
   {
     int n = mask / 8;
     int m = ~((1 << (8 - (mask % 8))) - 1);
 
-    if (mask % 8 == 0 || (((unsigned char *)addr)[n] & m) == (((unsigned char *)dest)[n] & m))
-      return 1;
+    if (mask % 8 == 0 || (addr[n] & m) == (dest[n] & m))
+      return true;
   }
 
-  return 0;
+  return false;
 }
 
 /*
  * convert prefix information to ascii string with length
  */
 const char *
-patricia_prefix_toa(const prefix_t *prefix, int with_len)
+patricia_prefix_toa(const patricia_prefix_t *prefix, bool with_len)
 {
   static char buf[INET6_ADDRSTRLEN + sizeof("/128")];
 
@@ -89,14 +82,17 @@ patricia_prefix_toa(const prefix_t *prefix, int with_len)
   inet_ntop(prefix->family, &prefix->add.sin6, buf, INET6_ADDRSTRLEN);
 
   if (with_len)
-    sprintf(buf + strlen(buf), "/%d", prefix->bitlen);
+  {
+    size_t len = strlen(buf);
+    snprintf(buf + len, sizeof(buf) - len, "/%d", prefix->bitlen);
+  }
+
   return buf;
 }
 
-static prefix_t *
-New_Prefix2(int family, void *dest, int bitlen, prefix_t *prefix)
+static patricia_prefix_t *
+New_Prefix2(int family, const void *dest, int bitlen, patricia_prefix_t *prefix)
 {
-  int dynamic_allocated = 0;
   int addr_size = 0;
 
   switch (family)
@@ -110,61 +106,50 @@ New_Prefix2(int family, void *dest, int bitlen, prefix_t *prefix)
     default: return NULL;
   }
 
+  bool dynamic_allocated = false;
   if (prefix == NULL)
   {
     prefix = io_calloc(sizeof(*prefix));
-    dynamic_allocated = 1;
+    dynamic_allocated = true;
   }
 
   memcpy(&prefix->add.sin6, dest, addr_size);
   prefix->bitlen = (bitlen >= 0) ? bitlen : addr_size * 8;
   prefix->family = family;
-  prefix->ref_count = dynamic_allocated == 1;
+  prefix->ref_count = dynamic_allocated ? 1 : 0;
 
-/* fprintf(stderr, "[C %s, %d]\n", patricia_prefix_toa(prefix), prefix->ref_count); */
   return prefix;
 }
 
-static prefix_t *
-New_Prefix(int family, void *dest, int bitlen)
+static patricia_prefix_t *
+New_Prefix(int family, const void *dest, int bitlen)
 {
   return New_Prefix2(family, dest, bitlen, NULL);
 }
 
 /* ascii2prefix
  */
-static prefix_t *
+static patricia_prefix_t *
 ascii2prefix(int family, const char *string)
 {
-  char save[INET6_ADDRSTRLEN];
   int bitlen, maxbitlen = 0;
-  union
-  {
-    struct in_addr sin;
-    struct in6_addr sin6;
-  } sin;
 
   assert(string);
 
   /* Easy way to handle both families */
   if (family == 0)
-  {
-    family = AF_INET;
-
-    if (strchr(string, ':'))
-      family = AF_INET6;
-  }
+    family = strchr(string, ':') ? AF_INET6 : AF_INET;
 
   if (family == AF_INET)
     maxbitlen = sizeof(struct in_addr)  * 8;
   else if (family == AF_INET6)
     maxbitlen = sizeof(struct in6_addr) * 8;
 
+  char save[INET6_ADDRSTRLEN];
   const char *const cp = strchr(string, '/');
   if (cp)
   {
     size_t length = cp - string;
-
     if (length >= sizeof(save))
       return NULL;
 
@@ -182,13 +167,20 @@ ascii2prefix(int family, const char *string)
   else
     bitlen = maxbitlen;
 
+  union
+  {
+    struct in_addr sin;
+    struct in6_addr sin6;
+  } sin;
+
   if (inet_pton(family, string, &sin) <= 0)
     return NULL;
+
   return New_Prefix(family, &sin, bitlen);
 }
 
-static prefix_t *
-Ref_Prefix(prefix_t *prefix)
+static patricia_prefix_t *
+Ref_Prefix(patricia_prefix_t *prefix)
 {
   if (prefix == NULL)
     return NULL;
@@ -199,12 +191,11 @@ Ref_Prefix(prefix_t *prefix)
 
   prefix->ref_count++;
 
-/* fprintf(stderr, "[A %s, %d]\n", patricia_prefix_toa(prefix), prefix->ref_count); */
   return prefix;
 }
 
 static void
-Deref_Prefix(prefix_t *prefix)
+Deref_Prefix(patricia_prefix_t *prefix)
 {
   if (prefix == NULL)
     return;
@@ -214,20 +205,17 @@ Deref_Prefix(prefix_t *prefix)
   if (--prefix->ref_count <= 0)
     io_free(prefix);
 }
-/* } */
-
-/* #define PATRICIA_DEBUG 1 */
 
 /* these routines support continuous mask only */
 
 patricia_tree_t *
 patricia_new(unsigned int maxbits)
 {
-  patricia_tree_t *patricia = io_calloc(sizeof(*patricia));
-  patricia->maxbits = maxbits;
+  patricia_tree_t *tree = io_calloc(sizeof(*tree));
+  tree->maxbits = maxbits;
 
   assert(maxbits <= PATRICIA_MAXBITS);  /* XXX */
-  return patricia;
+  return tree;
 }
 
 /*
@@ -235,214 +223,139 @@ patricia_new(unsigned int maxbits)
  * before deleting the node
  */
 void
-patricia_clear(patricia_tree_t *patricia, void (*func)(void *))
+patricia_clear(patricia_tree_t *tree, void (*func)(void *))
 {
-  assert(patricia);
+  assert(tree);
 
-  if (patricia->head)
+  if (tree->head == NULL)
+    return;
+
+  patricia_node_t *Xstack[PATRICIA_MAXBITS + 1];
+  patricia_node_t **Xsp = Xstack;
+
+  patricia_node_t *Xrn = tree->head;
+  while (Xrn)
   {
-    patricia_node_t *Xstack[PATRICIA_MAXBITS + 1];
-    patricia_node_t **Xsp = Xstack;
-    patricia_node_t *Xrn = patricia->head;
+    patricia_node_t *l = Xrn->l;
+    patricia_node_t *r = Xrn->r;
 
-    while (Xrn)
+    if (Xrn->prefix)
     {
-      patricia_node_t *l = Xrn->l;
-      patricia_node_t *r = Xrn->r;
+      Deref_Prefix(Xrn->prefix);
 
-      if (Xrn->prefix)
-      {
-        Deref_Prefix(Xrn->prefix);
-
-        if (Xrn->data && func)
-          func(Xrn->data);
-      }
-      else
-      {
-        assert(Xrn->data == NULL);
-      }
-
-      io_free(Xrn);
-      patricia->num_active_node--;
-
-      if (l)
-      {
-        if (r)
-        {
-          *Xsp++ = r;
-        }
-
-        Xrn = l;
-      }
-      else if (r)
-      {
-        Xrn = r;
-      }
-      else if (Xsp != Xstack)
-      {
-        Xrn = *(--Xsp);
-      }
-      else
-      {
-        Xrn = NULL;
-      }
+      if (Xrn->data && func)
+        func(Xrn->data);
     }
+    else
+      assert(Xrn->data == NULL);
+
+    io_free(Xrn);
+    tree->num_active_node--;
+
+    if (l)
+    {
+      if (r)
+        *Xsp++ = r;
+
+      Xrn = l;
+    }
+    else if (r)
+      Xrn = r;
+    else if (Xsp != Xstack)
+      Xrn = *(--Xsp);
+    else
+      Xrn = NULL;
   }
 
-  patricia->head = NULL;
+  tree->head = NULL;
 
-  assert(patricia->num_active_node == 0);
+  assert(tree->num_active_node == 0);
 }
 
 void
-patricia_destroy(patricia_tree_t *patricia, void (*func)(void *))
+patricia_destroy(patricia_tree_t *tree, void (*func)(void *))
 {
-  patricia_clear(patricia, func);
-  io_free(patricia);
+  patricia_clear(tree, func);
+  io_free(tree);
 }
 
 /*
  * if func is supplied, it will be called as func(node->prefix, node->data)
  */
 void
-patricia_process(patricia_tree_t *patricia, void (*func)(prefix_t *, void *))
+patricia_process(patricia_tree_t *tree, void (*func)(patricia_prefix_t *, void *))
 {
-  patricia_node_t *node;
-
   assert(func);
 
-  PATRICIA_WALK(patricia->head, node) {
+  patricia_node_t *node;
+  PATRICIA_WALK(tree->head, node) {
     func(node->prefix, node->data);
   } PATRICIA_WALK_END;
 }
 
 patricia_node_t *
-patricia_search_exact(patricia_tree_t *patricia, prefix_t *prefix)
+patricia_search_exact(patricia_tree_t *tree, patricia_prefix_t *prefix)
 {
-  patricia_node_t *node;
-  unsigned char *addr;
-  unsigned int bitlen;
-
-  assert(patricia);
+  assert(tree);
   assert(prefix);
-  assert(prefix->bitlen <= patricia->maxbits);
+  assert(prefix->bitlen <= tree->maxbits);
 
-  if (patricia->head == NULL)
+  if (tree->head == NULL)
     return NULL;
 
-  node = patricia->head;
-  addr = prefix_touchar(prefix);
-  bitlen = prefix->bitlen;
+  patricia_node_t *node = tree->head;
+  const uint8_t *addr = prefix_to_uint8(prefix);
+  unsigned int bitlen = prefix->bitlen;
 
   while (node->bit < bitlen)
   {
     if (BIT_TEST(addr[node->bit >> 3], 0x80 >> (node->bit & 0x07)))
-    {
-#ifdef PATRICIA_DEBUG
-      if (node->prefix)
-        fprintf(stderr, "patricia_search_exact: take right %s/%d\n",
-                patricia_prefix_toa(node->prefix), node->prefix->bitlen);
-      else
-        fprintf(stderr, "patricia_search_exact: take right at %u\n", node->bit);
-#endif  /* PATRICIA_DEBUG */
       node = node->r;
-    }
     else
-    {
-#ifdef PATRICIA_DEBUG
-      if (node->prefix)
-        fprintf(stderr, "patricia_search_exact: take left %s/%d\n",
-                patricia_prefix_toa(node->prefix), node->prefix->bitlen);
-      else
-        fprintf(stderr, "patricia_search_exact: take left at %u\n", node->bit);
-#endif  /* PATRICIA_DEBUG */
       node = node->l;
-    }
 
     if (node == NULL)
       return NULL;
   }
 
-#ifdef PATRICIA_DEBUG
-  if (node->prefix)
-    fprintf(stderr, "patricia_search_exact: stop at %s/%d\n",
-            patricia_prefix_toa(node->prefix), node->prefix->bitlen);
-  else
-    fprintf(stderr, "patricia_search_exact: stop at %u\n", node->bit);
-#endif  /* PATRICIA_DEBUG */
-
   if (node->bit > bitlen || node->prefix == NULL)
     return NULL;
 
   assert(node->bit == bitlen);
   assert(node->bit == node->prefix->bitlen);
 
-  if (comp_with_mask(prefix_tochar(node->prefix), prefix_tochar(prefix), bitlen))
-  {
-#ifdef PATRICIA_DEBUG
-    fprintf(stderr, "patricia_search_exact: found %s/%d\n",
-            patricia_prefix_toa(node->prefix), node->prefix->bitlen);
-#endif  /* PATRICIA_DEBUG */
+  if (comp_with_mask(prefix_to_uint8(node->prefix), prefix_to_uint8(prefix), bitlen))
     return node;
-  }
 
   return NULL;
 }
 
 /* if inclusive != 0, "best" may be the given prefix itself */
 patricia_node_t *
-patricia_search_best2(patricia_tree_t *patricia, prefix_t *prefix, int inclusive)
+patricia_search_best2(patricia_tree_t *tree, patricia_prefix_t *prefix, bool inclusive)
 {
-  patricia_node_t *node;
-  patricia_node_t *stack[PATRICIA_MAXBITS + 1];
-  unsigned char *addr;
-  unsigned int bitlen;
-  int cnt = 0;
-
-  assert(patricia);
+  assert(tree);
   assert(prefix);
-  assert(prefix->bitlen <= patricia->maxbits);
+  assert(prefix->bitlen <= tree->maxbits);
 
-  if (patricia->head == NULL)
+  if (tree->head == NULL)
     return NULL;
 
-  node = patricia->head;
-  addr = prefix_touchar(prefix);
-  bitlen = prefix->bitlen;
+  patricia_node_t *node = tree->head;
+  patricia_node_t *stack[PATRICIA_MAXBITS + 1];
+  const uint8_t *addr = prefix_to_uint8(prefix);
+  unsigned int bitlen = prefix->bitlen;
+  int cnt = 0;
 
   while (node->bit < bitlen)
   {
     if (node->prefix)
-    {
-#ifdef PATRICIA_DEBUG
-      fprintf(stderr, "patricia_search_best: push %s/%d\n",
-              patricia_prefix_toa(node->prefix), node->prefix->bitlen);
-#endif  /* PATRICIA_DEBUG */
       stack[cnt++] = node;
-    }
 
     if (BIT_TEST(addr[node->bit >> 3], 0x80 >> (node->bit & 0x07)))
-    {
-#ifdef PATRICIA_DEBUG
-      if (node->prefix)
-        fprintf(stderr, "patricia_search_best: take right %s/%d\n",
-                patricia_prefix_toa(node->prefix), node->prefix->bitlen);
-      else
-        fprintf(stderr, "patricia_search_best: take right at %u\n", node->bit);
-#endif  /* PATRICIA_DEBUG */
       node = node->r;
-    }
     else
-    {
-#ifdef PATRICIA_DEBUG
-      if (node->prefix)
-        fprintf(stderr, "patricia_search_best: take left %s/%d\n",
-                patricia_prefix_toa(node->prefix), node->prefix->bitlen);
-      else
-        fprintf(stderr, "patricia_search_best: take left at %u\n", node->bit);
-#endif  /* PATRICIA_DEBUG */
       node = node->l;
-    }
 
     if (node == NULL)
       break;
@@ -451,90 +364,55 @@ patricia_search_best2(patricia_tree_t *patricia, prefix_t *prefix, int inclusive
   if (inclusive && node && node->prefix)
     stack[cnt++] = node;
 
-#ifdef PATRICIA_DEBUG
-  if (node == NULL)
-    fprintf(stderr, "patricia_search_best: stop at null\n");
-  else if (node->prefix)
-    fprintf(stderr, "patricia_search_best: stop at %s/%d\n",
-            patricia_prefix_toa(node->prefix), node->prefix->bitlen);
-  else
-    fprintf(stderr, "patricia_search_best: stop at %u\n", node->bit);
-#endif  /* PATRICIA_DEBUG */
-
   if (cnt <= 0)
     return NULL;
 
   while (--cnt >= 0)
   {
     node = stack[cnt];
-#ifdef PATRICIA_DEBUG
-    fprintf(stderr, "patricia_search_best: pop %s/%d\n",
-           patricia_prefix_toa(node->prefix), node->prefix->bitlen);
-#endif  /* PATRICIA_DEBUG */
 
-    if (comp_with_mask(prefix_tochar(node->prefix),
-                       prefix_tochar(prefix), node->prefix->bitlen) && node->prefix->bitlen <= bitlen)
-    {
-#ifdef PATRICIA_DEBUG
-      fprintf(stderr, "patricia_search_best: found %s/%d\n",
-              patricia_prefix_toa(node->prefix), node->prefix->bitlen);
-#endif  /* PATRICIA_DEBUG */
+    if (comp_with_mask(prefix_to_uint8(node->prefix),
+                       prefix_to_uint8(prefix), node->prefix->bitlen) && node->prefix->bitlen <= bitlen)
       return node;
-    }
   }
 
   return NULL;
 }
 
 patricia_node_t *
-patricia_search_best(patricia_tree_t *patricia, prefix_t *prefix)
+patricia_search_best(patricia_tree_t *tree, patricia_prefix_t *prefix)
 {
-  return patricia_search_best2(patricia, prefix, 1);
+  return patricia_search_best2(tree, prefix, 1);
 }
 
 patricia_node_t *
-patricia_lookup(patricia_tree_t *patricia, prefix_t *prefix)
+patricia_lookup(patricia_tree_t *tree, patricia_prefix_t *prefix)
 {
-  patricia_node_t *node, *new_node, *parent, *glue;
-  unsigned char *addr, *test_addr;
-  unsigned int bitlen, check_bit, differ_bit;
-  int j, r;
-
-  assert(patricia);
+  assert(tree);
   assert(prefix);
-  assert(prefix->bitlen <= patricia->maxbits);
+  assert(prefix->bitlen <= tree->maxbits);
 
-  if (patricia->head == NULL)
+  if (tree->head == NULL)
   {
-    node = io_calloc(sizeof(*node));
+    patricia_node_t *node = io_calloc(sizeof(*node));
     node->bit = prefix->bitlen;
     node->prefix = Ref_Prefix(prefix);
-    patricia->head = node;
-#ifdef PATRICIA_DEBUG
-    fprintf(stderr, "patricia_lookup: new_node #0 %s/%d (head)\n",
-            patricia_prefix_toa(prefix), prefix->bitlen);
-#endif  /* PATRICIA_DEBUG */
-    patricia->num_active_node++;
+    tree->head = node;
+    tree->num_active_node++;
+
     return node;
   }
 
-  addr = prefix_touchar(prefix);
-  bitlen = prefix->bitlen;
-  node = patricia->head;
+  const uint8_t *addr = prefix_to_uint8(prefix);
+  unsigned int bitlen = prefix->bitlen;
+  patricia_node_t *node = tree->head;
 
   while (node->bit < bitlen || node->prefix == NULL)
   {
-    if (node->bit < patricia->maxbits && BIT_TEST(addr[node->bit >> 3], 0x80 >> (node->bit & 0x07)))
+    if (node->bit < tree->maxbits && BIT_TEST(addr[node->bit >> 3], 0x80 >> (node->bit & 0x07)))
     {
       if (node->r == NULL)
         break;
-#ifdef PATRICIA_DEBUG
-      if (node->prefix)
-        fprintf(stderr, "patricia_lookup: take right %s/%d\n",
-                patricia_prefix_toa(node->prefix), node->prefix->bitlen);
-      else
-        fprintf(stderr, "patricia_lookup: take right at %u\n", node->bit);
-#endif  /* PATRICIA_DEBUG */
 
       node = node->r;
     }
@@ -542,13 +420,6 @@ patricia_lookup(patricia_tree_t *patricia, prefix_t *prefix)
     {
       if (node->l == NULL)
         break;
-#ifdef PATRICIA_DEBUG
-      if (node->prefix)
-        fprintf(stderr, "patricia_lookup: take left %s/%d\n",
-                patricia_prefix_toa(node->prefix), node->prefix->bitlen);
-      else
-        fprintf(stderr, "patricia_lookup: take left at %u\n", node->bit);
-#endif  /* PATRICIA_DEBUG */
 
       node = node->l;
     }
@@ -557,16 +428,13 @@ patricia_lookup(patricia_tree_t *patricia, prefix_t *prefix)
   }
 
   assert(node->prefix);
-#ifdef PATRICIA_DEBUG
-  fprintf(stderr, "patricia_lookup: stop at %s/%d\n",
-          patricia_prefix_toa(node->prefix), node->prefix->bitlen);
-#endif /* PATRICIA_DEBUG */
 
-  test_addr = prefix_touchar(node->prefix);
+  const uint8_t *test_addr = prefix_to_uint8(node->prefix);
 
   /* Find the first bit different */
-  check_bit = (node->bit < bitlen) ? node->bit : bitlen;
-  differ_bit = 0;
+  unsigned int check_bit = node->bit < bitlen ? node->bit : bitlen;
+  unsigned int differ_bit = 0;
+  int j, r;
 
   for (unsigned int i = 0; i * 8 < check_bit; i++)
   {
@@ -589,58 +457,35 @@ patricia_lookup(patricia_tree_t *patricia, prefix_t *prefix)
 
   if (differ_bit > check_bit)
     differ_bit = check_bit;
-#ifdef PATRICIA_DEBUG
-  fprintf(stderr, "patricia_lookup: differ_bit %d\n", differ_bit);
-#endif  /* PATRICIA_DEBUG */
-
-  parent = node->parent;
 
+  patricia_node_t *parent = node->parent;
   while (parent && parent->bit >= differ_bit)
   {
     node = parent;
     parent = node->parent;
-
-#ifdef PATRICIA_DEBUG
-    if (node->prefix)
-      fprintf(stderr, "patricia_lookup: up to %s/%d\n",
-              patricia_prefix_toa(node->prefix), node->prefix->bitlen);
-    else
-      fprintf(stderr, "patricia_lookup: up to %u\n", node->bit);
-#endif  /* PATRICIA_DEBUG */
   }
 
   if (differ_bit == bitlen && node->bit == bitlen)
   {
     if (node->prefix)
-    {
-#ifdef PATRICIA_DEBUG
-      fprintf(stderr, "patricia_lookup: found %s/%d\n",
-              patricia_prefix_toa(node->prefix), node->prefix->bitlen);
-#endif  /* PATRICIA_DEBUG */
-
       return node;
-    }
 
     node->prefix = Ref_Prefix(prefix);
-#ifdef PATRICIA_DEBUG
-    fprintf(stderr, "patricia_lookup: new node #1 %s/%d (glue mod)\n",
-            patricia_prefix_toa(prefix), prefix->bitlen);
-#endif  /* PATRICIA_DEBUG */
-
     assert(node->data == NULL);
+
     return node;
   }
 
-  new_node = io_calloc(sizeof(*new_node));
+  patricia_node_t *new_node = io_calloc(sizeof(*new_node));
   new_node->bit = prefix->bitlen;
   new_node->prefix = Ref_Prefix(prefix);
-  patricia->num_active_node++;
+  tree->num_active_node++;
 
   if (node->bit == differ_bit)
   {
     new_node->parent = node;
 
-    if (node->bit < patricia->maxbits && BIT_TEST(addr[node->bit >> 3], 0x80 >> (node->bit & 0x07)))
+    if (node->bit < tree->maxbits && BIT_TEST(addr[node->bit >> 3], 0x80 >> (node->bit & 0x07)))
     {
       assert(node->r == NULL);
       node->r = new_node;
@@ -650,54 +495,39 @@ patricia_lookup(patricia_tree_t *patricia, prefix_t *prefix)
       assert(node->l == NULL);
       node->l = new_node;
     }
-#ifdef PATRICIA_DEBUG
-    fprintf(stderr, "patricia_lookup: new_node #2 %s/%d (child)\n",
-            patricia_prefix_toa(prefix), prefix->bitlen);
-#endif  /* PATRICIA_DEBUG */
+
     return new_node;
   }
 
   if (bitlen == differ_bit)
   {
-    if (bitlen < patricia->maxbits && BIT_TEST(test_addr[bitlen >> 3], 0x80 >> (bitlen & 0x07)))
-    {
+    if (bitlen < tree->maxbits && BIT_TEST(test_addr[bitlen >> 3], 0x80 >> (bitlen & 0x07)))
       new_node->r = node;
-    }
     else
-    {
       new_node->l = node;
-    }
 
     new_node->parent = node->parent;
 
     if (node->parent == NULL)
     {
-      assert(patricia->head == node);
-      patricia->head = new_node;
+      assert(tree->head == node);
+      tree->head = new_node;
     }
     else if (node->parent->r == node)
-    {
       node->parent->r = new_node;
-    }
     else
-    {
       node->parent->l = new_node;
-    }
 
     node->parent = new_node;
-#ifdef PATRICIA_DEBUG
-    fprintf(stderr, "patricia_lookup: new_node #3 %s/%d (parent)\n",
-            patricia_prefix_toa(prefix), prefix->bitlen);
-#endif  /* PATRICIA_DEBUG */
   }
   else
   {
-    glue = io_calloc(sizeof(*glue));
+    patricia_node_t *glue = io_calloc(sizeof(*glue));
     glue->bit = differ_bit;
     glue->parent = node->parent;
-    patricia->num_active_node++;
+    tree->num_active_node++;
 
-    if (differ_bit < patricia->maxbits && BIT_TEST(addr[differ_bit >> 3], 0x80 >> (differ_bit & 0x07)))
+    if (differ_bit < tree->maxbits && BIT_TEST(addr[differ_bit >> 3], 0x80 >> (differ_bit & 0x07)))
     {
       glue->r = new_node;
       glue->l = node;
@@ -712,43 +542,28 @@ patricia_lookup(patricia_tree_t *patricia, prefix_t *prefix)
 
     if (node->parent == NULL)
     {
-      assert(patricia->head == node);
-      patricia->head = glue;
+      assert(tree->head == node);
+      tree->head = glue;
     }
     else if (node->parent->r == node)
-    {
       node->parent->r = glue;
-    }
     else
-    {
       node->parent->l = glue;
-    }
 
     node->parent = glue;
-#ifdef PATRICIA_DEBUG
-    fprintf(stderr, "patricia_lookup: new_node #4 %s/%d (glue+node)\n",
-            patricia_prefix_toa(prefix), prefix->bitlen);
-#endif  /* PATRICIA_DEBUG */
   }
 
   return new_node;
 }
 
 void
-patricia_remove(patricia_tree_t *patricia, patricia_node_t *node)
+patricia_remove(patricia_tree_t *tree, patricia_node_t *node)
 {
-  patricia_node_t *parent, *child;
-
-  assert(patricia);
+  assert(tree);
   assert(node);
 
   if (node->r && node->l)
   {
-#ifdef PATRICIA_DEBUG
-    fprintf(stderr, "patricia_remove: #0 %s/%d (r & l)\n",
-            patricia_prefix_toa(node->prefix), node->prefix->bitlen);
-#endif  /* PATRICIA_DEBUG */
-
     /*
      * This might be a placeholder node -- have to check and make sure
      * there is a prefix associated with it !
@@ -762,22 +577,18 @@ patricia_remove(patricia_tree_t *patricia, patricia_node_t *node)
     return;
   }
 
+  patricia_node_t *parent, *child;
   if (node->r == NULL && node->l == NULL)
   {
-#ifdef PATRICIA_DEBUG
-    fprintf(stderr, "patricia_remove: #1 %s/%d (!r & !l)\n",
-            patricia_prefix_toa(node->prefix), node->prefix->bitlen);
-#endif  /* PATRICIA_DEBUG */
-
     parent = node->parent;
     Deref_Prefix(node->prefix);
     io_free(node);
-    patricia->num_active_node--;
+    tree->num_active_node--;
 
     if (parent == NULL)
     {
-      assert(patricia->head == node);
-      patricia->head = NULL;
+      assert(tree->head == node);
+      tree->head = NULL;
       return;
     }
 
@@ -800,13 +611,11 @@ patricia_remove(patricia_tree_t *patricia, patricia_node_t *node)
     /* We need to remove parent too */
     if (parent->parent == NULL)
     {
-      assert(patricia->head == parent);
-      patricia->head = child;
+      assert(tree->head == parent);
+      tree->head = child;
     }
     else if (parent->parent->r == parent)
-    {
       parent->parent->r = child;
-    }
     else
     {
       assert(parent->parent->l == parent);
@@ -815,19 +624,12 @@ patricia_remove(patricia_tree_t *patricia, patricia_node_t *node)
 
     child->parent = parent->parent;
     io_free(parent);
-    patricia->num_active_node--;
+    tree->num_active_node--;
     return;
   }
 
-#ifdef PATRICIA_DEBUG
-  fprintf(stderr, "patricia_remove: #2 %s/%d (r ^ l)\n",
-          patricia_prefix_toa(node->prefix), node->prefix->bitlen);
-#endif  /* PATRICIA_DEBUG */
-
   if (node->r)
-  {
     child = node->r;
-  }
   else
   {
     assert(node->l);
@@ -839,19 +641,17 @@ patricia_remove(patricia_tree_t *patricia, patricia_node_t *node)
 
   Deref_Prefix(node->prefix);
   io_free(node);
-  patricia->num_active_node--;
+  tree->num_active_node--;
 
   if (parent == NULL)
   {
-    assert(patricia->head == node);
-    patricia->head = child;
+    assert(tree->head == node);
+    tree->head = child;
     return;
   }
 
   if (parent->r == node)
-  {
     parent->r = child;
-  }
   else
   {
     assert(parent->l == node);
@@ -859,52 +659,48 @@ patricia_remove(patricia_tree_t *patricia, patricia_node_t *node)
   }
 }
 
-/* { from demo.c */
 patricia_node_t *
 patricia_make_and_lookup(patricia_tree_t *tree, const char *string)
 {
-  prefix_t *prefix = ascii2prefix(0, string);
+  patricia_prefix_t *prefix = ascii2prefix(0, string);
+  if (prefix == NULL)
+    return NULL;
 
-  if (prefix)
-  {
-    patricia_node_t *node = patricia_lookup(tree, prefix);
-    Deref_Prefix(prefix);
-    return node;
-  }
-
-  return NULL;
+  patricia_node_t *node = patricia_lookup(tree, prefix);
+  Deref_Prefix(prefix);
+  return node;
 }
 
 patricia_node_t *
-patricia_make_and_lookup_addr(patricia_tree_t *tree, struct sockaddr *addr, int bitlen)
+patricia_make_and_lookup_addr(patricia_tree_t *tree, const struct sockaddr *addr, int bitlen)
 {
   int family;
-  void *dest;
+  const void *dest;
 
   if (addr->sa_family == AF_INET6)
   {
     if (bitlen == 0 || bitlen > 128)
       bitlen = 128;
+
     family = AF_INET6;
-    dest = &((struct sockaddr_in6 *)addr)->sin6_addr;
+    dest = &((const struct sockaddr_in6 *)addr)->sin6_addr;
   }
   else
   {
     if (bitlen == 0 || bitlen > 32)
       bitlen = 32;
+
     family = AF_INET;
-    dest = &((struct sockaddr_in *)addr)->sin_addr;
+    dest = &((const struct sockaddr_in *)addr)->sin_addr;
   }
 
-  prefix_t *prefix = New_Prefix(family, dest, bitlen);
-  if (prefix)
-  {
-    patricia_node_t *node = patricia_lookup(tree, prefix);
-    Deref_Prefix(prefix);
-    return node;
-  }
+  patricia_prefix_t *prefix = New_Prefix(family, dest, bitlen);
+  if (prefix == NULL)
+    return NULL;
 
-  return NULL;
+  patricia_node_t *node = patricia_lookup(tree, prefix);
+  Deref_Prefix(prefix);
+  return node;
 }
 
 void
@@ -918,94 +714,87 @@ patricia_lookup_then_remove(patricia_tree_t *tree, const char *string)
 patricia_node_t *
 patricia_try_search_exact(patricia_tree_t *tree, const char *string)
 {
-  prefix_t *prefix = ascii2prefix(0, string);
+  patricia_prefix_t *prefix = ascii2prefix(0, string);
+  if (prefix == NULL)
+    return NULL;
 
-  if (prefix)
-  {
-    patricia_node_t *node = patricia_search_exact(tree, prefix);
-    Deref_Prefix(prefix);
-    return node;
-  }
-
-  return NULL;
+  patricia_node_t *node = patricia_search_exact(tree, prefix);
+  Deref_Prefix(prefix);
+  return node;
 }
 
 patricia_node_t *
 patricia_try_search_best(patricia_tree_t *tree, const char *string)
 {
-  prefix_t *prefix = ascii2prefix(0, string);
+  patricia_prefix_t *prefix = ascii2prefix(0, string);
+  if (prefix == NULL)
+    return NULL;
 
-  if (prefix)
-  {
-    patricia_node_t *node = patricia_search_best(tree, prefix);
-    Deref_Prefix(prefix);
-    return node;
-  }
-
-  return NULL;
+  patricia_node_t *node = patricia_search_best(tree, prefix);
+  Deref_Prefix(prefix);
+  return node;
 }
 
 patricia_node_t *
-patricia_try_search_exact_addr(patricia_tree_t *tree, struct sockaddr *addr, int bitlen)
+patricia_try_search_exact_addr(patricia_tree_t *tree, const struct sockaddr *addr, int bitlen)
 {
   int family;
-  void *dest;
+  const void *dest;
 
   if (addr->sa_family == AF_INET6)
   {
     if (bitlen == 0 || bitlen > 128)
       bitlen = 128;
+
     family = AF_INET6;
-    dest = &((struct sockaddr_in6 *)addr)->sin6_addr;
+    dest = &((const struct sockaddr_in6 *)addr)->sin6_addr;
   }
   else
   {
     if (bitlen == 0 || bitlen > 32)
       bitlen = 32;
+
     family = AF_INET;
-    dest = &((struct sockaddr_in *)addr)->sin_addr;
+    dest = &((const struct sockaddr_in *)addr)->sin_addr;
   }
 
-  prefix_t *prefix = New_Prefix(family, dest, bitlen);
-  if (prefix)
-  {
-    patricia_node_t *node = patricia_search_exact(tree, prefix);
-    Deref_Prefix(prefix);
-    return node;
-  }
+  patricia_prefix_t *prefix = New_Prefix(family, dest, bitlen);
+  if (prefix == NULL)
+    return NULL;
 
-  return NULL;
+  patricia_node_t *node = patricia_search_exact(tree, prefix);
+  Deref_Prefix(prefix);
+  return node;
 }
 
 patricia_node_t *
-patricia_try_search_best_addr(patricia_tree_t *tree, struct sockaddr *addr, int bitlen)
+patricia_try_search_best_addr(patricia_tree_t *tree, const struct sockaddr *addr, int bitlen)
 {
   int family;
-  void *dest;
+  const void *dest;
 
   if (addr->sa_family == AF_INET6)
   {
     if (bitlen == 0 || bitlen > 128)
       bitlen = 128;
+
     family = AF_INET6;
-    dest = &((struct sockaddr_in6 *)addr)->sin6_addr;
+    dest = &((const struct sockaddr_in6 *)addr)->sin6_addr;
   }
   else
   {
     if (bitlen == 0 || bitlen > 32)
       bitlen = 32;
+
     family = AF_INET;
-    dest = &((struct sockaddr_in *)addr)->sin_addr;
+    dest = &((const struct sockaddr_in *)addr)->sin_addr;
   }
 
-  prefix_t *prefix = New_Prefix(family, dest, bitlen);
-  if (prefix)
-  {
-    patricia_node_t *node = patricia_search_best(tree, prefix);
-    Deref_Prefix(prefix);
-    return node;
-  }
+  patricia_prefix_t *prefix = New_Prefix(family, dest, bitlen);
+  if (prefix == NULL)
+    return NULL;
 
-  return NULL;
+  patricia_node_t *node = patricia_search_best(tree, prefix);
+  Deref_Prefix(prefix);
+  return node;
 }
-/* } */
