@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -43,9 +43,10 @@
  */
 
 #include "stdinc.h"
+#include "io_string.h"
+#include "io_time.h"
 #include "list.h"
 #include "event.h"
-#include "irc_string.h"
 #include "rng_mt.h"
 #include "fdlist.h"
 #include "comm.h"
@@ -53,7 +54,6 @@
 #include "res.h"
 #include "reslib.h"
 #include "memory.h"
-#include "conf.h"
 #include "address.h"
 
 #if (CHAR_BIT != 8)
@@ -79,7 +79,7 @@ struct reslist
 {
   list_node_t node;                           /**< Doubly linked list node. */
   unsigned int id;                           /**< Request ID (from request header). */
-  char type;                                 /**< Current request type. */
+  unsigned int type;                         /**< Current request type. */
   char retries;                              /**< Retry counter */
   unsigned int sends;                        /**< Number of sends (>1 means resent). */
   uintmax_t sentat;                          /**< Timestamp we last sent this request. */
@@ -114,14 +114,13 @@ static struct reslist *
 make_request(dns_callback_fnc callback, void *ctx)
 {
   struct reslist *request = io_calloc(sizeof(*request));
-
-  request->sentat = event_base->time.sec_monotonic;
+  request->sentat = io_time_get(IO_TIME_MONOTONIC_SEC);
   request->retries = 2;
   request->timeout = 4;  /* Start at 4 and exponential inc. */
   request->callback = callback;
   request->callback_ctx = ctx;
-
   list_add(request, &request->node, &request_list);
+
   return request;
 }
 
@@ -140,7 +139,7 @@ static bool
 res_ourserver(const struct io_addr *inp)
 {
   for (unsigned int i = 0; i < reslib_nscount; ++i)
-    if (address_compare(inp, &reslib_nsaddr_list[i], true, true, 0))
+    if (address_match(inp, &reslib_nsaddr_list[i], true, true, 0))
       return true;
 
   return false;
@@ -157,7 +156,7 @@ start_resolver(void)
 
   if (ResolverFileDescriptor == NULL)
   {
-    int fd = comm_socket(reslib_nsaddr_list[0].ss.ss_family, SOCK_DGRAM, 0);
+    int fd = comm_socket(address_get_family(&reslib_nsaddr_list[0]), SOCK_DGRAM, 0);
     if (fd == -1)
       return;
 
@@ -195,7 +194,6 @@ delete_resolver_queries(const void *vptr)
   LIST_FOREACH_SAFE(node, node_next, request_list.head)
   {
     struct reslist *request = node->data;
-
     if (request->callback_ctx == vptr)
       rem_request(request);
   }
@@ -221,7 +219,7 @@ send_res_msg(const unsigned char *msg, int len, unsigned int rcount)
 
   for (unsigned int i = 0; i < max_queries; ++i)
     sendto(ResolverFileDescriptor->fd, msg, len, 0,
-           (struct sockaddr *)&reslib_nsaddr_list[i], reslib_nsaddr_list[i].ss_len);
+           (struct sockaddr *)&reslib_nsaddr_list[i], address_length(&reslib_nsaddr_list[i]));
 }
 
 /*
@@ -277,8 +275,7 @@ query_name(const char *name, int query_class, int type, struct reslist *request)
  * do_query_name - nameserver lookup name
  */
 static void
-do_query_name(dns_callback_fnc callback, void *ctx, const char *name,
-              struct reslist *request, int type)
+do_query_name(dns_callback_fnc callback, void *ctx, const char *name, struct reslist *request, int type)
 {
   char host_name[RFC1035_MAX_DOMAIN_LENGTH + 1];
 
@@ -299,13 +296,11 @@ do_query_name(dns_callback_fnc callback, void *ctx, const char *name,
  * do_query_number - Use this to do reverse IP# lookups.
  */
 static void
-do_query_number(dns_callback_fnc callback, void *ctx,
-                const struct io_addr *addr,
-                struct reslist *request)
+do_query_number(dns_callback_fnc callback, void *ctx, const struct io_addr *addr, struct reslist *request)
 {
   char ipbuf[128] = "";
 
-  if (addr->ss.ss_family == AF_INET)
+  if (address_is_ipv4(addr))
   {
     const struct sockaddr_in *v4 = (const struct sockaddr_in *)addr;
     const uint8_t *cp = (const uint8_t *)&v4->sin_addr.s_addr;
@@ -314,8 +309,9 @@ do_query_number(dns_callback_fnc callback, void *ctx,
              (unsigned int)(cp[3]), (unsigned int)(cp[2]),
              (unsigned int)(cp[1]), (unsigned int)(cp[0]));
   }
-  else if (addr->ss.ss_family == AF_INET6)
+  else
   {
+    assert(address_is_ipv6(addr));
     const struct sockaddr_in6 *v6 = (const struct sockaddr_in6 *)addr;
     const uint8_t *cp = (const uint8_t *)&v6->sin6_addr.s6_addr;
 
@@ -344,7 +340,7 @@ do_query_number(dns_callback_fnc callback, void *ctx,
   {
     request = make_request(callback, ctx);
     request->type = T_PTR;
-    request->addr = *addr;
+    address_copy(&request->addr, addr);
   }
 
   query_name(ipbuf, C_IN, T_PTR, request);
@@ -397,8 +393,6 @@ proc_answer(struct reslist *request, HEADER *header, unsigned char *buf, unsigne
   unsigned char *current = buf + sizeof(HEADER); /* current position in buf */
   unsigned int type = 0;       /* answer type */
   unsigned int rd_length = 0;
-  struct sockaddr_in *v4;      /* conversion */
-  struct sockaddr_in6 *v6;
 
   for (; header->qdcount > 0; --header->qdcount)
   {
@@ -445,36 +439,14 @@ proc_answer(struct reslist *request, HEADER *header, unsigned char *buf, unsigne
     switch (type)
     {
       case T_A:
-        if (request->type != T_A)
-          return false;
-
-        /*
-         * Check for invalid rd_length or too many addresses
-         */
-        if (rd_length != sizeof(struct in_addr))
-          return false;
-
-        request->addr.ss_len = sizeof(struct sockaddr_in);
-        v4 = (struct sockaddr_in *)&request->addr;
-        v4->sin_family = AF_INET;
-        memcpy(&v4->sin_addr, current, sizeof(struct in_addr));
-        return true;
-
       case T_AAAA:
-        if (request->type != T_AAAA)
+        if (request->type != type)
           return false;
 
-        if (rd_length != sizeof(struct in6_addr))
-          return false;
-
-        request->addr.ss_len = sizeof(struct sockaddr_in6);
-        v6 = (struct sockaddr_in6 *)&request->addr;
-        v6->sin6_family = AF_INET6;
-        memcpy(&v6->sin6_addr, current, sizeof(struct in6_addr));
-        return true;
+        return address_from_bytes(&request->addr, type == T_A ? AF_INET : AF_INET6, current, rd_length);
 
       case T_PTR:
-        if (request->type != T_PTR)
+        if (request->type != type)
           return false;
 
         n = reslib_dn_expand(buf, eob, current, hostbuf, sizeof(hostbuf));
@@ -503,12 +475,15 @@ static void
 res_readreply(fde_t *F, void *data)
 {
   unsigned char buf[sizeof(HEADER) + MAXPACKET];
-  ssize_t rc;
   struct io_addr addr;
-  socklen_t len = sizeof(addr);
 
-  while ((rc = recvfrom(F->fd, buf, sizeof(buf), 0, (struct sockaddr *)&addr, &len)) != -1)
+  while (true)
   {
+    socklen_t len = sizeof(addr);
+    ssize_t rc = recvfrom(F->fd, buf, sizeof(buf), 0, (struct sockaddr *)&addr, &len);
+    if (rc == -1)
+      break;
+
     if (rc <= (ssize_t)sizeof(HEADER))
       continue;
 
@@ -574,7 +549,7 @@ res_readreply(fde_t *F, void *data)
       /*
        * Lookup the 'authoritative' name that we were given for the ip#.
        */
-      if (request->addr.ss.ss_family == AF_INET6)
+      if (address_is_ipv6(&request->addr))
         gethost_byname_type(request->callback, request->callback_ctx, request->name, T_AAAA);
       else
         gethost_byname_type(request->callback, request->callback_ctx, request->name, T_A);
@@ -608,7 +583,7 @@ resolver_timeout(void *unused)
     struct reslist *request = node->data;
     uintmax_t timeout = request->sentat + request->timeout;
 
-    if (event_base->time.sec_monotonic >= timeout)
+    if (io_time_get(IO_TIME_MONOTONIC_SEC) >= timeout)
     {
       if (--request->retries <= 0)
       {
@@ -617,7 +592,7 @@ resolver_timeout(void *unused)
       }
       else
       {
-        request->sentat = event_base->time.sec_monotonic;
+        request->sentat = io_time_get(IO_TIME_MONOTONIC_SEC);
         request->timeout += request->timeout;
         resend_query(request);
       }
