@@ -90,9 +90,9 @@
  */
 
 #include "stdinc.h"
+#include "io_string.h"
 #include "res.h"
 #include "reslib.h"
-#include "irc_string.h"
 
 #define MAXLINE 128
 
@@ -120,23 +120,15 @@ static int reslib_mklower(int);
 static void
 reslib_add_nameserver(const char *arg)
 {
-  struct addrinfo hints, *res;
-
   /* Done max number of nameservers? */
   if (reslib_nscount >= RESLIB_MAXNS)
     return;
 
-  memset(&hints, 0, sizeof(hints));
-  hints.ai_family = PF_UNSPEC;
-  hints.ai_socktype = SOCK_DGRAM;
-  hints.ai_flags = AI_PASSIVE | AI_NUMERICHOST;
-
-  if (getaddrinfo(arg, "domain", &hints, &res))
+  if (address_from_string(arg, &reslib_nsaddr_list[reslib_nscount]) == false)
     return;
 
-  memcpy(&reslib_nsaddr_list[reslib_nscount].ss, res->ai_addr, res->ai_addrlen);
-  reslib_nsaddr_list[reslib_nscount++].ss_len = res->ai_addrlen;
-  freeaddrinfo(res);
+  address_set_port(&reslib_nsaddr_list[reslib_nscount], NS_DEFAULTPORT);
+  reslib_nscount++;
 }
 
 /* parse_resvconf()
@@ -241,7 +233,7 @@ reslib_dn_expand(const unsigned char *msg, const unsigned char *eom,
    root domain is returned as ".", not "".  */
 static int
 reslib_ns_name_uncompress(const unsigned char *msg, const unsigned char *eom,
-                       const unsigned char *src, char *dst, size_t dstsiz)
+                          const unsigned char *src, char *dst, size_t dstsiz)
 {
   unsigned char tmp[NS_MAXCDNAME];
   int n = reslib_ns_name_unpack(msg, eom, src, tmp, sizeof tmp);
@@ -257,8 +249,7 @@ reslib_ns_name_uncompress(const unsigned char *msg, const unsigned char *eom,
    Returns -1 if it fails, or consumed octets if it succeeds.  */
 static int
 reslib_ns_name_unpack(const unsigned char *msg, const unsigned char *eom,
-                      const unsigned char *src, unsigned char *dst,
-                      size_t dstsiz)
+                      const unsigned char *src, unsigned char *dst, size_t dstsiz)
 {
   const unsigned char *srcp, *dstlim;
   unsigned char *dstp;
@@ -860,8 +851,7 @@ reslib_ns_name_compress(const char *src, unsigned char *dst, size_t dstsiz,
    pointer to the start of the message.  */
 static int
 reslib_dn_find(const unsigned char *domain, const unsigned char *msg,
-               const unsigned char **dnptrs,
-               const unsigned char **lastdnptr)
+               const unsigned char **dnptrs, const unsigned char **lastdnptr)
 {
   const unsigned char *dn, *cp, *sp;
   const unsigned char **cpp;
