@@ -2,7 +2,7 @@
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
  *  Copyright (c) 2004 Kevin L. Mitchell <klmitch@mit.edu>
- *  Copyright (c) 2006-2024 ircd-hybrid development team
+ *  Copyright (c) 2006-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,106 +26,54 @@
 
 #include "stdinc.h"
 #include "misc.h"
+#include "module.h"
+#include "io_string.h"
+#include "cap.h"
 #include "client.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "user.h"
 #include "send.h"
 #include "parse.h"
-#include "module.h"
-#include "irc_string.h"
 
-
-typedef int (*bqcmp)(const void *, const void *);
-
-static struct capabilities
-{
-  unsigned int cap;
-  bool sticky;  /**< Cap may not be cleared once set */
-  const char *name;
-  size_t name_len;
-} capab_list[] = {
-#define _CAP(cap, flags, name)  \
-    { (cap), (flags), (name), sizeof(name) - 1 }
-  _CAP(CAP_UHNAMES, false, "userhost-in-names"),
-  _CAP(CAP_MULTI_PREFIX, false, "multi-prefix"),
-  _CAP(CAP_AWAY_NOTIFY, false, "away-notify"),
-  _CAP(CAP_EXTENDED_JOIN, false, "extended-join"),
-  _CAP(CAP_ACCOUNT_NOTIFY, false, "account-notify"),
-  _CAP(CAP_CAP_NOTIFY, false, "cap-notify"),
-  _CAP(CAP_INVITE_NOTIFY, false, "invite-notify"),
-  _CAP(CAP_CHGHOST, false, "chghost")
-#undef _CAP
-};
-
-static int
-capab_sort(const struct capabilities *cap1, const struct capabilities *cap2)
-{
-  return strcasecmp(cap1->name, cap2->name);
-}
-
-static int
-capab_search(const char *key, const struct capabilities *cap)
-{
-  const char *rb = cap->name;
-
-  while (ToLower(*key) == ToLower(*rb))  /* Walk equivalent part of strings */
-    if (*key++ == '\0')  /* Hit the end, all right... */
-      return 0;
-    else  /* OK, let's move on... */
-      ++rb;
-
-  /*
-   * If the character they differ on happens to be a space, and it happens
-   * to be the same length as the capability name, then we've found a
-   * match; otherwise, return the difference of the two.
-   */
-  return (IsSpace(*key) && *rb == '\0') ? 0 : (ToLower(*key) - ToLower(*rb));
-}
-
-static struct capabilities *
-find_cap(const char **caplist_p, int *neg_p)
+static const struct Cap *
+find_cap(const char **caplist_p, bool *negate_p)
 {
+  /* Skip leading whitespace. */
   const char *caplist = *caplist_p;
-  struct capabilities *cap = NULL;
-
-  *neg_p = 0;  /* Clear negative flag... */
-
-  /* Next, find first non-whitespace character... */
   while (*caplist && IsSpace(*caplist))
     ++caplist;
 
-  /* We are now at the beginning of an element of the list; is it negative? */
-  if (*caplist == '-')
+  *negate_p = *caplist == '-';  /* Check if the capability is negative. */
+  if (*negate_p)
+    ++caplist;  /* Move past the '-'. */
+
+  const char *caplist_start = caplist;
+  /* Move the pointer to the end of the capability name. */
+  while (*caplist && !IsSpace(*caplist))
+    ++caplist;
+
+  /* If the capability name is empty, return NULL. */
+  if (caplist_start == caplist)
   {
-    ++caplist;  /* Yes; step past the flag... */
-    *neg_p = 1;  /* Remember that it is negative... */
+    *caplist_p = caplist;  /* Update the pointer for the next iteration. */
+    return NULL;
   }
 
-  /* OK, now see if we can look up the capability... */
-  if (*caplist)
-  {
-    if (!(cap = bsearch(caplist, capab_list, IO_ARRAY_LENGTH(capab_list),
-                        sizeof(struct capabilities), (bqcmp)capab_search)))
-    {
-      /* Couldn't find the capability; advance to first whitespace character */
-      while (*caplist && !IsSpace(*caplist))
-        ++caplist;
-    }
-    else
-      caplist += cap->name_len;  /* Advance to end of capability name */
+  const size_t name_len = caplist - caplist_start;
+  char name[name_len + 1];
+  strlcpy(name, caplist_start, sizeof(name));
 
-    /* Strip trailing spaces */
-    while (*caplist && IsSpace(*caplist))
-      ++caplist;
-  }
+  /* Skip trailing whitespace. */
+  while (*caplist && IsSpace(*caplist))
+    ++caplist;
 
-  assert(caplist != *caplist_p || !*caplist);  /* We *must* advance */
+  assert(caplist != *caplist_p || *caplist == '\0');  /* We *must* advance */
 
-  /* Move ahead in capability list string--or zero pointer if we hit end */
+  /* Update the input pointer for the next capability in the list. */
   *caplist_p = *caplist ? caplist : NULL;
 
-  return cap;  /* And return the capability (if any) */
+  return cap_find(name);  /* And return the capability (if any) */
 }
 
 /** Send a CAP \a subcmd list of capability changes to \a source.
@@ -141,17 +89,18 @@ send_caplist(struct Client *source,
              const unsigned int *const set,
              const unsigned int *const rem, const char *subcmd)
 {
-  char capbuf[IRCD_BUFSIZE] = "", pfx[4];
+  char capbuf[IRCD_BUFSIZE] = "";
   char cmdbuf[IRCD_BUFSIZE] = "";
-  unsigned int i, loc, len, pfx_len, clen;
+  unsigned int loc = 0, len, clen;
 
   /* Set up the buffer for the final LS message... */
-  clen = snprintf(cmdbuf, sizeof(cmdbuf), ":%s CAP %s %s ", me.name,
-                  source->name[0] ? source->name : "*", subcmd);
+  clen = snprintf(cmdbuf, sizeof(cmdbuf), ":%s CAP %s %s ",
+                  me.name, source->name[0] ? source->name : "*", subcmd);
 
-  for (i = 0, loc = 0; i < IO_ARRAY_LENGTH(capab_list); ++i)
+  list_node_t *node;
+  LIST_FOREACH(node, cap_get_list()->head)
   {
-    const struct capabilities *cap = &capab_list[i];
+    const struct Cap *cap = node->data;
 
     /*
      * This is a little bit subtle, but just involves applying de
@@ -159,16 +108,17 @@ send_caplist(struct Client *source,
      * capability if (and only if) it is set in \a rem or \a set, or
      * if both are null.
      */
-    if (!(rem && (*rem & cap->cap)) &&
-        !(set && (*set & cap->cap)) && (rem || set))
+    if (!(rem && (*rem & cap->flag)) &&
+        !(set && (*set & cap->flag)) && (rem || set))
       continue;
 
     /* Build the prefix (space separator and any modifiers needed). */
-    pfx_len = 0;
+    char pfx[4];
+    unsigned int pfx_len = 0;
 
     if (loc)
       pfx[pfx_len++] = ' ';
-    if (rem && (*rem & cap->cap))
+    if (rem && (*rem & cap->flag))
       pfx[pfx_len++] = '-';
 
     pfx[pfx_len] = '\0';
@@ -179,11 +129,11 @@ send_caplist(struct Client *source,
     {
       /* Would add too much; must flush */
       sendto_one(source, "%s* :%s", cmdbuf, capbuf);
-      capbuf[(loc = 0)] = '\0';  /* Re-terminate the buffer... */
+      loc = 0;
+      capbuf[0] = '\0';  /* Re-terminate the buffer... */
     }
 
-    loc += snprintf(capbuf + loc, sizeof(capbuf) - loc,
-                    "%s%s", pfx, cap->name);
+    loc += snprintf(capbuf + loc, sizeof(capbuf) - loc, "%s%s", pfx, cap->name);
   }
 
   sendto_one(source, "%s:%s", cmdbuf, capbuf);
@@ -209,7 +159,6 @@ cap_req(struct Client *source, const char *arg)
 {
   unsigned int set = 0, rem = 0;
   unsigned int cs = source->connection->cap;  /* Enabled capabilities */
-  int neg = 0;
 
   if (IsUnknown(source))  /* Registration hasn't completed; suspend it... */
     source->connection->registration |= REG_NEED_CAP;
@@ -218,14 +167,13 @@ cap_req(struct Client *source, const char *arg)
   for (const char *cl = arg; cl; )
   {
     /* Look up capability... */
-    const struct capabilities *cap = find_cap(&cl, &neg);
     bool error = false;
+    bool negate;
+    const struct Cap *cap = find_cap(&cl, &negate);
 
     if (cap == NULL)
       error = true;
-    else if (neg && cap->sticky)
-      error = true;
-    else if (neg && (cap->cap & CAP_CAP_NOTIFY) && HasFlag(source, FLAGS_CAP302))
+    else if (negate && (cap->flag & CAP_CAP_NOTIFY) && HasFlag(source, FLAGS_CAP302))
       error = true;
 
     if (error)
@@ -235,18 +183,18 @@ cap_req(struct Client *source, const char *arg)
       return;  /* Can't complete requested op... */
     }
 
-    if (neg)
+    if (negate)
     {
       /* Set or clear the capability... */
-      rem |=  cap->cap;
-      set &= ~cap->cap;
-      cs  &= ~cap->cap;
+      rem |=  cap->flag;
+      set &= ~cap->flag;
+      cs  &= ~cap->flag;
     }
     else
     {
-      rem &= ~cap->cap;
-      set |=  cap->cap;
-      cs  |=  cap->cap;
+      rem &= ~cap->flag;
+      set |=  cap->flag;
+      cs  |=  cap->flag;
     }
   }
 
@@ -279,7 +227,7 @@ cap_list(struct Client *source, const char *arg)
 
 static struct subcmd
 {
-  const char *cmd;
+  const char *name;
   void (*proc)(struct Client *, const char *);
 } cmdlist[] = {
   { "END",  cap_end  },
@@ -289,9 +237,11 @@ static struct subcmd
 };
 
 static int
-subcmd_search(const char *cmd, const struct subcmd *elem)
+subcmd_cmp(const void *const name_, const void *const elem_)
 {
-  return strcasecmp(cmd, elem->cmd);
+  const char *const name = name_;
+  const struct subcmd *const elem = elem_;
+  return strcasecmp(name, elem->name);
 }
 
 /*! \brief CAP command handler
@@ -310,11 +260,10 @@ static void
 m_cap(struct Client *source, int parc, char *parv[])
 {
   const char *subcmd = parv[1], *caplist = parv[2];
-  struct subcmd *cmd = NULL;
 
   /* Find the subcommand handler */
-  if (!(cmd = bsearch(subcmd, cmdlist, IO_ARRAY_LENGTH(cmdlist),
-                      sizeof(struct subcmd), (bqcmp)subcmd_search)))
+  const struct subcmd *cmd = bsearch(subcmd, cmdlist, IO_ARRAY_LENGTH(cmdlist), sizeof(*cmd), subcmd_cmp);
+  if (cmd == NULL)
   {
     sendto_one_numeric(source, &me, ERR_INVALIDCAPCMD, subcmd);
     return;
@@ -337,8 +286,6 @@ static struct Command command_table =
 static void
 init_handler(void)
 {
-  /* First, let's sort the array */
-  qsort(capab_list, IO_ARRAY_LENGTH(capab_list), sizeof(struct capabilities), (bqcmp)capab_sort);
   command_add(&command_table);
 }
 
