@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,13 +25,12 @@
 
 #include "stdinc.h"
 #include "list.h"
+#include "module.h"
 #include "client.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "send.h"
 #include "parse.h"
-#include "module.h"
-
 
 /*! \brief CLOSE command handler
  *
@@ -64,7 +63,7 @@ mo_close(struct Client *source, int parc, char *parv[])
      * Exit here is safe, because it is guaranteed not to be source
      * because it is unregistered and source is an oper.
      */
-    exit_client(target, "Oper Closing");
+    client_exit(target, "Oper Closing");
   }
 
   sendto_one_numeric(source, &me, RPL_CLOSEEND, closed);
