@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,9 +24,10 @@
  */
 
 #include "stdinc.h"
+#include "io_time.h"
 #include "list.h"
 #include "client.h"
-#include "irc_string.h"
+#include "io_string.h"
 #include "conf.h"
 #include "conf_cluster.h"
 #include "conf_shared.h"
@@ -63,7 +64,7 @@ dline_check(const struct AddressRec *arec)
       {
         case HM_IPV6:
         case HM_IPV4:
-          if (address_compare(&client->addr, &arec->Mask.ipa.addr, false, false, arec->Mask.ipa.bits))
+          if (address_match(&client->addr, &arec->Mask.ipa.addr, false, false, arec->Mask.ipa.bits))
             conf_try_ban(client, CLIENT_BAN_DLINE, arec->conf->reason);
           break;
         default:
@@ -87,7 +88,7 @@ dline_handle(struct Client *source, const struct aline_ctx *aline)
   int bits = 0;
   unsigned int min_cidr = 0;
 
-  switch (parse_netmask(aline->host, &addr, &bits))
+  switch (address_parse_netmask(aline->host, &addr, &bits))
   {
     case HM_IPV4:
       min_cidr = ConfigGeneral.dline_min_cidr;
@@ -126,20 +127,20 @@ dline_handle(struct Client *source, const struct aline_ctx *aline)
   conf = conf_make(CONF_DLINE);
   conf->host = io_strdup(aline->host);
   conf->reason = io_strdup(buf);
-  conf->setat = event_base->time.sec_real;
+  conf->setat = io_time_get(IO_TIME_REALTIME_SEC);
   SetConfDatabase(conf);
 
   if (aline->duration)
   {
-    conf->until = event_base->time.sec_real + aline->duration;
+    conf->until = conf->setat + aline->duration;
 
     if (IsClient(source))
       sendto_one_notice(source, &me, ":Added temporary %ju min. D-Line [%s]",
                         aline->duration / 60, conf->host);
 
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                         "%s added temporary %ju min. D-Line for [%s] [%s]",
-                         get_oper_name(source), aline->duration / 60, conf->host, conf->reason);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                   "%s added temporary %ju min. D-Line for [%s] [%s]",
+                   get_oper_name(source), aline->duration / 60, conf->host, conf->reason);
     log_write(LOG_TYPE_DLINE, "%s added temporary %ju min. D-Line for [%s] [%s]",
               get_oper_name(source), aline->duration / 60, conf->host, conf->reason);
   }
@@ -148,8 +149,8 @@ dline_handle(struct Client *source, const struct aline_ctx *aline)
     if (IsClient(source))
       sendto_one_notice(source, &me, ":Added D-Line [%s]", conf->host);
 
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE, "%s added D-Line for [%s] [%s]",
-                         get_oper_name(source), conf->host, conf->reason);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "%s added D-Line for [%s] [%s]",
+                   get_oper_name(source), conf->host, conf->reason);
     log_write(LOG_TYPE_DLINE, "%s added D-Line for [%s] [%s]",
               get_oper_name(source), conf->host, conf->reason);
   }
