@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,14 +24,16 @@
  */
 
 #include "stdinc.h"
+#include "io_time.h"
 #include "list.h"
+#include "cap.h"
 #include "channel.h"
 #include "channel_invite.h"
 #include "channel_mode.h"
 #include "client.h"
 #include "conf.h"
 #include "hash.h"
-#include "irc_string.h"
+#include "io_string.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "send.h"
@@ -39,6 +41,31 @@
 #include "module.h"
 
 
+static void
+send_invite(struct Client *source, struct Client *target, struct Channel *channel)
+{
+  channel->last_invite_time = io_time_get(IO_TIME_MONOTONIC_SEC);
+
+  if (MyConnect(target))
+  {
+    sendto_one(target, ":%s!%s@%s INVITE %s :%s",
+               source->name, source->username, source->host, target->name, channel->name);
+
+    if (channel_has_mode(channel, MODE_INVITEONLY))
+      invite_add(channel, target);  /* Add the invite if channel is +i */
+  }
+
+  if (channel_has_mode(channel, MODE_INVITEONLY))
+    sendto_channel_local(NULL, channel, CHACCESS_HALFOP, 0, CAP_INVITE_NOTIFY,
+                         ":%s NOTICE %%%s :%s is inviting %s to %s.",
+                         me.name, channel->name, source->name, target->name, channel->name);
+
+  sendto_channel_local(NULL, channel, CHACCESS_HALFOP, CAP_INVITE_NOTIFY, 0, ":%s!%s@%s INVITE %s %s",
+                       source->name, source->username, source->host, target->name, channel->name);
+  sendto_servers(source, 0, 0, ":%s INVITE %s %s %ju",
+                 source->id, target->id, channel->name, channel->creation_time);
+}
+
 /*! \brief INVITE command handler
  *
  * \param source Pointer to allocated Client struct from which the message
@@ -67,7 +94,7 @@ m_invite(struct Client *source, int parc, char *parv[])
     return;
   }
 
-  if (EmptyString(parv[2]))
+  if (string_is_empty(parv[2]))
   {
     sendto_one_numeric(source, &me, ERR_NEEDMOREPARAMS, "INVITE");
     return;
@@ -106,13 +133,13 @@ m_invite(struct Client *source, int parc, char *parv[])
     return;
   }
 
-  if (HasCMode(channel, MODE_NOINVITE))
+  if (channel_has_mode(channel, MODE_NOINVITE))
   {
     sendto_one_numeric(source, &me, ERR_NOINVITE, target->name, channel->name);
     return;
   }
 
-  if ((source->connection->invite.last_attempt + ConfigChannel.invite_client_time) < event_base->time.sec_monotonic)
+  if ((source->connection->invite.last_attempt + ConfigChannel.invite_client_time) < io_time_get(IO_TIME_MONOTONIC_SEC))
     source->connection->invite.count = 0;
 
   if (source->connection->invite.count > ConfigChannel.invite_client_count)
@@ -121,40 +148,21 @@ m_invite(struct Client *source, int parc, char *parv[])
     return;
   }
 
-  if ((channel->last_invite_time + ConfigChannel.invite_delay_channel) > event_base->time.sec_monotonic)
+  if ((channel->last_invite_time + ConfigChannel.invite_delay_channel) > io_time_get(IO_TIME_MONOTONIC_SEC))
   {
     sendto_one_numeric(source, &me, ERR_TOOMANYINVITE, channel->name, "channel");
     return;
   }
 
-  source->connection->invite.last_attempt = event_base->time.sec_monotonic;
+  source->connection->invite.last_attempt = io_time_get(IO_TIME_MONOTONIC_SEC);
   source->connection->invite.count++;
-  channel->last_invite_time = event_base->time.sec_monotonic;
 
   sendto_one_numeric(source, &me, RPL_INVITING, target->name, channel->name);
 
-  if (target->away[0])
+  if (target->away)
     sendto_one_numeric(source, &me, RPL_AWAY, target->name, target->away);
 
-  if (MyConnect(target))
-  {
-    sendto_one(target, ":%s!%s@%s INVITE %s :%s",
-               source->name, source->username, source->host, target->name, channel->name);
-
-    if (HasCMode(channel, MODE_INVITEONLY))
-      invite_add(channel, target);  /* Add the invite if channel is +i */
-  }
-
-  if (HasCMode(channel, MODE_INVITEONLY))
-    sendto_channel_local(NULL, channel, CHACCESS_HALFOP, 0, CAP_INVITE_NOTIFY,
-                         ":%s NOTICE %%%s :%s is inviting %s to %s.",
-                         me.name, channel->name, source->name, target->name, channel->name);
-
-  sendto_channel_local(NULL, channel, CHACCESS_HALFOP, CAP_INVITE_NOTIFY, 0,
-                       ":%s!%s@%s INVITE %s %s", source->name, source->username,
-                       source->host, target->name, channel->name);
-  sendto_server(source, 0, 0, ":%s INVITE %s %s %ju",
-                source->id, target->id, channel->name, channel->creation_time);
+  send_invite(source, target, channel);
 }
 
 /*! \brief INVITE command handler
@@ -187,30 +195,9 @@ ms_invite(struct Client *source, int parc, char *parv[])
   if (strtoumax(parv[3], NULL, 10) > channel->creation_time)
     return;
 
-  channel->last_invite_time = event_base->time.sec_monotonic;
-
-  if (MyConnect(target))
-  {
-    sendto_one(target, ":%s!%s@%s INVITE %s :%s",
-               source->name, source->username, source->host, target->name, channel->name);
-
-    if (HasCMode(channel, MODE_INVITEONLY))
-      invite_add(channel, target);  /* Add the invite if channel is +i */
-  }
-
-  if (HasCMode(channel, MODE_INVITEONLY))
-    sendto_channel_local(NULL, channel, CHACCESS_HALFOP, 0, CAP_INVITE_NOTIFY,
-                         ":%s NOTICE %%%s :%s is inviting %s to %s.",
-                         me.name, channel->name, source->name, target->name, channel->name);
-
-  sendto_channel_local(NULL, channel, CHACCESS_HALFOP, CAP_INVITE_NOTIFY, 0,
-                       ":%s!%s@%s INVITE %s %s", source->name, source->username,
-                       source->host, target->name, channel->name);
-  sendto_server(source, 0, 0, ":%s INVITE %s %s %ju",
-                source->id, target->id, channel->name, channel->creation_time);
+  send_invite(source, target, channel);
 }
 
-
 static struct Command command_table =
 {
   .name = "INVITE",
