@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,9 +24,10 @@
  */
 
 #include "stdinc.h"
+#include "io_time.h"
 #include "list.h"
 #include "client.h"
-#include "irc_string.h"
+#include "io_string.h"
 #include "ircd.h"
 #include "conf.h"
 #include "conf_cluster.h"
@@ -67,7 +68,7 @@ kline_check(const struct AddressRec *arec)
         break;
       case HM_IPV6:
       case HM_IPV4:
-        if (address_compare(&client->addr, &arec->Mask.ipa.addr, false, false, arec->Mask.ipa.bits))
+        if (address_match(&client->addr, &arec->Mask.ipa.addr, false, false, arec->Mask.ipa.bits))
           conf_try_ban(client, CLIENT_BAN_KLINE, arec->conf->reason);
         break;
       default:
@@ -99,7 +100,7 @@ kline_handle(struct Client *source, const struct aline_ctx *aline)
     return;
   }
 
-  switch (parse_netmask(aline->host, &iphost, &bits))
+  switch (address_parse_netmask(aline->host, &iphost, &bits))
   {
     case HM_IPV4:
       min_cidr = ConfigGeneral.kline_min_cidr;
@@ -138,21 +139,21 @@ kline_handle(struct Client *source, const struct aline_ctx *aline)
   conf = conf_make(CONF_KLINE);
   conf->user = io_strdup(aline->user);
   conf->host = io_strdup(aline->host);
-  conf->setat = event_base->time.sec_real;
+  conf->setat = io_time_get(IO_TIME_REALTIME_SEC);
   conf->reason = io_strdup(buf);
   SetConfDatabase(conf);
 
   if (aline->duration)
   {
-    conf->until = event_base->time.sec_real + aline->duration;
+    conf->until = conf->setat + aline->duration;
 
     if (IsClient(source))
       sendto_one_notice(source, &me, ":Added temporary %ju min. K-Line [%s@%s]",
                         aline->duration / 60, conf->user, conf->host);
 
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                         "%s added temporary %ju min. K-Line for [%s@%s] [%s]",
-                         get_oper_name(source), aline->duration / 60, conf->user, conf->host, conf->reason);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                   "%s added temporary %ju min. K-Line for [%s@%s] [%s]",
+                   get_oper_name(source), aline->duration / 60, conf->user, conf->host, conf->reason);
     log_write(LOG_TYPE_KLINE, "%s added temporary %ju min. K-Line for [%s@%s] [%s]",
               get_oper_name(source), aline->duration / 60, conf->user, conf->host, conf->reason);
   }
@@ -162,8 +163,8 @@ kline_handle(struct Client *source, const struct aline_ctx *aline)
       sendto_one_notice(source, &me, ":Added K-Line [%s@%s]",
                         conf->user, conf->host);
 
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE, "%s added K-Line for [%s@%s] [%s]",
-                         get_oper_name(source), conf->user, conf->host, conf->reason);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "%s added K-Line for [%s@%s] [%s]",
+                   get_oper_name(source), conf->user, conf->host, conf->reason);
     log_write(LOG_TYPE_KLINE, "%s added K-Line for [%s@%s] [%s]",
               get_oper_name(source), conf->user, conf->host, conf->reason);
   }
