@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 #include "stdinc.h"
 #include "client.h"
 #include "ircd.h"
-#include "irc_string.h"
+#include "io_string.h"
 #include "numeric.h"
 #include "send.h"
 #include "conf_cluster.h"
@@ -58,9 +58,9 @@ mo_locops(struct Client *source, int parc, char *parv[])
     return;
   }
 
-  sendto_realops_flags(UMODE_LOCOPS, L_ALL, SEND_LOCOPS, "from %s: %s",
-                       source->name, message);
-  cluster_distribute(source, "LOCOPS", 0, CLUSTER_LOCOPS, message);
+  sendto_clients(UMODE_LOCOPS, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_LOCOPS, "from %s: %s",
+                 source->name, message);
+  cluster_distribute(source, "LOCOPS", 0, CLUSTER_LOCOPS, "%s", message);
 }
 
 /*! \brief LOCOPS command handler
@@ -88,8 +88,8 @@ ms_locops(struct Client *source, int parc, char *parv[])
     return;
 
   if (shared_find(SHARED_LOCOPS, source->servptr->name, "*", "*"))
-    sendto_realops_flags(UMODE_LOCOPS, L_ALL, SEND_LOCOPS, "from %s: %s",
-                         source->name, message);
+    sendto_clients(UMODE_LOCOPS, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_LOCOPS, "from %s: %s",
+                   source->name, message);
 }
 
 static struct Command command_table =
