@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,15 +24,17 @@
  */
 
 #include "stdinc.h"
+#include "io_time.h"
 #include "list.h"
 #include "hash.h"
 #include "fdlist.h"
-#include "irc_string.h"
+#include "io_string.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "conf.h"
 #include "conf_resv.h"
 #include "user.h"
+#include "user_mode.h"
 #include "whowas.h"
 #include "send.h"
 #include "channel.h"
@@ -44,6 +46,7 @@
 #include "id.h"
 #include "ipcache.h"
 #include "extban.h"
+#include "ircd_hook.h"
 
 
 /* check_clean_nick()
@@ -71,19 +74,18 @@ check_clean_nick(struct Client *source, const char *nick)
     return true;
 
   ++ServerStats.is_kill;
-  sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE, "Bad/long Nick: %s From: %s(via %s)",
-                       nick, IsServer(source) ? source->name : source->servptr->name,
-                       source->from->name);
+  sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "Bad/long Nick: %s From: %s(via %s)",
+                 nick, IsServer(source) ? source->name : source->servptr->name, source->from->name);
   sendto_one(source, ":%s KILL %s :%s (Bad Nickname)",
              me.id, nick, me.name);
 
   /* Bad nick change */
   if (!IsServer(source))
   {
-    sendto_server(source, 0, 0, ":%s KILL %s :%s (Bad Nickname)",
-                  me.id, source->id, me.name);
+    sendto_servers(source, 0, 0, ":%s KILL %s :%s (Bad Nickname)",
+                   me.id, source->id, me.name);
     AddFlag(source, FLAGS_KILLED);
-    exit_client(source, "Bad Nickname");
+    client_exit(source, "Bad Nickname");
   }
 
   return false;
@@ -98,9 +100,9 @@ check_clean_uid(struct Client *source, const char *nick, const char *uid)
     return true;
 
   ++ServerStats.is_kill;
-  sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                       "Bad UID: %s Nickname: %s From: %s(via %s)",
-                       uid, nick, source->name, source->from->name);
+  sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                 "Bad UID: %s Nickname: %s From: %s(via %s)",
+                 uid, nick, source->name, source->from->name);
   sendto_one(source, ":%s KILL %s :%s (Bad UID)",
              me.id, uid, me.name);
   return false;
@@ -124,9 +126,9 @@ check_clean_user(struct Client *source, const char *nick, const char *user)
     return true;
 
   ++ServerStats.is_kill;
-  sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                       "Bad/Long Username: %s Nickname: %s From: %s(via %s)",
-                       user, nick, source->name, source->from->name);
+  sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                 "Bad/Long Username: %s Nickname: %s From: %s(via %s)",
+                 user, nick, source->name, source->from->name);
   sendto_one(source, ":%s KILL %s :%s (Bad Username)",
              me.id, nick, me.name);
   return false;
@@ -150,9 +152,9 @@ check_clean_host(struct Client *source, const char *nick, const char *host)
     return true;
 
   ++ServerStats.is_kill;
-  sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                       "Bad/Long Hostname: %s Nickname: %s From: %s(via %s)",
-                       host, nick, source->name, source->from->name);
+  sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                 "Bad/Long Hostname: %s Nickname: %s From: %s(via %s)",
+                 host, nick, source->name, source->from->name);
   sendto_one(source, ":%s KILL %s :%s (Bad Hostname)",
              me.id, nick, me.name);
   return false;
@@ -172,7 +174,7 @@ set_initial_nick(struct Client *source, const char *nick)
 {
   bool samenick = irccmp(source->name, nick) == 0;
   if (samenick == false)
-    source->tsinfo = event_base->time.sec_real;
+    source->tsinfo = io_time_get(IO_TIME_REALTIME_SEC);
 
   source->connection->registration &= ~REG_NEED_NICK;
 
@@ -198,15 +200,15 @@ set_initial_nick(struct Client *source, const char *nick)
  * side effects - changes nick of a LOCAL user
  */
 static void
-change_local_nick(struct Client *source, const char *nick)
+nick_change_local(struct Client *source, const char *nick)
 {
-  assert(source->name[0] && !EmptyString(nick));
+  assert(source->name[0] && !string_is_empty(nick));
   assert(MyClient(source));
 
-  if ((source->connection->nick.last_attempt + ConfigGeneral.max_nick_time) < event_base->time.sec_monotonic)
+  if ((source->connection->nick.last_attempt + ConfigGeneral.max_nick_time) < io_time_get(IO_TIME_MONOTONIC_SEC))
     source->connection->nick.count = 0;
 
-  if (ConfigGeneral.anti_nick_flood && !HasUMode(source, UMODE_OPER) &&
+  if (ConfigGeneral.anti_nick_flood && user_mode_has_flag(source, UMODE_OPER) == false &&
       (source->connection->nick.count > ConfigGeneral.max_nick_changes))
   {
     sendto_one_numeric(source, &me, ERR_NICKTOOFAST,
@@ -214,39 +216,39 @@ change_local_nick(struct Client *source, const char *nick)
     return;
   }
 
-  source->connection->nick.last_attempt = event_base->time.sec_monotonic;
+  source->connection->nick.last_attempt = io_time_get(IO_TIME_MONOTONIC_SEC);
   source->connection->nick.count++;
 
   bool samenick = irccmp(source->name, nick) == 0;
   if (samenick == false)
   {
-    source->tsinfo = event_base->time.sec_real;
+    source->tsinfo = io_time_get(IO_TIME_REALTIME_SEC);
     clear_ban_cache_list(&source->channel);
     monitor_signoff(source);
 
-    if (HasUMode(source, UMODE_REGISTERED))
+    if (user_mode_has_flag(source, UMODE_REGISTERED))
     {
-      const unsigned int oldmodes = source->umodes;
-      DelUMode(source, UMODE_REGISTERED);
+      const uint64_t mode_flags_old = source->umodes;
+      user_mode_unset_flag(source, UMODE_REGISTERED);
 
-      send_umode(source, oldmodes, true, false);
+      user_mode_send(source, mode_flags_old, USER_MODE_SEND_CLIENT);
     }
   }
 
+  ircd_hook_nick_change_ctx ctx = { .client = source, .nick = nick };
+  hook_dispatch(ircd_hook_nick_change_local, &ctx);
+
   /*
    * Client just changing their nick. If they are on a channel, send
    * note of change to all clients on that channel. Propagate notice
    * to other servers.
    */
-  sendto_realops_flags(UMODE_NCHANGE, L_ALL, SEND_NOTICE,
-                       "Nick change: From %s to %s [%s@%s]",
-                       source->name, nick, source->username, source->host);
   sendto_common_channels_local(source, true, 0, 0, ":%s!%s@%s NICK :%s",
                                source->name, source->username, source->host, nick);
   whowas_add_history(source, true);
 
-  sendto_server(source, 0, 0, ":%s NICK %s :%ju",
-                source->id, nick, source->tsinfo);
+  sendto_servers(source, 0, 0, ":%s NICK %s :%ju",
+                 source->id, nick, source->tsinfo);
 
   hash_del_client(source);
   strlcpy(source->name, nick, sizeof(source->name));
@@ -272,9 +274,9 @@ change_local_nick(struct Client *source, const char *nick)
  *      - parv[2] = timestamp
  */
 static void
-change_remote_nick(struct Client *source, char *parv[])
+nick_change_remote(struct Client *source, char *parv[])
 {
-  assert(!EmptyString(parv[1]));
+  assert(!string_is_empty(parv[1]));
   assert(IsClient(source));
   assert(source->name[0]);
 
@@ -282,22 +284,22 @@ change_remote_nick(struct Client *source, char *parv[])
   bool samenick = irccmp(source->name, parv[1]) == 0;
   if (samenick == false)
   {
-    DelUMode(source, UMODE_REGISTERED);
+    user_mode_unset_flag(source, UMODE_REGISTERED);
     monitor_signoff(source);
 
     source->tsinfo = strtoumax(parv[2], NULL, 10);
     assert(source->tsinfo);
   }
 
-  sendto_realops_flags(UMODE_NCHANGE, L_ALL, SEND_NOTICE,
-                       "Nick change: From %s to %s [%s@%s]",
-                       source->name, parv[1], source->username, source->host);
+  ircd_hook_nick_change_ctx ctx = { .client = source, .nick = parv[1] };
+  hook_dispatch(ircd_hook_nick_change_remote, &ctx);
+
   sendto_common_channels_local(source, true, 0, 0, ":%s!%s@%s NICK :%s",
                                source->name, source->username, source->host, parv[1]);
 
   whowas_add_history(source, true);
-  sendto_server(source, 0, 0, ":%s NICK %s :%ju",
-                source->id, parv[1], source->tsinfo);
+  sendto_servers(source, 0, 0, ":%s NICK %s :%ju",
+                 source->id, parv[1], source->tsinfo);
 
   /* Set the new nick name */
   hash_del_client(source);
@@ -346,40 +348,21 @@ uid_from_server(struct Client *source, int parc, char *parv[])
   strlcpy(client_p->account, parv[10], sizeof(client_p->account));
   strlcpy(client_p->info, parv[11], sizeof(client_p->info));
 
-  struct addrinfo hints, *res = NULL;
-  memset(&hints, 0, sizeof(hints));
-  hints.ai_family = AF_UNSPEC;
-  hints.ai_flags = AI_PASSIVE | AI_NUMERICHOST;
-
-  if (getaddrinfo(client_p->sockhost, NULL, &hints, &res) == 0)
+  if (address_from_string(client_p->sockhost, &client_p->addr))
   {
-    memcpy(&client_p->addr, res->ai_addr, res->ai_addrlen);
-    client_p->addr.ss_len = res->ai_addrlen;
-
     struct ip_entry *ipcache = ipcache_record_find_or_add(&client_p->addr);
     ++ipcache->count_remote;
     AddFlag(client_p, FLAGS_IPHASH);
   }
 
-  if (res)
-    freeaddrinfo(res);
-
   hash_add_client(client_p);
   hash_add_id(client_p);
 
   /* Parse user modes */
   for (const char *m = &parv[4][1]; *m; ++m)
   {
-    const struct user_modes *tab = umode_map[(unsigned char)*m];
-    if (tab == NULL)
-      continue;
-
-    if ((tab->flag & UMODE_INVISIBLE) && !HasUMode(client_p, UMODE_INVISIBLE))
-      ++Count.invisi;
-    else if ((tab->flag & UMODE_OPER) && !HasUMode(client_p, UMODE_OPER))
-      ++Count.oper;
-
-    AddUMode(client_p, tab->flag);
+    const struct UserMode *mode = user_mode_find(*m);
+    user_mode_set_mode_exec(client_p, mode, USER_MODE_SOURCE_REGULAR);
   }
 
   user_register_remote(client_p);
@@ -422,20 +405,20 @@ perform_uid_introduction_collides(struct Client *source, struct Client *target,
   /* If their TS's are the same, kill both */
   if (newts == target->tsinfo)
   {
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                         "Nick collision on %s(%s <- %s)(both killed)",
-                         target->name, target->from->name, source->from->name);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                   "Nick collision on %s(%s <- %s)(both killed)",
+                   target->name, target->from->name, source->from->name);
 
     sendto_one(source, ":%s KILL %s :%s (Nick collision (new))",
                me.id, uid, me.name);
-    sendto_server(NULL, 0, 0, ":%s KILL %s :%s (Nick collision (new))",
-                  me.id, target->id, me.name);
+    sendto_servers(NULL, 0, 0, ":%s KILL %s :%s (Nick collision (new))",
+                   me.id, target->id, me.name);
 
     ++ServerStats.is_kill;
     sendto_one_numeric(target, &me, ERR_NICKCOLLISION, target->name);
 
     AddFlag(target, FLAGS_KILLED);
-    exit_client(target, "Nick collision (new)");
+    client_exit(target, "Nick collision (new)");
     return false;
   }
 
@@ -456,22 +439,22 @@ perform_uid_introduction_collides(struct Client *source, struct Client *target,
   }
 
   if (sameuser)
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                         "Nick collision on %s(%s <- %s)(older killed)",
-                         target->name, target->from->name, source->from->name);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                   "Nick collision on %s(%s <- %s)(older killed)",
+                   target->name, target->from->name, source->from->name);
   else
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                         "Nick collision on %s(%s <- %s)(newer killed)",
-                         target->name, target->from->name, source->from->name);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                   "Nick collision on %s(%s <- %s)(newer killed)",
+                   target->name, target->from->name, source->from->name);
 
   ++ServerStats.is_kill;
   sendto_one_numeric(target, &me, ERR_NICKCOLLISION, target->name);
 
-  sendto_server(NULL, 0, 0, ":%s KILL %s :%s (Nick collision (new))",
-                me.id, target->id, me.name);
+  sendto_servers(NULL, 0, 0, ":%s KILL %s :%s (Nick collision (new))",
+                 me.id, target->id, me.name);
 
   AddFlag(target, FLAGS_KILLED);
-  exit_client(target, "Nick collision");
+  client_exit(target, "Nick collision");
 
   return true;
 }
@@ -502,23 +485,23 @@ perform_nick_change_collides(struct Client *source, struct Client *target,
   /* It's a client changing nick and causing a collide */
   if (newts == target->tsinfo)
   {
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-               "Nick change collision from %s to %s(%s <- %s)(both killed)",
-               source->name, target->name, target->from->name,
-               source->from->name);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                   "Nick change collision from %s to %s(%s <- %s)(both killed)",
+                   source->name, target->name, target->from->name,
+                   source->from->name);
 
     sendto_one_numeric(target, &me, ERR_NICKCOLLISION, target->name);
     ServerStats.is_kill += 2;
 
-    sendto_server(NULL, 0, 0, ":%s KILL %s :%s (Nick change collision)",
-                  me.id, source->id, me.name);
-    sendto_server(NULL, 0, 0, ":%s KILL %s :%s (Nick change collision)",
-                  me.id, target->id, me.name);
+    sendto_servers(NULL, 0, 0, ":%s KILL %s :%s (Nick change collision)",
+                   me.id, source->id, me.name);
+    sendto_servers(NULL, 0, 0, ":%s KILL %s :%s (Nick change collision)",
+                   me.id, target->id, me.name);
 
     AddFlag(source, FLAGS_KILLED);
     AddFlag(target, FLAGS_KILLED);
-    exit_client(source, "Nick collision (old)");
-    exit_client(target, "Nick collision (new)");
+    client_exit(source, "Nick collision (old)");
+    client_exit(target, "Nick collision (new)");
     return false;
   }
 
@@ -529,46 +512,46 @@ perform_nick_change_collides(struct Client *source, struct Client *target,
   if ((sameuser && newts < target->tsinfo) || (sameuser == false && newts > target->tsinfo))
   {
     if (sameuser)
-      sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                           "Nick change collision from %s to %s(%s <- %s)(older killed)",
-                           source->name, target->name, target->from->name,
-                           source->from->name);
+      sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                     "Nick change collision from %s to %s(%s <- %s)(older killed)",
+                     source->name, target->name, target->from->name,
+                     source->from->name);
     else
-      sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                           "Nick change collision from %s to %s(%s <- %s)(newer killed)",
-                           source->name, target->name, target->from->name,
-                           source->from->name);
+      sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                     "Nick change collision from %s to %s(%s <- %s)(newer killed)",
+                     source->name, target->name, target->from->name,
+                     source->from->name);
 
     ++ServerStats.is_kill;
 
-    sendto_server(NULL, 0, 0, ":%s KILL %s :%s (Nick change collision)",
-                  me.id, source->id, me.name);
+    sendto_servers(NULL, 0, 0, ":%s KILL %s :%s (Nick change collision)",
+                   me.id, source->id, me.name);
     AddFlag(source, FLAGS_KILLED);
 
     if (sameuser)
-      exit_client(source, "Nick collision (old)");
+      client_exit(source, "Nick collision (old)");
     else
-      exit_client(source, "Nick collision (new)");
+      client_exit(source, "Nick collision (new)");
     return false;
   }
 
   if (sameuser)
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                         "Nick collision on %s(%s <- %s)(older killed)",
-                         target->name, target->from->name, source->from->name);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                   "Nick collision on %s(%s <- %s)(older killed)",
+                   target->name, target->from->name, source->from->name);
   else
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                         "Nick collision on %s(%s <- %s)(newer killed)",
-                         target->name, target->from->name, source->from->name);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                   "Nick collision on %s(%s <- %s)(newer killed)",
+                   target->name, target->from->name, source->from->name);
 
-  sendto_server(NULL, 0, 0, ":%s KILL %s :%s (Nick collision)",
-                me.id, target->id, me.name);
+  sendto_servers(NULL, 0, 0, ":%s KILL %s :%s (Nick collision)",
+                 me.id, target->id, me.name);
 
   ++ServerStats.is_kill;
   sendto_one_numeric(target, &me, ERR_NICKCOLLISION, target->name);
 
   AddFlag(target, FLAGS_KILLED);
-  exit_client(target, "Nick collision");
+  client_exit(target, "Nick collision");
 
   return true;
 }
@@ -587,7 +570,7 @@ perform_nick_change_collides(struct Client *source, struct Client *target,
 static void
 mr_nick(struct Client *source, int parc, char *parv[])
 {
-  if (EmptyString(parv[1]))
+  if (string_is_empty(parv[1]))
   {
     sendto_one_numeric(source, &me, ERR_NONICKNAMEGIVEN);
     return;
@@ -609,9 +592,8 @@ mr_nick(struct Client *source, int parc, char *parv[])
   if (resv)
   {
     sendto_one_numeric(source, &me, ERR_ERRONEUSNICKNAME, nick, resv->reason);
-    sendto_realops_flags(UMODE_REJ, L_ALL, SEND_NOTICE,
-                         "Forbidding reserved nick %s from user %s",
-                         nick, client_get_name(source, HIDE_IP));
+    sendto_clients(UMODE_REJ, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "Forbidding reserved nick %s from user %s",
+                   nick, client_get_name(source, HIDE_IP));
     return;
   }
 
@@ -640,7 +622,7 @@ m_nick(struct Client *source, int parc, char *parv[])
 
   assert(MyClient(source));
 
-  if (EmptyString(parv[1]))
+  if (string_is_empty(parv[1]))
   {
     sendto_one_numeric(source, &me, ERR_NONICKNAMEGIVEN);
     return;
@@ -658,13 +640,12 @@ m_nick(struct Client *source, int parc, char *parv[])
   }
 
   if (!HasFlag(source, FLAGS_EXEMPTRESV) &&
-      !(HasUMode(source, UMODE_OPER) && HasOFlag(source, OPER_FLAG_NICK_RESV)) &&
+      !(user_mode_has_flag(source, UMODE_OPER) && HasOFlag(source, OPER_FLAG_NICK_RESV)) &&
       (resv = resv_find(nick, match)))
   {
     sendto_one_numeric(source, &me, ERR_ERRONEUSNICKNAME, nick, resv->reason);
-    sendto_realops_flags(UMODE_REJ, L_ALL, SEND_NOTICE,
-                         "Forbidding reserved nick %s from user %s",
-                         nick, client_get_name(source, HIDE_IP));
+    sendto_clients(UMODE_REJ, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "Forbidding reserved nick %s from user %s",
+                   nick, client_get_name(source, HIDE_IP));
     return;
   }
 
@@ -675,7 +656,7 @@ m_nick(struct Client *source, int parc, char *parv[])
 
     if (member_highest_rank(member) < CHACCESS_VOICE)
     {
-      if (HasCMode(member->channel, MODE_NONICKCHANGE))
+      if (channel_has_mode(member->channel, MODE_NONICKCHANGE))
       {
         sendto_one_numeric(source, &me, ERR_NONICKCHANGE, member->channel->name);
         return;
@@ -691,7 +672,7 @@ m_nick(struct Client *source, int parc, char *parv[])
 
   struct Client *target = hash_find_client(nick);
   if (target == NULL)
-    change_local_nick(source, nick);
+    nick_change_local(source, nick);
   else if (target == source)
   {
     /*
@@ -701,7 +682,7 @@ m_nick(struct Client *source, int parc, char *parv[])
 
     /* Check the nick isn't exactly the same */
     if (strcmp(target->name, nick))
-      change_local_nick(source, nick);
+      nick_change_local(source, nick);
   }
   else if (IsUnknown(target))
   {
@@ -709,8 +690,8 @@ m_nick(struct Client *source, int parc, char *parv[])
      * If the client that has the nick isn't registered yet (NICK but no
      * USER) then drop the unregistered client
      */
-    exit_client(target, "Overridden by other sign on");
-    change_local_nick(source, nick);
+    client_exit(target, "Overridden by other sign on");
+    nick_change_local(source, nick);
   }
   else
     sendto_one_numeric(source, &me, ERR_NICKNAMEINUSE, target->name);
@@ -742,20 +723,20 @@ ms_nick(struct Client *source, int parc, char *parv[])
   /* If the nick doesn't exist, allow it and process like normal */
   struct Client *target = hash_find_client(parv[1]);
   if (target == NULL)
-    change_remote_nick(source, parv);
+    nick_change_remote(source, parv);
   else if (IsUnknown(target))
   {
     /* We're not living in the past anymore, an unknown client is local only. */
-    exit_client(target, "Overridden by other sign on");
-    change_remote_nick(source, parv);
+    client_exit(target, "Overridden by other sign on");
+    nick_change_remote(source, parv);
   }
   else if (target == source)
   {
     if (strcmp(target->name, parv[1]))
-      change_remote_nick(source, parv);
+      nick_change_remote(source, parv);
   }
   else if (perform_nick_change_collides(source, target, parc, parv))
-    change_remote_nick(source, parv);
+    nick_change_remote(source, parv);
 }
 
 /*! \brief UID command handler
@@ -798,16 +779,16 @@ ms_uid(struct Client *source, int parc, char *parv[])
   struct Client *target = hash_find_id(parv[9]);
   if (target)
   {
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                         "ID collision on %s(%s <- %s)(both killed)",
-                         target->name, target->from->name, source->from->name);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                   "ID collision on %s(%s <- %s)(both killed)",
+                   target->name, target->from->name, source->from->name);
 
-    sendto_server(NULL, 0, 0, ":%s KILL %s :%s (ID collision)",
-                  me.id, target->id, me.name);
+    sendto_servers(NULL, 0, 0, ":%s KILL %s :%s (ID collision)",
+                   me.id, target->id, me.name);
 
     ++ServerStats.is_kill;
     AddFlag(target, FLAGS_KILLED);
-    exit_client(target, "ID Collision");
+    client_exit(target, "ID Collision");
     return;
   }
 
@@ -816,7 +797,7 @@ ms_uid(struct Client *source, int parc, char *parv[])
     uid_from_server(source, parc, parv);
   else if (IsUnknown(target))
   {
-    exit_client(target, "Overridden by other sign on");
+    client_exit(target, "Overridden by other sign on");
     uid_from_server(source, parc, parv);
   }
   else if (perform_uid_introduction_collides(source, target, parc, parv))
