@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,8 +28,9 @@
 #include "ircd.h"
 #include "numeric.h"
 #include "send.h"
-#include "irc_string.h"
+#include "io_string.h"
 #include "parse.h"
+#include "user_mode.h"
 #include "module.h"
 #include "hash.h"
 #include "conf.h"
@@ -52,21 +53,21 @@ m_ping(struct Client *source, int parc, char *parv[])
 {
   struct Client *target = NULL;
 
-  if (EmptyString(parv[1]))
+  if (string_is_empty(parv[1]))
   {
     sendto_one_numeric(source, &me, ERR_NOORIGIN);
     return;
   }
 
   const char *const destination = parv[2];  /* Will get NULL or pointer (parc >= 2!!) */
-  if (ConfigServerHide.disable_remote_commands && !HasUMode(source, UMODE_OPER))
+  if (ConfigServerHide.disable_remote_commands && user_mode_has_flag(source, UMODE_OPER) == false)
   {
     sendto_one(source, ":%s PONG %s :%s",
                me.name, (destination) ? destination : me.name, parv[1]);
     return;
   }
 
-  if (EmptyString(destination) || ((target = hash_find_server(destination)) && IsMe(target)))
+  if (string_is_empty(destination) || ((target = hash_find_server(destination)) && IsMe(target)))
     sendto_one(source, ":%s PONG %s :%s", me.name, me.name, parv[1]);
   else if (target)
     sendto_one(target, ":%s PING %s :%s",
@@ -92,14 +93,14 @@ ms_ping(struct Client *source, int parc, char *parv[])
 {
   struct Client *target = NULL;
 
-  if (EmptyString(parv[1]))
+  if (string_is_empty(parv[1]))
   {
     sendto_one_numeric(source, &me, ERR_NOORIGIN);
     return;
   }
 
   const char *const destination = parv[2];  /* Will get NULL or pointer (parc >= 2!!) */
-  if (EmptyString(destination) || ((target = hash_find_server(destination)) && IsMe(target)))
+  if (string_is_empty(destination) || ((target = hash_find_server(destination)) && IsMe(target)))
     sendto_one(source, ":%s PONG %s :%s",
                ID_or_name(&me, source), me.name, ID_or_name(source, source));
   else if (target)
