@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
 
 #include "stdinc.h"
 #include "client.h"
-#include "irc_string.h"
+#include "io_string.h"
 #include "ircd.h"
 #include "list.h"
 #include "numeric.h"
@@ -47,9 +47,9 @@ static void
 rehash_conf(struct Client *source)
 {
   sendto_one_numeric(source, &me, RPL_REHASHING, "CONF");
-  sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                       "%s is rehashing configuration file(s)",
-                       get_oper_name(source));
+  sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                 "%s is rehashing configuration file(s)",
+                 get_oper_name(source));
   log_write(LOG_TYPE_IRCD, "REHASH CONF from %s",
             get_oper_name(source));
   conf_rehash(false);
@@ -63,9 +63,9 @@ static void
 rehash_motd(struct Client *source)
 {
   sendto_one_numeric(source, &me, RPL_REHASHING, "MOTD");
-  sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                       "%s is forcing re-reading of MOTD files",
-                       get_oper_name(source));
+  sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                 "%s is forcing re-reading of MOTD files",
+                 get_oper_name(source));
   log_write(LOG_TYPE_IRCD, "REHASH MOTD from %s",
             get_oper_name(source));
   motd_recache();
@@ -79,9 +79,9 @@ static void
 rehash_dns(struct Client *source)
 {
   sendto_one_numeric(source, &me, RPL_REHASHING, "DNS");
-  sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                       "%s is rehashing DNS",
-                       get_oper_name(source));
+  sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                 "%s is rehashing DNS",
+                 get_oper_name(source));
   log_write(LOG_TYPE_IRCD, "REHASH DNS from %s",
             get_oper_name(source));
   restart_resolver();
@@ -122,7 +122,7 @@ mo_rehash(struct Client *source, int parc, char *parv[])
   const char *option = NULL;
   const char *server = NULL;
 
-  if (!EmptyString(parv[2]))
+  if (!string_is_empty(parv[2]))
   {
     if (!HasOFlag(source, OPER_FLAG_REHASH_REMOTE))
     {
@@ -149,17 +149,16 @@ mo_rehash(struct Client *source, int parc, char *parv[])
     if (irccmp(tab->option, option))
       continue;
 
-    if (!EmptyString(server))
+    if (!string_is_empty(server))
       sendto_match_servs(source, server, 0, "REHASH %s %s", server, option);
 
-    if (EmptyString(server) || match(server, me.name) == 0)
+    if (string_is_empty(server) || match(server, me.name) == 0)
       tab->handler(source);
 
     return;
   }
 
-  sendto_one_notice(source, &me, ":%s is not a valid option. "
-                    "Choose from CONF, DNS, MOTD",
+  sendto_one_notice(source, &me, ":%s is not a valid option. Choose from CONF, DNS, MOTD",
                     option);
 }
 
@@ -186,8 +185,7 @@ ms_rehash(struct Client *source, int parc, char *parv[])
   if (match(server, me.name))
     return;
 
-  if (!shared_find(SHARED_REHASH, source->servptr->name,
-                   source->username, source->host))
+  if (!shared_find(SHARED_REHASH, source->servptr->name, source->username, source->host))
     return;
 
   for (const struct RehashStruct *tab = rehash_cmd_table; tab->handler; ++tab)
