@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 2001-2024 ircd-hybrid development team
+ *  Copyright (c) 2001-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,9 +24,10 @@
  */
 
 #include "stdinc.h"
+#include "io_time.h"
 #include "client.h"
 #include "ircd.h"
-#include "irc_string.h"
+#include "io_string.h"
 #include "numeric.h"
 #include "server_capab.h"
 #include "send.h"
@@ -72,20 +73,20 @@ resv_handle(struct Client *source, const struct aline_ctx *aline)
   }
 
   resv = resv_make(aline->mask, aline->reason, NULL);
-  resv->setat = event_base->time.sec_real;
+  resv->created_at = io_time_get(IO_TIME_REALTIME_SEC);
   resv->in_database = true;
 
   if (aline->duration)
   {
-    resv->expire = event_base->time.sec_real + aline->duration;
+    resv->expires_at = resv->created_at + aline->duration;
 
     if (IsClient(source))
       sendto_one_notice(source, &me, ":Added temporary %ju min. RESV [%s]",
                         aline->duration / 60, resv->mask);
 
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                         "%s added temporary %ju min. RESV for [%s] [%s]",
-                         get_oper_name(source), aline->duration / 60, resv->mask, resv->reason);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                   "%s added temporary %ju min. RESV for [%s] [%s]",
+                   get_oper_name(source), aline->duration / 60, resv->mask, resv->reason);
     log_write(LOG_TYPE_RESV, "%s added temporary %ju min. RESV for [%s] [%s]",
               get_oper_name(source), aline->duration / 60, resv->mask, resv->reason);
   }
@@ -95,8 +96,8 @@ resv_handle(struct Client *source, const struct aline_ctx *aline)
       sendto_one_notice(source, &me, ":Added RESV [%s] [%s]",
                         resv->mask, resv->reason);
 
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE, "%s added RESV for [%s] [%s]",
-                         get_oper_name(source), resv->mask, resv->reason);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "%s added RESV for [%s] [%s]",
+                   get_oper_name(source), resv->mask, resv->reason);
     log_write(LOG_TYPE_RESV, "%s added RESV for [%s] [%s]",
               get_oper_name(source), resv->mask, resv->reason);
   }
