@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,13 +24,13 @@
  */
 
 #include "stdinc.h"
+#include "io_time.h"
 #include "list.h"
 #include "client.h"
 #include "client_svstag.h"
-#include "event.h"
 #include "hash.h"
 #include "id.h"
-#include "irc_string.h"
+#include "io_string.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "conf.h"
@@ -40,7 +40,7 @@
 #include "misc.h"
 #include "server.h"
 #include "server_capab.h"
-#include "user.h"
+#include "user_mode.h"
 #include "send.h"
 #include "parse.h"
 #include "memory.h"
@@ -89,19 +89,19 @@ server_send_client(struct Client *client_p, struct Client *target)
   sendto_one(client_p, ":%s UID %s %u %ju %s %s %s %s %s %s %s :%s",
              target->servptr->id,
              target->name, target->hopcount + 1,
-             target->tsinfo, user_get_mode_str(target->umodes),
+             target->tsinfo, user_mode_to_str(target->umodes),
              target->username, target->host, target->realhost,
              target->sockhost, target->id,
              target->account, target->info);
 
-  if (!EmptyString(target->tls_certfp))
+  if (!string_is_empty(target->tls_certfp))
     sendto_one(client_p, ":%s CERTFP %s", target->id, target->tls_certfp);
 
-  if (!EmptyString(target->tls_cipher))
+  if (!string_is_empty(target->tls_cipher))
     sendto_one(client_p, ":%s METADATA client %s cipher :%s",
                target->servptr->id, target->id, target->tls_cipher);
 
-  if (target->away[0])
+  if (target->away)
     sendto_one(client_p, ":%s AWAY :%s", target->id, target->away);
 
   list_node_t *node;
@@ -110,7 +110,7 @@ server_send_client(struct Client *client_p, struct Client *target)
     const struct ServicesTag *svstag = node->data;
     sendto_one(client_p, ":%s SVSTAG %s %ju %u +%s :%s", me.id,
                target->id, target->tsinfo, svstag->numeric,
-               user_get_mode_str(svstag->umodes), svstag->tag);
+               user_mode_to_str(svstag->umodes), svstag->tag);
   }
 }
 
@@ -161,14 +161,13 @@ server_estab(struct Client *client_p)
   if (ConfigServerInfo.hub == 0 && list_length(&local_server_list))
   {
     ++ServerStats.is_ref;
-    exit_client(client_p, "I'm a leaf not a hub");
+    client_exit(client_p, "I'm a leaf not a hub");
     return;
   }
 
   if (IsUnknown(client_p))
   {
-    const struct MaskItem *const conf = client_p->connection->confs.head->data;
-
+    const struct MaskItem *const conf = list_peek_head(&client_p->connection->confs);
     sendto_one(client_p, "PASS %s", conf->spasswd);
 
     sendto_one(client_p, "CAPAB :%s", capab_get(NULL, true));
@@ -178,7 +177,7 @@ server_estab(struct Client *client_p)
   }
 
   sendto_one(client_p, ":%s SVINFO %u %u 0 :%ju",
-             me.id, TS_CURRENT, TS_MINIMUM, event_base->time.sec_real);
+             me.id, SERVER_TS_PROTOCOL_CURRENT, SERVER_TS_PROTOCOL_MINIMUM, io_time_get(IO_TIME_REALTIME_SEC));
 
   SetServer(client_p);
   client_p->servptr = &me;
@@ -202,8 +201,8 @@ server_estab(struct Client *client_p)
   server_make(client_p);
 
   /* Fixing eob timings.. -gnp */
-  client_p->connection->created_monotonic = event_base->time.sec_monotonic;
-  client_p->connection->created_real = event_base->time.sec_real;
+  client_p->connection->created_monotonic = io_time_get(IO_TIME_MONOTONIC_SEC);
+  client_p->connection->created_real = io_time_get(IO_TIME_REALTIME_SEC);
 
   if (service_find(client_p->name, irccmp))
     AddFlag(client_p, FLAGS_SERVICE);
@@ -214,16 +213,16 @@ server_estab(struct Client *client_p)
     client_p->tls_cipher = io_strdup(tls_get_cipher(&client_p->connection->fd->tls));
 
     /* Show the real host/IP to admins */
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE,
-                         "Link with %s established: [TLS: %s] (Capabilities: %s)",
-                         client_get_name(client_p, SHOW_IP), client_p->tls_cipher,
-                         capab_get(client_p, true));
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE,
+                   "Link with %s established: [TLS: %s] (Capabilities: %s)",
+                   client_get_name(client_p, SHOW_IP), client_p->tls_cipher,
+                   capab_get(client_p, true));
 
     /* Now show the masked hostname/IP to opers */
-    sendto_realops_flags(UMODE_SERVNOTICE, L_OPER, SEND_NOTICE,
-                         "Link with %s established: [TLS: %s] (Capabilities: %s)",
-                         client_get_name(client_p, MASK_IP), client_p->tls_cipher,
-                         capab_get(client_p, true));
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER, SEND_TYPE_NOTICE,
+                   "Link with %s established: [TLS: %s] (Capabilities: %s)",
+                   client_get_name(client_p, MASK_IP), client_p->tls_cipher,
+                   capab_get(client_p, true));
     log_write(LOG_TYPE_IRCD, "Link with %s established: [TLS: %s] (Capabilities: %s)",
               client_get_name(client_p, SHOW_IP), client_p->tls_cipher,
               capab_get(client_p, true));
@@ -231,21 +230,21 @@ server_estab(struct Client *client_p)
   else
   {
     /* Show the real host/IP to admins */
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE,
-                         "Link with %s established: (Capabilities: %s)",
-                         client_get_name(client_p, SHOW_IP), capab_get(client_p, true));
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE,
+                   "Link with %s established: (Capabilities: %s)",
+                   client_get_name(client_p, SHOW_IP), capab_get(client_p, true));
     /* Now show the masked hostname/IP to opers */
-    sendto_realops_flags(UMODE_SERVNOTICE, L_OPER, SEND_NOTICE,
-                         "Link with %s established: (Capabilities: %s)",
-                         client_get_name(client_p, MASK_IP), capab_get(client_p, true));
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER, SEND_TYPE_NOTICE,
+                   "Link with %s established: (Capabilities: %s)",
+                   client_get_name(client_p, MASK_IP), capab_get(client_p, true));
     log_write(LOG_TYPE_IRCD, "Link with %s established: (Capabilities: %s)",
               client_get_name(client_p, SHOW_IP), capab_get(client_p, true));
   }
 
   fd_note(client_p->connection->fd, "Server: %s", client_p->name);
 
-  sendto_server(client_p, 0, 0, ":%s SID %s 2 %s +%s :%s",
-                me.id, client_p->name, client_p->id, IsHidden(client_p) ? "h" : "", client_p->info);
+  sendto_servers(client_p, 0, 0, ":%s SID %s 2 %s +%s :%s",
+                 me.id, client_p->name, client_p->id, IsHidden(client_p) ? "h" : "", client_p->info);
 
   /*
    * Pass on my client information to the new server
@@ -277,7 +276,7 @@ server_estab(struct Client *client_p)
 
   server_burst(client_p);
 
-  if (IsCapable(client_p, CAPAB_EOB))
+  if (capab_has_flag(client_p, CAPAB_EOB))
   {
     LIST_FOREACH_PREV(node, global_server_list.tail)
     {
@@ -322,11 +321,11 @@ server_check(const char *name, struct Client *client_p)
     if (!irccmp(conf->host, client_p->host) ||
         !irccmp(conf->host, client_p->sockhost))
     {
-      if (match_conf_password(client_p->connection->password, conf) == false)
+      if (conf_match_password(client_p->connection->password, conf) == false)
         return SERVER_CHECK_INVALID_PASSWORD;
 
-      if (!EmptyString(conf->certfp))
-        if (EmptyString(client_p->tls_certfp) || strcasecmp(client_p->tls_certfp, conf->certfp))
+      if (!string_is_empty(conf->certfp))
+        if (string_is_empty(client_p->tls_certfp) || strcasecmp(client_p->tls_certfp, conf->certfp))
           return SERVER_CHECK_INVALID_CERTIFICATE;
 
       conf_attach(client_p, conf);
@@ -355,43 +354,43 @@ mr_server(struct Client *source, int parc, char *parv[])
 
   if (listener_has_flag(source->connection->listener, LISTENER_CLIENT))
   {
-    exit_client(source, "Use a different port");
+    client_exit(source, "Use a different port");
     return;
   }
 
   if (server_valid_name(name) == false)
   {
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE,
-          "Unauthorized server connection attempt from %s: Bogus server name "
-          "for server %s", client_get_name(source, SHOW_IP), name);
-    sendto_realops_flags(UMODE_SERVNOTICE, L_OPER, SEND_NOTICE,
-          "Unauthorized server connection attempt from %s: Bogus server name "
-          "for server %s", client_get_name(source, MASK_IP), name);
-    exit_client(source, "Bogus server name");
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE,
+                   "Unauthorized server connection attempt from %s: Bogus server name for server %s",
+                   client_get_name(source, SHOW_IP), name);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER, SEND_TYPE_NOTICE,
+                   "Unauthorized server connection attempt from %s: Bogus server name for server %s",
+                   client_get_name(source, MASK_IP), name);
+    client_exit(source, "Bogus server name");
     return;
   }
 
   if (valid_sid(sid) == false)
   {
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE,
-                         "Link %s introduced server with bogus server ID %s",
-                         client_get_name(source, SHOW_IP), sid);
-    sendto_realops_flags(UMODE_SERVNOTICE, L_OPER, SEND_NOTICE,
-                         "Link %s introduced server with bogus server ID %s",
-                         client_get_name(source, MASK_IP), sid);
-    exit_client(source, "Bogus server ID introduced");
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE,
+                   "Link %s introduced server with bogus server ID %s",
+                   client_get_name(source, SHOW_IP), sid);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER, SEND_TYPE_NOTICE,
+                   "Link %s introduced server with bogus server ID %s",
+                   client_get_name(source, MASK_IP), sid);
+    client_exit(source, "Bogus server ID introduced");
     return;
   }
 
   if (IsHandshake(source) && irccmp(source->name, name))
   {
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE,
-                         "Link %s introduced server with mismatching server name %s",
-                         client_get_name(source, SHOW_IP), name);
-    sendto_realops_flags(UMODE_SERVNOTICE, L_OPER, SEND_NOTICE,
-                         "Link %s introduced server with mismatching server name %s",
-                         client_get_name(source, MASK_IP), name);
-    exit_client(source, "Mismatching server name introduced");
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE,
+                   "Link %s introduced server with mismatching server name %s",
+                   client_get_name(source, SHOW_IP), name);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER, SEND_TYPE_NOTICE,
+                   "Link %s introduced server with mismatching server name %s",
+                   client_get_name(source, MASK_IP), name);
+    client_exit(source, "Mismatching server name introduced");
     return;
   }
 
@@ -420,30 +419,30 @@ mr_server(struct Client *source, int parc, char *parv[])
   {
     if (warn)
     {
-      sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE,
-                           "Unauthorized server connection attempt from %s: %s for server %s",
-                           client_get_name(source, SHOW_IP), error, name);
-      sendto_realops_flags(UMODE_SERVNOTICE, L_OPER, SEND_NOTICE,
-                           "Unauthorized server connection attempt from %s: %s for server %s",
-                           client_get_name(source, MASK_IP), error, name);
+      sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE,
+                     "Unauthorized server connection attempt from %s: %s for server %s",
+                     client_get_name(source, SHOW_IP), error, name);
+      sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER, SEND_TYPE_NOTICE,
+                     "Unauthorized server connection attempt from %s: %s for server %s",
+                     client_get_name(source, MASK_IP), error, name);
     }
 
-    exit_client(source, error);
+    client_exit(source, error);
     return;
   }
 
   if (service_find(name, irccmp) == NULL)
   {
-    if ((ConfigChannel.enable_owner == 0) != !IsCapable(source, CAPAB_QOP) ||
-        (ConfigChannel.enable_admin == 0) != !IsCapable(source, CAPAB_AOP))
+    if ((ConfigChannel.enable_owner == 0) != !capab_has_flag(source, CAPAB_QOP) ||
+        (ConfigChannel.enable_admin == 0) != !capab_has_flag(source, CAPAB_AOP))
     {
-      sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE,
-                           "Link %s introduced server with mismatching AOP/QOP capabilities",
-                           client_get_name(source, SHOW_IP));
-      sendto_realops_flags(UMODE_SERVNOTICE, L_OPER, SEND_NOTICE,
-                           "Link %s introduced server with mismatching AOP/QOP capabilities",
-                           client_get_name(source, MASK_IP));
-      exit_client(source, "Mismatching AOP/QOP capabilities");
+      sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE,
+                     "Link %s introduced server with mismatching AOP/QOP capabilities",
+                     client_get_name(source, SHOW_IP));
+      sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER, SEND_TYPE_NOTICE,
+                     "Link %s introduced server with mismatching AOP/QOP capabilities",
+                     client_get_name(source, MASK_IP));
+      client_exit(source, "Mismatching AOP/QOP capabilities");
       return;
     }
   }
@@ -462,26 +461,26 @@ mr_server(struct Client *source, int parc, char *parv[])
      * Definitely don't do that here. This is from an unregistered
      * connect - A1kmm.
      */
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE,
-                         "Attempt to re-introduce server %s from %s",
-                         name, client_get_name(source, SHOW_IP));
-    sendto_realops_flags(UMODE_SERVNOTICE, L_OPER, SEND_NOTICE,
-                         "Attempt to re-introduce server %s from %s",
-                         name, client_get_name(source, MASK_IP));
-    exit_client(source, "Server already exists");
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE,
+                   "Attempt to re-introduce server %s from %s",
+                   name, client_get_name(source, SHOW_IP));
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER, SEND_TYPE_NOTICE,
+                   "Attempt to re-introduce server %s from %s",
+                   name, client_get_name(source, MASK_IP));
+    client_exit(source, "Server already exists");
     return;
   }
 
   target = hash_find_id(sid);
   if (target)
   {
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE,
-                         "Attempt to re-introduce server %s SID %s from %s",
-                         name, sid, client_get_name(source, SHOW_IP));
-    sendto_realops_flags(UMODE_SERVNOTICE, L_OPER, SEND_NOTICE,
-                         "Attempt to re-introduce server %s SID %s from %s",
-                         name, sid, client_get_name(source, MASK_IP));
-    exit_client(source, "Server ID already exists");
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE,
+                   "Attempt to re-introduce server %s SID %s from %s",
+                   name, sid, client_get_name(source, SHOW_IP));
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER, SEND_TYPE_NOTICE,
+                   "Attempt to re-introduce server %s SID %s from %s",
+                   name, sid, client_get_name(source, MASK_IP));
+    client_exit(source, "Server ID already exists");
     return;
   }
 
@@ -491,7 +490,7 @@ mr_server(struct Client *source, int parc, char *parv[])
    */
   target = find_servconn_in_progress(name);
   if (target && (target != source))
-    exit_client(target, "Overridden");
+    client_exit(target, "Overridden");
 
   /*
    * If we are connecting (Handshake), we already have the name from the
@@ -531,25 +530,25 @@ ms_sid(struct Client *source, int parc, char *parv[])
 
   if (server_valid_name(parv[1]) == false)
   {
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE,
-                         "Link %s introduced server with bogus server name %s",
-                         client_get_name(source->from, SHOW_IP), parv[1]);
-    sendto_realops_flags(UMODE_SERVNOTICE, L_OPER, SEND_NOTICE,
-                         "Link %s introduced server with bogus server name %s",
-                         client_get_name(source->from, MASK_IP), parv[1]);
-    exit_client(source->from, "Bogus server name introduced");
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE,
+                   "Link %s introduced server with bogus server name %s",
+                   client_get_name(source->from, SHOW_IP), parv[1]);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER, SEND_TYPE_NOTICE,
+                   "Link %s introduced server with bogus server name %s",
+                   client_get_name(source->from, MASK_IP), parv[1]);
+    client_exit(source->from, "Bogus server name introduced");
     return;
   }
 
   if (valid_sid(parv[3]) == false)
   {
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE,
-                         "Link %s introduced server with bogus server ID %s",
-                         client_get_name(source->from, SHOW_IP), parv[3]);
-    sendto_realops_flags(UMODE_SERVNOTICE, L_OPER, SEND_NOTICE,
-                         "Link %s introduced server with bogus server ID %s",
-                         client_get_name(source->from, MASK_IP), parv[3]);
-    exit_client(source->from, "Bogus server ID introduced");
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE,
+                   "Link %s introduced server with bogus server ID %s",
+                   client_get_name(source->from, SHOW_IP), parv[3]);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER, SEND_TYPE_NOTICE,
+                   "Link %s introduced server with bogus server ID %s",
+                   client_get_name(source->from, MASK_IP), parv[3]);
+    client_exit(source->from, "Bogus server ID introduced");
     return;
   }
 
@@ -557,13 +556,13 @@ ms_sid(struct Client *source, int parc, char *parv[])
   struct Client *target = hash_find_id(parv[3]);
   if (target)
   {
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE,
-                         "Link %s cancelled, server ID %s already exists",
-                         client_get_name(source->from, SHOW_IP), parv[3]);
-    sendto_realops_flags(UMODE_SERVNOTICE, L_OPER, SEND_NOTICE,
-                         "Link %s cancelled, server ID %s already exists",
-                         client_get_name(source->from, MASK_IP), parv[3]);
-    exit_client(source->from, "Link cancelled, server ID already exists");
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE,
+                   "Link %s cancelled, server ID %s already exists",
+                   client_get_name(source->from, SHOW_IP), parv[3]);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER, SEND_TYPE_NOTICE,
+                   "Link %s cancelled, server ID %s already exists",
+                   client_get_name(source->from, MASK_IP), parv[3]);
+    client_exit(source->from, "Link cancelled, server ID already exists");
     return;
   }
 
@@ -571,13 +570,13 @@ ms_sid(struct Client *source, int parc, char *parv[])
   target = hash_find_server(parv[1]);
   if (target)
   {
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE,
-                         "Link %s cancelled, server %s already exists",
-                         client_get_name(source->from, SHOW_IP), parv[1]);
-    sendto_realops_flags(UMODE_SERVNOTICE, L_OPER, SEND_NOTICE,
-                         "Link %s cancelled, server %s already exists",
-                         client_get_name(source->from, MASK_IP), parv[1]);
-    exit_client(source->from, "Server exists");
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE,
+                   "Link %s cancelled, server %s already exists",
+                   client_get_name(source->from, SHOW_IP), parv[1]);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER, SEND_TYPE_NOTICE,
+                   "Link %s cancelled, server %s already exists",
+                   client_get_name(source->from, MASK_IP), parv[1]);
+    client_exit(source->from, "Server exists");
     return;
   }
 
@@ -587,13 +586,13 @@ ms_sid(struct Client *source, int parc, char *parv[])
    */
   target = find_servconn_in_progress(parv[1]);
   if (target && (target != source->from))
-    exit_client(target, "Overridden");
+    client_exit(target, "Overridden");
 
   /*
    * See if the newly found server is behind a guaranteed
    * leaf. If so, close the link.
    */
-  const struct MaskItem *conf = source->from->connection->confs.head->data;
+  const struct MaskItem *conf = list_peek_head(&source->from->connection->confs);
   bool hlined = list_find_cmp(&conf->hub_list , parv[1], match) != NULL;
   bool llined = list_find_cmp(&conf->leaf_list, parv[1], match) != NULL;
 
@@ -626,13 +625,13 @@ ms_sid(struct Client *source, int parc, char *parv[])
   if (hlined == false)
   {
     /* OOOPs nope can't HUB */
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE,
-                         "Non-Hub link %s introduced %s.",
-                         client_get_name(source->from, SHOW_IP), parv[1]);
-    sendto_realops_flags(UMODE_SERVNOTICE, L_OPER, SEND_NOTICE,
-                         "Non-Hub link %s introduced %s.",
-                         client_get_name(source->from, MASK_IP), parv[1]);
-    exit_client(source, "No matching hub_mask.");
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE,
+                   "Non-Hub link %s introduced %s.",
+                   client_get_name(source->from, SHOW_IP), parv[1]);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER, SEND_TYPE_NOTICE,
+                   "Non-Hub link %s introduced %s.",
+                   client_get_name(source->from, MASK_IP), parv[1]);
+    client_exit(source, "No matching hub_mask.");
     return;
   }
 
@@ -640,13 +639,13 @@ ms_sid(struct Client *source, int parc, char *parv[])
   if (llined)
   {
     /* OOOPs nope can't HUB this leaf */
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE,
-                         "Link %s introduced leafed server %s.",
-                         client_get_name(source->from, SHOW_IP), parv[1]);
-    sendto_realops_flags(UMODE_SERVNOTICE, L_OPER, SEND_NOTICE,
-                         "Link %s introduced leafed server %s.",
-                         client_get_name(source->from, MASK_IP), parv[1]);
-    exit_client(source->from, "Leafed server.");
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE,
+                   "Link %s introduced leafed server %s.",
+                   client_get_name(source->from, SHOW_IP), parv[1]);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER, SEND_TYPE_NOTICE,
+                   "Link %s introduced leafed server %s.",
+                   client_get_name(source->from, MASK_IP), parv[1]);
+    client_exit(source->from, "Leafed server.");
     return;
   }
 
@@ -672,11 +671,10 @@ ms_sid(struct Client *source, int parc, char *parv[])
   hash_add_client(target);
   hash_add_id(target);
 
-  sendto_server(source->from, 0, 0, ":%s SID %s %u %s +%s :%s",
-                source->id, target->name, target->hopcount + 1,
-                target->id, IsHidden(target) ? "h" : "", target->info);
-  sendto_realops_flags(UMODE_EXTERNAL, L_ALL, SEND_NOTICE, "Server %s being introduced by %s",
-                       target->name, source->name);
+  sendto_servers(source->from, 0, 0, ":%s SID %s %u %s +%s :%s",
+                 source->id, target->name, target->hopcount + 1, target->id, IsHidden(target) ? "h" : "", target->info);
+  sendto_clients(UMODE_EXTERNAL, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "Server %s being introduced by %s",
+                 target->name, source->name);
 }
 
 static struct Command command_table[] =
