@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,9 +24,10 @@
  */
 
 #include "stdinc.h"
+#include "io_time.h"
 #include "list.h"
 #include "client.h"
-#include "irc_string.h"
+#include "io_string.h"
 #include "ircd.h"
 #include "listener.h"
 #include "parse.h"
@@ -45,6 +46,7 @@
 #include "misc.h"
 #include "server.h"
 #include "server_capab.h"
+#include "user_mode.h"
 #include "event.h"
 #include "module.h"
 #include "whowas.h"
@@ -216,7 +218,7 @@ stats_gecos(struct Client *client, int parc, char *parv[])
   {
     const struct GecosItem *gecos = node->data;
     sendto_one_numeric(client, &me, RPL_STATSXLINE,
-                       gecos->expire ? 'x' : 'X', gecos->mask, gecos->reason);
+                       gecos->expires_at ? 'x' : 'X', gecos->mask, gecos->reason);
   }
 }
 
@@ -229,7 +231,7 @@ stats_gecos(struct Client *client, int parc, char *parv[])
 static void
 stats_operator(struct Client *client, int parc, char *parv[])
 {
-  if (!HasUMode(client, UMODE_OPER) && ConfigGeneral.stats_o_oper_only)
+  if (!user_mode_has_flag(client, UMODE_OPER) && ConfigGeneral.stats_o_oper_only)
   {
     sendto_one_numeric(client, &me, ERR_NOPRIVILEGES);
     return;
@@ -241,7 +243,7 @@ stats_operator(struct Client *client, int parc, char *parv[])
     const struct MaskItem *conf = node->data;
 
     /* Don't allow non opers to see oper privs */
-    if (HasUMode(client, UMODE_OPER))
+    if (user_mode_has_flag(client, UMODE_OPER))
       sendto_one_numeric(client, &me, RPL_STATSOLINE, 'O', conf->user, conf->host,
                          conf->name, oper_privs_as_string(conf->port),
                          conf->class->name);
@@ -274,7 +276,7 @@ stats_connect(struct Client *client, int parc, char *parv[])
     /*
      * Allow admins to see actual ips unless 'hide_server_ips' is enabled
      */
-    if (ConfigServerHide.hide_server_ips == 0 && HasUMode(client, UMODE_ADMIN))
+    if (ConfigServerHide.hide_server_ips == 0 && user_mode_has_flag(client, UMODE_ADMIN))
       sendto_one_numeric(client, &me, RPL_STATSCLINE,
                          'C', conf->host, buf, conf->name, conf->port, conf->class->name);
     else
@@ -298,14 +300,14 @@ stats_resv(struct Client *client, int parc, char *parv[])
   {
     const struct ResvItem *resv = node->data;
     sendto_one_numeric(client, &me, RPL_STATSQLINE,
-                       resv->expire ? 'q' : 'Q', resv->mask, resv->reason);
+                       resv->expires_at ? 'q' : 'Q', resv->mask, resv->reason);
   }
 
   LIST_FOREACH(node, resv_nick_get_list()->head)
   {
     const struct ResvItem *resv = node->data;
     sendto_one_numeric(client, &me, RPL_STATSQLINE,
-                       resv->expire ? 'q' : 'Q', resv->mask, resv->reason);
+                       resv->expires_at ? 'q' : 'Q', resv->mask, resv->reason);
   }
 }
 
@@ -318,14 +320,12 @@ stats_memory(struct Client *client, int parc, char *parv[])
   unsigned int channel_bans = 0;
   unsigned int channel_except = 0;
   unsigned int channel_invex = 0;
-  unsigned int wwu = 0;                  /* whowas users */
   unsigned int number_ips_stored = 0;        /* number of ip addresses hashed */
   size_t channel_ban_memory = 0;
   size_t channel_except_memory = 0;
   size_t channel_invex_memory = 0;
   unsigned int safelist_count = 0;
   size_t safelist_memory = 0;
-  size_t wwm = 0;               /* whowas array memory used       */
   size_t mem_ips_stored = 0;        /* memory used by ip address hash */
   unsigned int local_client_count  = 0;
   unsigned int remote_client_count = 0;
@@ -443,9 +443,11 @@ stats_memory(struct Client *client, int parc, char *parv[])
   sendto_one_numeric(client, &me, RPL_STATSDEBUG | SND_EXPLICIT, "z :Safelist %u(%zu)",
                      safelist_count, safelist_memory);
 
-  whowas_count_memory(&wwu, &wwm);
-  sendto_one_numeric(client, &me, RPL_STATSDEBUG | SND_EXPLICIT, "z :Whowas users %u(%zu)",
-                     wwu, wwm);
+  unsigned int group_count, whowas_count;
+  size_t group_bytes, whowas_bytes;
+  whowas_count_memory(&group_count, &group_bytes, &whowas_count, &whowas_bytes);
+  sendto_one_numeric(client, &me, RPL_STATSDEBUG | SND_EXPLICIT, "z :Whowas groups %u(%zu), users %u(%zu)",
+                     group_count, group_bytes, whowas_count, whowas_bytes);
 
   motd_memory_count(client);
 
@@ -468,9 +470,8 @@ stats_dns_servers(struct Client *client, int parc, char *parv[])
   for (unsigned int i = 0; i < reslib_nscount; ++i)
   {
     char buf[HOSTIPLEN + 1];
-    getnameinfo((const struct sockaddr *)&reslib_nsaddr_list[i], reslib_nsaddr_list[i].ss_len,
-                buf, sizeof(buf), NULL, 0, NI_NUMERICHOST);
-    sendto_one_numeric(client, &me, RPL_STATSALINE, buf);
+    if (address_to_string(&reslib_nsaddr_list[i], buf, sizeof(buf)))
+      sendto_one_numeric(client, &me, RPL_STATSALINE, buf);
   }
 }
 
@@ -576,7 +577,7 @@ stats_events(struct Client *client, int parc, char *parv[])
   {
     const struct event *ev = node->data;
     sendto_one_numeric(client, &me, RPL_STATSDEBUG | SND_EXPLICIT, "E :%-30s %-4ju seconds",
-                       ev->name, ev->next - event_base->time.sec_monotonic);
+                       ev->name, ev->next - io_time_get(IO_TIME_MONOTONIC_SEC));
   }
 }
 
@@ -644,7 +645,7 @@ show_iline_prefix(const struct Client *client, const struct MaskItem *conf)
     *bufptr++ = '=';
   if (IsConfCanFlood(conf))
     *bufptr++ = '|';
-  if (HasUMode(client, UMODE_OPER))
+  if (user_mode_has_flag(client, UMODE_OPER))
   {
     if (IsConfExemptKline(conf))
       *bufptr++ = '^';
@@ -662,7 +663,7 @@ static void
 stats_auth(struct Client *client, int parc, char *parv[])
 {
   /* Oper only, if unopered, return ERR_NOPRIVILEGES */
-  if (ConfigGeneral.stats_i_oper_only && !HasUMode(client, UMODE_OPER))
+  if (ConfigGeneral.stats_i_oper_only && !user_mode_has_flag(client, UMODE_OPER))
   {
     sendto_one_numeric(client, &me, ERR_NOPRIVILEGES);
     return;
@@ -679,11 +680,11 @@ stats_auth(struct Client *client, int parc, char *parv[])
         continue;
 
       const struct MaskItem *conf = arec->conf;
-      if (IsConfDoSpoofIp(conf) && !HasUMode(client, UMODE_OPER))
+      if (IsConfDoSpoofIp(conf) && user_mode_has_flag(client, UMODE_OPER) == false)
         continue;
 
       sendto_one_numeric(client, &me, RPL_STATSILINE, 'I',
-                         conf->name == NULL ? "*" : conf->name,
+                         string_default(conf->name, "*"),
                          show_iline_prefix(client, conf),
                          conf->host, conf->port,
                          conf->class->name);
@@ -702,7 +703,7 @@ static void
 stats_kill(struct Client *client, int parc, char *parv[])
 {
   /* Oper only, if unopered, return ERR_NOPRIVILEGES */
-  if (ConfigGeneral.stats_k_oper_only && !HasUMode(client, UMODE_OPER))
+  if (ConfigGeneral.stats_k_oper_only && !user_mode_has_flag(client, UMODE_OPER))
   {
     sendto_one_numeric(client, &me, ERR_NOPRIVILEGES);
     return;
@@ -733,7 +734,7 @@ static void
 stats_tkill(struct Client *client, int parc, char *parv[])
 {
   /* Oper only, if unopered, return ERR_NOPRIVILEGES */
-  if (ConfigGeneral.stats_k_oper_only && !HasUMode(client, UMODE_OPER))
+  if (ConfigGeneral.stats_k_oper_only && !user_mode_has_flag(client, UMODE_OPER))
   {
     sendto_one_numeric(client, &me, ERR_NOPRIVILEGES);
     return;
@@ -762,7 +763,7 @@ stats_tkill(struct Client *client, int parc, char *parv[])
 static void
 stats_messages(struct Client *client, int parc, char *parv[])
 {
-  if (!HasUMode(client, UMODE_OPER) && ConfigGeneral.stats_m_oper_only)
+  if (!user_mode_has_flag(client, UMODE_OPER) && ConfigGeneral.stats_m_oper_only)
     sendto_one_numeric(client, &me, ERR_NOPRIVILEGES);
   else
     command_report(client);
@@ -777,8 +778,7 @@ stats_pseudo(struct Client *client, int parc, char *parv[])
   {
     const struct PseudoItem *pseudo = node->data;
     sendto_one_numeric(client, &me, RPL_STATSPSEUDO,
-                       pseudo->command, pseudo->name, pseudo->nick, pseudo->server,
-                       pseudo->prepend ? pseudo->prepend : "*");
+                       pseudo->command, pseudo->name, pseudo->nick, pseudo->server, string_default(pseudo->prepend, "*"));
   }
 }
 
@@ -798,21 +798,21 @@ stats_operedup(struct Client *client, int parc, char *parv[])
   {
     const struct Client *target = node->data;
 
-    if (HasUMode(target, UMODE_HIDDEN) && !HasUMode(client, UMODE_OPER))
+    if (user_mode_has_flag(target, UMODE_HIDDEN) && user_mode_has_flag(client, UMODE_OPER) == false)
       continue;
 
     const char *duration = "n/a";
-    if (HasUMode(client, UMODE_OPER) || !HasUMode(target, UMODE_HIDEIDLE))
+    if (user_mode_has_flag(client, UMODE_OPER) || user_mode_has_flag(target, UMODE_HIDEIDLE) == false)
       duration = time_format_duration(client_get_idle_time(client, target));
 
-    if (MyConnect(client) && HasUMode(client, UMODE_OPER))
+    if (MyConnect(client) && user_mode_has_flag(client, UMODE_OPER))
       sendto_one_numeric(client, &me, RPL_STATSDEBUG | SND_EXPLICIT, "p :[%c][%s] %s (%s@%s) Idle: %s",
-                         HasUMode(target, UMODE_ADMIN) ? 'A' : 'O',
+                         user_mode_has_flag(target, UMODE_ADMIN) ? 'A' : 'O',
                          oper_privs_as_string(target->connection->operflags),
                          target->name, target->username, target->host, duration);
     else
       sendto_one_numeric(client, &me, RPL_STATSDEBUG | SND_EXPLICIT, "p :[%c] %s (%s@%s) Idle: %s",
-                         HasUMode(target, UMODE_ADMIN) ? 'A' : 'O',
+                         user_mode_has_flag(target, UMODE_ADMIN) ? 'A' : 'O',
                          target->name, target->username, target->host, duration);
     ++count;
   }
@@ -829,7 +829,7 @@ stats_operedup(struct Client *client, int parc, char *parv[])
 static void
 stats_ports(struct Client *client, int parc, char *parv[])
 {
-  if (ConfigGeneral.stats_P_oper_only && !HasUMode(client, UMODE_OPER))
+  if (ConfigGeneral.stats_P_oper_only && !user_mode_has_flag(client, UMODE_OPER))
   {
     sendto_one_numeric(client, &me, ERR_NOPRIVILEGES);
     return;
@@ -844,7 +844,7 @@ stats_ports(struct Client *client, int parc, char *parv[])
 
     if (listener_has_flag(listener, LISTENER_HIDDEN))
     {
-      if (!HasUMode(client, UMODE_ADMIN))
+      if (user_mode_has_flag(client, UMODE_ADMIN) == false)
         continue;
       *bufptr++ = 'H';
     }
@@ -859,14 +859,14 @@ stats_ports(struct Client *client, int parc, char *parv[])
       *bufptr++ = 'D';
     *bufptr = '\0';
 
-    if (HasUMode(client, UMODE_ADMIN) && ConfigServerHide.hide_server_ips == 0)
+    if (user_mode_has_flag(client, UMODE_ADMIN) && ConfigServerHide.hide_server_ips == 0)
       sendto_one_numeric(client, &me, RPL_STATSPLINE,
-                         'P', listener->port, listener->name, listener->ref_count, buf,
-                         listener->active == true ? "active" : "disabled");
+                         'P', listener_get_port(listener), listener_get_name(listener), listener->ref_count, buf,
+                         listener_is_active(listener) ? "active" : "disabled");
     else
       sendto_one_numeric(client, &me, RPL_STATSPLINE,
-                         'P', listener->port, me.name, listener->ref_count, buf,
-                         listener->active == true ? "active" : "disabled");
+                         'P', listener_get_port(listener), me.name, listener->ref_count, buf,
+                         listener_is_active(listener) ? "active" : "disabled");
   }
 }
 
@@ -881,7 +881,7 @@ stats_tstats(struct Client *client, int parc, char *parv[])
     const struct Client *target = node->data;
     sp.is_sbs += target->connection->send.bytes;
     sp.is_sbr += target->connection->recv.bytes;
-    sp.is_sti += event_base->time.sec_monotonic - target->connection->created_monotonic;
+    sp.is_sti += io_time_get(IO_TIME_MONOTONIC_SEC) - target->connection->created_monotonic;
     sp.is_sv++;
   }
 
@@ -890,7 +890,7 @@ stats_tstats(struct Client *client, int parc, char *parv[])
     const struct Client *target = node->data;
     sp.is_cbs += target->connection->send.bytes;
     sp.is_cbr += target->connection->recv.bytes;
-    sp.is_cti += event_base->time.sec_monotonic - target->connection->created_monotonic;
+    sp.is_cti += io_time_get(IO_TIME_MONOTONIC_SEC) - target->connection->created_monotonic;
     sp.is_cl++;
   }
 
@@ -920,13 +920,13 @@ stats_tstats(struct Client *client, int parc, char *parv[])
 static void
 stats_uptime(struct Client *client, int parc, char *parv[])
 {
-  if (!HasUMode(client, UMODE_OPER) && ConfigGeneral.stats_u_oper_only)
+  if (!user_mode_has_flag(client, UMODE_OPER) && ConfigGeneral.stats_u_oper_only)
     sendto_one_numeric(client, &me, ERR_NOPRIVILEGES);
   else
   {
     sendto_one_numeric(client, &me, RPL_STATSUPTIME,
-                       time_format_duration(event_base->time.sec_monotonic - me.connection->created_monotonic));
-    if (ConfigServerHide.disable_remote_commands == 0 || HasUMode(client, UMODE_OPER))
+                       time_format_duration(io_time_get(IO_TIME_MONOTONIC_SEC) - me.connection->created_monotonic));
+    if (ConfigServerHide.disable_remote_commands == 0 || user_mode_has_flag(client, UMODE_OPER))
        sendto_one_numeric(client, &me, RPL_STATSCONN, Count.max_loc_con,
                           Count.max_loc, Count.totalrestartcount);
   }
@@ -954,8 +954,8 @@ stats_servers(struct Client *client, int parc, char *parv[])
   {
     const struct Client *target = node->data;
     sendto_one_numeric(client, &me, RPL_STATSDEBUG | SND_EXPLICIT, "v :%s (%s!%s@%s) Idle: %s",
-                       target->name, (target->serv->by[0] ? target->serv->by : "Remote."),
-                       "*", "*", time_format_duration(event_base->time.sec_monotonic - target->connection->last_data));
+                       target->name, target->serv->initiator_name ? target->serv->initiator_name : "Remote.",
+                       "*", "*", time_format_duration(io_time_get(IO_TIME_MONOTONIC_SEC) - target->connection->last_data));
   }
 
   sendto_one_numeric(client, &me, RPL_STATSDEBUG | SND_EXPLICIT, "v :%u Server(s)",
@@ -996,14 +996,14 @@ stats_servlinks(struct Client *client, int parc, char *parv[])
 
     /* ":%s 211 %s %s %u %u %zu %u %zu :%ju %ju %s" */
     sendto_one_numeric(client, &me, RPL_STATSLINKINFO,
-                       client_get_name(target, HasUMode(client, UMODE_ADMIN) ? SHOW_IP : MASK_IP),
+                       client_get_name(target, user_mode_has_flag(client, UMODE_ADMIN) ? SHOW_IP : MASK_IP),
                        dbuf_length(&target->connection->buf_sendq),
                        target->connection->send.messages,
                        target->connection->send.bytes >> 10,
                        target->connection->recv.messages,
                        target->connection->recv.bytes >> 10,
-                       (event_base->time.sec_monotonic - target->connection->created_monotonic),
-                       (event_base->time.sec_monotonic - target->connection->last_data),
+                       (io_time_get(IO_TIME_MONOTONIC_SEC) - target->connection->created_monotonic),
+                       (io_time_get(IO_TIME_MONOTONIC_SEC) - target->connection->last_data),
                        capab_get(target, true));
   }
 
@@ -1017,7 +1017,7 @@ stats_servlinks(struct Client *client, int parc, char *parv[])
   sendto_one_numeric(client, &me, RPL_STATSDEBUG | SND_EXPLICIT, "? :Recv total: %7.2f %s",
                      _GMKv(recv_bytes), _GMKs(recv_bytes));
 
-  const uintmax_t uptime = (event_base->time.sec_monotonic - me.connection->created_monotonic);
+  const uintmax_t uptime = (io_time_get(IO_TIME_MONOTONIC_SEC) - me.connection->created_monotonic);
   sendto_one_numeric(client, &me, RPL_STATSDEBUG | SND_EXPLICIT, "? :Server send: %7.2f %s (%4.1f KiB/s)",
                      _GMKv((me.connection->send.bytes >> 10)),
                      _GMKs((me.connection->send.bytes >> 10)),
@@ -1044,7 +1044,7 @@ parse_stats_args(struct Client *client, int parc, char *parv[], bool *doall, boo
 {
   const char *const name = parv[2];
 
-  if (EmptyString(name))
+  if (string_is_empty(name))
     return NULL;
 
   if (irccmp(name, ID_or_name(&me, client)) == 0)
@@ -1083,7 +1083,7 @@ stats_L_list(struct Client *client, const char *name, bool doall, bool wilds,
       type = HIDE_IP;
 
     if (IsServer(target) || IsConnecting(target) || IsHandshake(target))
-      if (!HasUMode(client, UMODE_ADMIN))
+      if (user_mode_has_flag(client, UMODE_ADMIN) == false)
         type = MASK_IP;
 
     sendto_one_numeric(client, &me, RPL_STATSLINKINFO,
@@ -1093,8 +1093,8 @@ stats_L_list(struct Client *client, const char *name, bool doall, bool wilds,
                        target->connection->send.bytes >> 10,
                        target->connection->recv.messages,
                        target->connection->recv.bytes >> 10,
-                       (event_base->time.sec_monotonic - target->connection->created_monotonic),
-                       (event_base->time.sec_monotonic - target->connection->last_data),
+                       (io_time_get(IO_TIME_MONOTONIC_SEC) - target->connection->created_monotonic),
+                       (io_time_get(IO_TIME_MONOTONIC_SEC) - target->connection->last_data),
                        IsServer(target) ? capab_get(target, true) : "-");
   }
 }
@@ -1134,45 +1134,45 @@ stats_ltrace(struct Client *client, int parc, char *parv[])
 
 static const struct StatsHandler stats_tab[] =
 {
-  STATS_HANDLER_INIT('a', stats_dns_servers, UMODE_ADMIN),
-  STATS_HANDLER_INIT('A', stats_dns_servers, UMODE_ADMIN),
-  STATS_HANDLER_INIT('c', stats_connect, UMODE_OPER),
-  STATS_HANDLER_INIT('C', stats_connect, UMODE_OPER),
-  STATS_HANDLER_INIT('d', stats_tdeny, UMODE_OPER),
-  STATS_HANDLER_INIT('D', stats_deny, UMODE_OPER),
-  STATS_HANDLER_INIT('e', stats_exempt, UMODE_OPER),
-  STATS_HANDLER_INIT('E', stats_events, UMODE_ADMIN),
-  STATS_HANDLER_INIT('f', stats_fdlist, UMODE_ADMIN),
-  STATS_HANDLER_INIT('F', stats_fdlist, UMODE_ADMIN),
-  STATS_HANDLER_INIT('h', stats_hubleaf, UMODE_OPER),
-  STATS_HANDLER_INIT('H', stats_hubleaf, UMODE_OPER),
+  STATS_HANDLER_INIT('a', stats_dns_servers, &UMODE_ADMIN),
+  STATS_HANDLER_INIT('A', stats_dns_servers, &UMODE_ADMIN),
+  STATS_HANDLER_INIT('c', stats_connect, &UMODE_OPER),
+  STATS_HANDLER_INIT('C', stats_connect, &UMODE_OPER),
+  STATS_HANDLER_INIT('d', stats_tdeny, &UMODE_OPER),
+  STATS_HANDLER_INIT('D', stats_deny, &UMODE_OPER),
+  STATS_HANDLER_INIT('e', stats_exempt, &UMODE_OPER),
+  STATS_HANDLER_INIT('E', stats_events, &UMODE_ADMIN),
+  STATS_HANDLER_INIT('f', stats_fdlist, &UMODE_ADMIN),
+  STATS_HANDLER_INIT('F', stats_fdlist, &UMODE_ADMIN),
+  STATS_HANDLER_INIT('h', stats_hubleaf, &UMODE_OPER),
+  STATS_HANDLER_INIT('H', stats_hubleaf, &UMODE_OPER),
   STATS_HANDLER_INIT('i', stats_auth, 0),
   STATS_HANDLER_INIT('I', stats_auth, 0),
   STATS_HANDLER_INIT('k', stats_tkill, 0),
   STATS_HANDLER_INIT('K', stats_kill, 0),
-  STATS_HANDLER_INIT('l', stats_ltrace, UMODE_OPER),
-  STATS_HANDLER_INIT('L', stats_ltrace, UMODE_OPER),
+  STATS_HANDLER_INIT('l', stats_ltrace, &UMODE_OPER),
+  STATS_HANDLER_INIT('L', stats_ltrace, &UMODE_OPER),
   STATS_HANDLER_INIT('m', stats_messages, 0),
   STATS_HANDLER_INIT('M', stats_messages, 0),
   STATS_HANDLER_INIT('o', stats_operator, 0),
   STATS_HANDLER_INIT('O', stats_operator, 0),
   STATS_HANDLER_INIT('p', stats_operedup, 0),
   STATS_HANDLER_INIT('P', stats_ports, 0),
-  STATS_HANDLER_INIT('q', stats_resv, UMODE_OPER),
-  STATS_HANDLER_INIT('Q', stats_resv, UMODE_OPER),
-  STATS_HANDLER_INIT('s', stats_pseudo, UMODE_OPER),
-  STATS_HANDLER_INIT('S', stats_service, UMODE_OPER),
-  STATS_HANDLER_INIT('t', stats_tstats, UMODE_OPER),
-  STATS_HANDLER_INIT('T', motd_report, UMODE_OPER),
+  STATS_HANDLER_INIT('q', stats_resv, &UMODE_OPER),
+  STATS_HANDLER_INIT('Q', stats_resv, &UMODE_OPER),
+  STATS_HANDLER_INIT('s', stats_pseudo, &UMODE_OPER),
+  STATS_HANDLER_INIT('S', stats_service, &UMODE_OPER),
+  STATS_HANDLER_INIT('t', stats_tstats, &UMODE_OPER),
+  STATS_HANDLER_INIT('T', motd_report, &UMODE_OPER),
   STATS_HANDLER_INIT('u', stats_uptime, 0),
-  STATS_HANDLER_INIT('U', stats_shared, UMODE_OPER),
-  STATS_HANDLER_INIT('v', stats_servers, UMODE_OPER),
-  STATS_HANDLER_INIT('x', stats_gecos, UMODE_OPER),
-  STATS_HANDLER_INIT('X', stats_gecos, UMODE_OPER),
-  STATS_HANDLER_INIT('y', stats_class, UMODE_OPER),
-  STATS_HANDLER_INIT('Y', stats_class, UMODE_OPER),
-  STATS_HANDLER_INIT('z', stats_memory, UMODE_OPER),
-  STATS_HANDLER_INIT('?', stats_servlinks, UMODE_OPER)
+  STATS_HANDLER_INIT('U', stats_shared, &UMODE_OPER),
+  STATS_HANDLER_INIT('v', stats_servers, &UMODE_OPER),
+  STATS_HANDLER_INIT('x', stats_gecos, &UMODE_OPER),
+  STATS_HANDLER_INIT('X', stats_gecos, &UMODE_OPER),
+  STATS_HANDLER_INIT('y', stats_class, &UMODE_OPER),
+  STATS_HANDLER_INIT('Y', stats_class, &UMODE_OPER),
+  STATS_HANDLER_INIT('z', stats_memory, &UMODE_OPER),
+  STATS_HANDLER_INIT('?', stats_servlinks, &UMODE_OPER)
 };
 
 static void
@@ -1188,8 +1188,8 @@ do_stats(struct Client *client, int parc, char *parv[])
     else
       sendto_one_numeric(client, &me, ERR_NOPRIVILEGES);
 
-    sendto_realops_flags(UMODE_SPY, L_ALL, SEND_NOTICE, "STATS %c requested by %s (%s@%s) [%s]",
-                         letter, client->name, client->username, client->host, client->servptr->name);
+    sendto_clients(UMODE_SPY, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "STATS %c requested by %s (%s@%s) [%s]",
+                   letter, client->name, client->username, client->host, client->servptr->name);
   }
 
   sendto_one_numeric(client, &me, RPL_ENDOFSTATS, letter);
@@ -1207,17 +1207,17 @@ m_stats(struct Client *client, int parc, char *parv[])
   static uintmax_t last_used = 0;
 
   /* Check the user is actually allowed to do /stats, and isn't flooding */
-  if ((last_used + ConfigGeneral.pace_wait) > event_base->time.sec_monotonic)
+  if ((last_used + ConfigGeneral.pace_wait) > io_time_get(IO_TIME_MONOTONIC_SEC))
   {
     sendto_one_numeric(client, &me, RPL_LOAD2HI, "STATS");
     return;
   }
 
-  last_used = event_base->time.sec_monotonic;
+  last_used = io_time_get(IO_TIME_MONOTONIC_SEC);
 
   /* Is the stats meant for us? */
   if (ConfigServerHide.disable_remote_commands == 0)
-    if (server_hunt(client, ":%s STATS %s :%s", 2, parv)->ret != HUNTED_ISME)
+    if (server_route_command(client, ":%s STATS %s :%s", 2, parv)->result != SERVER_ROUTE_ISME)
       return;
 
   do_stats(client, parc, parv);
@@ -1232,7 +1232,7 @@ m_stats(struct Client *client, int parc, char *parv[])
 static void
 ms_stats(struct Client *client, int parc, char *parv[])
 {
-  if (server_hunt(client, ":%s STATS %s :%s", 2, parv)->ret != HUNTED_ISME)
+  if (server_route_command(client, ":%s STATS %s :%s", 2, parv)->result != SERVER_ROUTE_ISME)
     return;
 
   do_stats(client, parc, parv);
