@@ -2,7 +2,7 @@
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
  *  Copyright (c) 1999 Bahamut development team.
- *  Copyright (c) 2013-2024 ircd-hybrid development team
+ *  Copyright (c) 2013-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@
 #include "send.h"
 #include "parse.h"
 #include "module.h"
-#include "irc_string.h"
+#include "io_string.h"
 
 
 /*! \brief SVSKILL command handler
@@ -63,18 +63,15 @@ ms_svskill(struct Client *source, int parc, char *parv[])
   const char *const comment = parv[3];
   if (MyConnect(target))
   {
-    char reason[REASONLEN + 1] = "SVSKilled: ";
-    strlcpy(reason + 11, comment, sizeof(reason) - 11);
-
-    exit_client(target, reason);
+    client_exit_fmt(target, "SVSKilled: %.*s", REASONLEN, comment);
     return;
   }
 
   if (target->from == source->from)
   {
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                         "Received wrong-direction SVSKILL for %s (behind %s) from %s",
-                         target->name, source->from->name, client_get_name(source, HIDE_IP));
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                   "Received wrong-direction SVSKILL for %s (behind %s) from %s",
+                   target->name, source->from->name, client_get_name(source, HIDE_IP));
     return;
   }
 
