@@ -2,7 +2,7 @@
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
  *  Copyright (c) 1999 Bahamut development team.
- *  Copyright (c) 2011-2024 ircd-hybrid development team
+ *  Copyright (c) 2011-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -33,8 +33,9 @@
 #include "send.h"
 #include "parse.h"
 #include "module.h"
-#include "irc_string.h"
+#include "io_string.h"
 #include "user.h"
+#include "user_mode.h"
 #include "hash.h"
 #include "monitor.h"
 #include "whowas.h"
@@ -78,9 +79,9 @@ ms_svsnick(struct Client *source, int parc, char *parv[])
   {
     if (target->from == source->from)
     {
-      sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                           "Received wrong-direction SVSNICK for %s (behind %s) from %s",
-                           target->name, source->from->name, client_get_name(source, HIDE_IP));
+      sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                     "Received wrong-direction SVSNICK for %s (behind %s) from %s",
+                     target->name, source->from->name, client_get_name(source, HIDE_IP));
       return;
     }
 
@@ -98,10 +99,10 @@ ms_svsnick(struct Client *source, int parc, char *parv[])
         return;
     }
     else if (IsUnknown(exists_p))
-      exit_client(exists_p, "SVSNICK Override");
+      client_exit(exists_p, "SVSNICK Override");
     else
     {
-      exit_client(target, "SVSNICK Collide");
+      client_exit(target, "SVSNICK Collide");
       return;
     }
   }
@@ -110,12 +111,11 @@ ms_svsnick(struct Client *source, int parc, char *parv[])
   clear_ban_cache_list(&target->channel);
   monitor_signoff(target);
 
-  if (HasUMode(target, UMODE_REGISTERED))
+  if (user_mode_has_flag(target, UMODE_REGISTERED))
   {
-    const unsigned int oldmodes = target->umodes;
-    DelUMode(target, UMODE_REGISTERED);
-
-    send_umode(target, oldmodes, true, false);
+    const uint64_t mode_flags_old = target->umodes;
+    user_mode_unset_flag(target, UMODE_REGISTERED);
+    user_mode_send(source, mode_flags_old, USER_MODE_SEND_CLIENT);
   }
 
   sendto_common_channels_local(target, true, 0, 0, ":%s!%s@%s NICK :%s",
@@ -123,8 +123,8 @@ ms_svsnick(struct Client *source, int parc, char *parv[])
 
   whowas_add_history(target, true);
 
-  sendto_server(NULL, 0, 0, ":%s NICK %s :%ju",
-                target->id, new_nick, target->tsinfo);
+  sendto_servers(NULL, 0, 0, ":%s NICK %s :%ju",
+                 target->id, new_nick, target->tsinfo);
 
   hash_del_client(target);
   strlcpy(target->name, new_nick, sizeof(target->name));
