@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 2014-2024 ircd-hybrid development team
+ *  Copyright (c) 2014-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 #include "list.h"
 #include "channel.h"
 #include "client.h"
-#include "irc_string.h"
+#include "io_string.h"
 #include "ircd.h"
 #include "send.h"
 #include "parse.h"
@@ -65,13 +65,13 @@ ms_svspart(struct Client *source, int parc, char *parv[])
 
   if (target->from == source->from)
   {
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                         "Received wrong-direction SVSPART for %s (behind %s) from %s",
-                         target->name, source->from->name, client_get_name(source, HIDE_IP));
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                   "Received wrong-direction SVSPART for %s (behind %s) from %s",
+                   target->name, source->from->name, client_get_name(source, HIDE_IP));
     return;
   }
 
-  if (EmptyString(parv[3]))
+  if (string_is_empty(parv[3]))
     sendto_one(target, ":%s SVSPART %s %s",
                source->id, target->id, parv[2]);
   else
