@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,14 +24,16 @@
  */
 
 #include "stdinc.h"
+#include "io_time.h"
 #include "list.h"
 #include "client.h"
 #include "hash.h"
-#include "irc_string.h"
+#include "io_string.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "server.h"
 #include "send.h"
+#include "user_mode.h"
 #include "parse.h"
 #include "module.h"
 #include "conf.h"
@@ -67,29 +69,29 @@ trace_send_status(struct Client *source, const struct Client *target)
   {
     case STAT_CONNECTING:
       sendto_one_numeric(source, &me, RPL_TRACECONNECTING,
-                         class_name, HasUMode(source, UMODE_ADMIN) ? name : target->name);
+                         class_name, user_mode_has_flag(source, UMODE_ADMIN) ? name : target->name);
       break;
     case STAT_HANDSHAKE:
       sendto_one_numeric(source, &me, RPL_TRACEHANDSHAKE,
-                         class_name, HasUMode(source, UMODE_ADMIN) ? name : target->name);
+                         class_name, user_mode_has_flag(source, UMODE_ADMIN) ? name : target->name);
       break;
     case STAT_ME:
       break;
     case STAT_UNKNOWN:
       sendto_one_numeric(source, &me, RPL_TRACEUNKNOWN,
                          class_name, name, target->sockhost,
-                         event_base->time.sec_monotonic - target->connection->created_monotonic);
+                         io_time_get(IO_TIME_MONOTONIC_SEC) - target->connection->created_monotonic);
       break;
     case STAT_CLIENT:
-      if (HasUMode(target, UMODE_OPER))
+      if (user_mode_has_flag(target, UMODE_OPER))
         sendto_one_numeric(source, &me, RPL_TRACEOPERATOR,
                            class_name, name, target->sockhost,
-                           event_base->time.sec_monotonic - target->connection->last_data,
+                           io_time_get(IO_TIME_MONOTONIC_SEC) - target->connection->last_data,
                            client_get_idle_time(source, target));
       else
         sendto_one_numeric(source, &me, RPL_TRACEUSER,
                            class_name, name, target->sockhost,
-                           event_base->time.sec_monotonic - target->connection->last_data,
+                           io_time_get(IO_TIME_MONOTONIC_SEC) - target->connection->last_data,
                            client_get_idle_time(source, target));
       break;
     case STAT_SERVER:
@@ -99,13 +101,13 @@ trace_send_status(struct Client *source, const struct Client *target)
 
       trace_get_dependent(&servers, &clients, target);
 
-      if (!HasUMode(source, UMODE_ADMIN))
+      if (user_mode_has_flag(source, UMODE_ADMIN) == false)
         name = client_get_name(target, MASK_IP);
 
       sendto_one_numeric(source, &me, RPL_TRACESERVER,
                          class_name, servers, clients, name,
-                         *(target->serv->by) ? target->serv->by : "*", "*",
-                         me.name, event_base->time.sec_monotonic - target->connection->last_data);
+                         target->serv->initiator_name ? target->serv->initiator_name : "*", "*",
+                         me.name, io_time_get(IO_TIME_MONOTONIC_SEC) - target->connection->last_data);
       break;
     }
 
@@ -118,13 +120,13 @@ trace_send_status(struct Client *source, const struct Client *target)
 static void
 do_trace(struct Client *source, const char *name)
 {
-  assert(HasUMode(source, UMODE_OPER));
+  assert(user_mode_has_flag(source, UMODE_OPER));
 
-  sendto_realops_flags(UMODE_SPY, L_ALL, SEND_NOTICE, "TRACE requested by %s (%s@%s) [%s]",
-                       source->name, source->username, source->host, source->servptr->name);
+  sendto_clients(UMODE_SPY, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "TRACE requested by %s (%s@%s) [%s]",
+                 source->name, source->username, source->host, source->servptr->name);
 
   bool doall = false;
-  if (EmptyString(name))
+  if (string_is_empty(name))
     doall = true;
   else if (match(name, me.name) == 0)
     doall = true;
@@ -191,17 +193,17 @@ static void
 mo_trace(struct Client *source, int parc, char *parv[])
 {
   if (parc > 2)
-    if (server_hunt(source, ":%s TRACE %s :%s", 2, parv)->ret != HUNTED_ISME)
+    if (server_route_command(source, ":%s TRACE %s :%s", 2, parv)->result != SERVER_ROUTE_ISME)
       return;
 
-  const struct server_hunt *hunt = server_hunt(source, ":%s TRACE :%s", 1, parv);
-  switch (hunt->ret)
+  const server_route_t *route = server_route_command(source, ":%s TRACE :%s", 1, parv);
+  switch (route->result)
   {
-    case HUNTED_PASS:
+    case SERVER_ROUTE_PASS:
       sendto_one_numeric(source, &me, RPL_TRACELINK,
-                         IRCD_VERSION, hunt->target->name, hunt->target->from->name);
+                         IRCD_VERSION, route->target->name, route->target->from->name);
       break;
-    case HUNTED_ISME:
+    case SERVER_ROUTE_ISME:
       do_trace(source, parv[1]);
       break;
     default:
