@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 #include "stdinc.h"
 #include "list.h"
 #include "client.h"
-#include "irc_string.h"
+#include "io_string.h"
 #include "conf.h"
 #include "conf_cluster.h"
 #include "conf_shared.h"
@@ -53,7 +53,7 @@ dline_remove(struct Client *source, const struct aline_ctx *aline)
   struct io_addr iphost, *piphost;
   struct MaskItem *conf;
 
-  if (parse_netmask(aline->host, &iphost, NULL) != HM_HOST)
+  if (address_parse_netmask(aline->host, &iphost, NULL) != HM_HOST)
     piphost = &iphost;
   else
     piphost = NULL;
@@ -77,8 +77,8 @@ dline_remove(struct Client *source, const struct aline_ctx *aline)
     sendto_one_notice(source, &me, ":D-Line for [%s] is removed",
                       conf->host);
 
-  sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE, "%s has removed the D-Line for: [%s]",
-                       get_oper_name(source), conf->host);
+  sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "%s has removed the D-Line for: [%s]",
+                 get_oper_name(source), conf->host);
   log_write(LOG_TYPE_DLINE, "%s removed D-Line for [%s]",
             get_oper_name(source), conf->host);
 
