@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 2001-2024 ircd-hybrid development team
+ *  Copyright (c) 2001-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 #include "stdinc.h"
 #include "list.h"
 #include "client.h"
-#include "irc_string.h"
+#include "io_string.h"
 #include "ircd.h"
 #include "conf.h"
 #include "conf_cluster.h"
@@ -64,8 +64,8 @@ resv_remove(struct Client *source, const struct aline_ctx *aline)
   if (IsClient(source))
     sendto_one_notice(source, &me, ":RESV for [%s] is removed", resv->mask);
 
-  sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE, "%s has removed the RESV for: [%s]",
-                       get_oper_name(source), resv->mask);
+  sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "%s has removed the RESV for: [%s]",
+                 get_oper_name(source), resv->mask);
   log_write(LOG_TYPE_RESV, "%s removed RESV for [%s]",
             get_oper_name(source), resv->mask);
 
