@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,19 +24,18 @@
  */
 
 #include "stdinc.h"
+#include "module.h"
+#include "io_string.h"
 #include "client.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "send.h"
-#include "irc_string.h"
+#include "user_mode.h"
 #include "parse.h"
-#include "module.h"
-
 
 /** See 5.7 Userhost message in RFC1459 */
 enum { RFC1459_MAX_USERHOST_LIST = 5 };
 
-
 /*! \brief USERHOST command handler
  *
  * \param source Pointer to allocated Client struct from which the message
@@ -77,16 +76,16 @@ m_userhost(struct Client *source, int parc, char *parv[])
     if (target == source)
       masklen = snprintf(response, sizeof(response), "%s%s=%c%s@%s",
                          target->name,
-                         HasUMode(target, UMODE_OPER) ? "*" : "",
-                         (target->away[0]) ? '-' : '+',
+                         user_mode_has_flag(target, UMODE_OPER) ? "*" : "",
+                         target->away ? '-' : '+',
                          target->username,
                          target->sockhost);
     else
       masklen = snprintf(response, sizeof(response), "%s%s=%c%s@%s",
-                         target->name, (HasUMode(target, UMODE_OPER) &&
-                                          (!HasUMode(target, UMODE_HIDDEN) ||
-                                            HasUMode(source, UMODE_OPER))) ? "*" : "",
-                         (target->away[0]) ? '-' : '+',
+                         target->name, (user_mode_has_flag(target, UMODE_OPER) &&
+                                        (user_mode_has_flag(target, UMODE_HIDDEN) == false ||
+                                         user_mode_has_flag(source, UMODE_OPER))) ? "*" : "",
+                         target->away ? '-' : '+',
                          target->username,
                          target->host);
 
