@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
  */
 
 #include "stdinc.h"
+#include "io_time.h"
 #include "client.h"
 #include "ircd.h"
 #include "numeric.h"
@@ -40,9 +41,7 @@ static const char serveropts[] =
 {
   'T',
   'S',
-#ifdef TS_CURRENT
-  '0' + TS_CURRENT,
-#endif
+  '0' + SERVER_TS_PROTOCOL_CURRENT,
 /* ONLY do TS */
 /* ALWAYS do TS_WARNINGS */
   'o',
@@ -66,16 +65,16 @@ m_version(struct Client *source, int parc, char *parv[])
 {
   static uintmax_t last_used = 0;
 
-  if ((last_used + ConfigGeneral.pace_wait_simple) > event_base->time.sec_monotonic)
+  if ((last_used + ConfigGeneral.pace_wait_simple) > io_time_get(IO_TIME_MONOTONIC_SEC))
   {
     sendto_one_numeric(source, &me, RPL_LOAD2HI, "VERSION");
     return;
   }
 
-  last_used = event_base->time.sec_monotonic;
+  last_used = io_time_get(IO_TIME_MONOTONIC_SEC);
 
   if (ConfigServerHide.disable_remote_commands == 0)
-    if (server_hunt(source, ":%s VERSION :%s", 1, parv)->ret != HUNTED_ISME)
+    if (server_route_command(source, ":%s VERSION :%s", 1, parv)->result != SERVER_ROUTE_ISME)
       return;
 
   sendto_one_numeric(source, &me, RPL_VERSION, IRCD_VERSION, me.name, serveropts);
@@ -96,7 +95,7 @@ m_version(struct Client *source, int parc, char *parv[])
 static void
 ms_version(struct Client *source, int parc, char *parv[])
 {
-  if (server_hunt(source, ":%s VERSION :%s", 1, parv)->ret != HUNTED_ISME)
+  if (server_route_command(source, ":%s VERSION :%s", 1, parv)->result != SERVER_ROUTE_ISME)
     return;
 
   sendto_one_numeric(source, &me, RPL_VERSION, IRCD_VERSION, me.name, serveropts);
