@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 2012-2024 ircd-hybrid development team
+ *  Copyright (c) 2012-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,11 +28,12 @@
 #include "client.h"
 #include "ircd.h"
 #include "send.h"
-#include "irc_string.h"
+#include "io_string.h"
 #include "parse.h"
 #include "module.h"
 #include "conf.h"
 #include "user.h"
+#include "user_mode.h"
 
 
 /*! \brief WEBIRC command handler
@@ -60,7 +61,7 @@ mr_webirc(struct Client *source, int parc, char *parv[])
 
   if (!valid_hostname(host))
   {
-    exit_client(source, "Invalid WebIRC hostname");
+    client_exit(source, "Invalid WebIRC hostname");
     return;
   }
 
@@ -75,41 +76,28 @@ mr_webirc(struct Client *source, int parc, char *parv[])
 
   if (!IsConfWebIRC(conf))
   {
-    exit_client(source, "Not a WebIRC auth block");
+    client_exit(source, "Not a WebIRC auth block");
     return;
   }
 
-  if (EmptyString(conf->passwd))
+  if (string_is_empty(conf->passwd))
   {
-    exit_client(source, "WebIRC auth blocks must have a password");
+    client_exit(source, "WebIRC auth blocks must have a password");
     return;
   }
 
-  if (match_conf_password(pass, conf) == false)
+  if (conf_match_password(pass, conf) == false)
   {
-    exit_client(source, "Invalid WebIRC password");
+    client_exit(source, "Invalid WebIRC password");
     return;
   }
 
-  struct addrinfo hints, *res;
-  memset(&hints, 0, sizeof(hints));
-
-  hints.ai_family = AF_UNSPEC;
-  hints.ai_socktype = SOCK_STREAM;
-  hints.ai_flags = AI_PASSIVE | AI_NUMERICHOST;
-
-  if (getaddrinfo(addr, NULL, &hints, &res))
+  if (address_from_string(addr, &source->addr) == false)
   {
-    exit_client(source, "Invalid WebIRC IP address");
+    client_exit(source, "Invalid WebIRC IP address");
     return;
   }
 
-  assert(res);
-
-  memcpy(&source->addr, res->ai_addr, res->ai_addrlen);
-  source->addr.ss_len = res->ai_addrlen;
-  freeaddrinfo(res);
-
   strlcpy(source->sockhost, addr, sizeof(source->sockhost));
 
   if (source->sockhost[0] == ':')
@@ -127,12 +115,12 @@ mr_webirc(struct Client *source, int parc, char *parv[])
   {
     if (conf->type == CONF_DLINE)
     {
-      exit_client(source, "D-lined");
+      client_exit(source, "D-lined");
       return;
     }
   }
 
-  AddUMode(source, UMODE_WEBIRC);
+  user_mode_set_flag(source, UMODE_WEBIRC);
   sendto_one_notice(source, &me, ":WebIRC host/IP set to %s %s", host, addr);
 }
 
