@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *  Copyright (c) 1997-1999 Andrea "Nemesi" Cocito
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -25,7 +25,12 @@
  */
 
 #include "stdinc.h"
+#include "address.h"
 #include "list.h"
+#include "module.h"
+#include "io_string.h"
+#include "io_time.h"
+#include "cap.h"
 #include "client.h"
 #include "channel.h"
 #include "channel_mode.h"
@@ -34,12 +39,10 @@
 #include "isupport.h"
 #include "numeric.h"
 #include "send.h"
-#include "irc_string.h"
+#include "user_mode.h"
+#include "ircd_hook.h"
 #include "conf.h"
 #include "parse.h"
-#include "module.h"
-#include "address.h"
-
 
 /** Maximum number of lines to send in response to a /WHO. */
 enum { WHO_MAX_REPLIES = 500 };
@@ -109,7 +112,7 @@ who_send(struct Client *source, const struct Client *target,
       {
         member = node->data;
 
-        if (PubChannel(member->channel) || source == target || member_find_link(source, member->channel))
+        if (channel_is_public(member->channel) || source == target || member_find_link(source, member->channel))
           break;
         member = NULL;
       }
@@ -122,12 +125,7 @@ who_send(struct Client *source, const struct Client *target,
    */
 
   if ((who->fields & WHO_FIELD_QTO))  /* Query token */
-  {
-    if (EmptyString(who->token))
-      p += snprintf(p, sizeof(buf) - (p - buf), " %s", "0");
-    else
-      p += snprintf(p, sizeof(buf) - (p - buf), " %s", who->token);
-  }
+    p += snprintf(p, sizeof(buf) - (p - buf), " %s", string_default(who->token, "0"));
 
   if (who->fields == 0 || (who->fields & WHO_FIELD_CHA))
     p += snprintf(p, sizeof(buf) - (p - buf), " %s", member ? member->channel->name : "*");
@@ -137,7 +135,7 @@ who_send(struct Client *source, const struct Client *target,
 
   if ((who->fields & WHO_FIELD_NIP))
   {
-    if (HasUMode(source, UMODE_OPER) || source == target)
+    if (user_mode_has_flag(source, UMODE_OPER) || source == target)
       p += snprintf(p, sizeof(buf) - (p - buf), " %s", target->sockhost);
     else
       p += snprintf(p, sizeof(buf) - (p - buf), " %s", "255.255.255.255");
@@ -148,7 +146,7 @@ who_send(struct Client *source, const struct Client *target,
 
   if (who->fields == 0 || (who->fields & WHO_FIELD_SER))
   {
-    if (!HasUMode(source, UMODE_OPER) &&
+    if (user_mode_has_flag(source, UMODE_OPER) == false &&
         (ConfigServerHide.hide_servers || IsHidden(target->servptr)))
       p += snprintf(p, sizeof(buf) - (p - buf), " %s", ConfigServerHide.hidden_name);
     else
@@ -160,30 +158,30 @@ who_send(struct Client *source, const struct Client *target,
 
   if (who->fields == 0 || (who->fields & WHO_FIELD_FLA))
   {
-    char status[16];
+    char status[32] = { [0] = target->away ? 'G' : 'H' };
 
-    if (HasUMode(source, UMODE_OPER))
-      snprintf(status, sizeof(status), "%c%s%s%s%s%s", target->away[0] ? 'G' : 'H',
-               HasUMode(target, UMODE_BOT) ? "B" : "",
-               HasUMode(target, UMODE_SECURE) ? "z" : "",
-               HasUMode(target, UMODE_REGISTERED) ? "r" : "",
-               HasUMode(target, UMODE_OPER) ? "*" : "",
-               member ? member_get_prefix(member, who->fields || !!HasCap(source, CAP_MULTI_PREFIX)) : "");
+    ircd_hook_who_send_ctx ctx = { .source = source, .target = target };
+    hook_dispatch(ircd_hook_who_send, &ctx);
+
+    if (ctx.modes_len)
+    {
+      ctx.modes[ctx.modes_len] = '\0';
+      strlcat(status, ctx.modes, sizeof(status));
+    }
+
+    if (member)
+    {
+      const char *prefix = member_get_prefix(member, who->fields || !!HasCap(source, CAP_MULTI_PREFIX));
+      if (!string_is_empty(prefix))
+        strlcat(status, prefix, sizeof(status));
+    }
 
-    else
-      snprintf(status, sizeof(status), "%c%s%s%s%s%s", target->away[0] ? 'G' : 'H',
-               HasUMode(target, UMODE_BOT) ? "B" : "",
-               HasUMode(target, UMODE_SECURE) ? "z" : "",
-               HasUMode(target, UMODE_REGISTERED) ? "r" : "",
-               HasUMode(target, UMODE_OPER) &&
-               !HasUMode(target, UMODE_HIDDEN) ? "*" : "",
-               member ? member_get_prefix(member, who->fields || !!HasCap(source, CAP_MULTI_PREFIX)) : "");
     p += snprintf(p, sizeof(buf) - (p - buf), " %s", status);
   }
 
   if (who->fields == 0 || (who->fields & WHO_FIELD_DIS))
   {
-    if (!HasUMode(source, UMODE_OPER) &&
+    if (user_mode_has_flag(source, UMODE_OPER) == false &&
         (ConfigServerHide.hide_servers || IsHidden(target->servptr)))
       p += snprintf(p, sizeof(buf) - (p - buf), " %s%u", who->fields == 0 ? ":" : "", 0);
     else
@@ -193,7 +191,7 @@ who_send(struct Client *source, const struct Client *target,
   if ((who->fields & WHO_FIELD_IDL))
   {
     if (MyClient(target) &&
-        (HasUMode(source, UMODE_OPER) || target == source))
+        (user_mode_has_flag(source, UMODE_OPER) || target == source))
       p += snprintf(p, sizeof(buf) - (p - buf), " %u", client_get_idle_time(source, target));
     else
       p += snprintf(p, sizeof(buf) - (p - buf), " %u", 0);
@@ -229,8 +227,8 @@ who_matches(struct Client *source, const struct Client *target,
             const char *mask, const struct WhoQuery *who)
 {
   if ((who->bitsel & WHOSELECT_OPER))
-    if (!HasUMode(target, UMODE_OPER) ||
-        (HasUMode(target, UMODE_HIDDEN) && !HasUMode(source, UMODE_OPER)))
+    if (user_mode_has_flag(target, UMODE_OPER) == false ||
+        (user_mode_has_flag(target, UMODE_HIDDEN) && user_mode_has_flag(source, UMODE_OPER) == false))
       return false;
 
   if (mask == NULL)
@@ -246,7 +244,7 @@ who_matches(struct Client *source, const struct Client *target,
   {
     if (match(mask, target->host) == 0)
       return true;
-    if (HasUMode(source, UMODE_OPER) && match(mask, target->realhost) == 0)
+    if (user_mode_has_flag(source, UMODE_OPER) && match(mask, target->realhost) == 0)
       return true;
   }
 
@@ -256,14 +254,14 @@ who_matches(struct Client *source, const struct Client *target,
   if ((who->matchsel & WHO_FIELD_ACC) && match(mask, target->account) == 0)
     return true;
 
-  if ((who->matchsel & WHO_FIELD_NIP) && HasUMode(source, UMODE_OPER))
+  if ((who->matchsel & WHO_FIELD_NIP) && user_mode_has_flag(source, UMODE_OPER))
   {
     struct io_addr addr;
     int bits = 0;
-    const int ret = parse_netmask(mask, &addr, &bits);
+    const int ret = address_parse_netmask(mask, &addr, &bits);
 
     if (ret == HM_IPV4 || ret == HM_IPV6)
-      if (address_compare(&target->addr, &addr, false, false, bits))
+      if (address_match(&target->addr, &addr, false, false, bits))
         return true;
 
     if (match(mask, target->sockhost) == 0)
@@ -271,7 +269,7 @@ who_matches(struct Client *source, const struct Client *target,
   }
 
   if ((who->matchsel & WHO_FIELD_SER))
-    if (HasUMode(source, UMODE_OPER) ||
+    if (user_mode_has_flag(source, UMODE_OPER) ||
         (ConfigServerHide.hide_servers == 0 && !IsHidden(target->servptr)))
       if (match(mask, target->servptr->name) == 0)
         return true;
@@ -296,7 +294,7 @@ who_on_common_channel(struct Client *source, struct Channel *channel,
     struct ChannelMember *member = node->data;
     struct Client *target = member->client;
 
-    if (!HasUMode(target, UMODE_INVISIBLE) || HasFlag(target, FLAGS_MARK))
+    if (user_mode_has_flag(target, UMODE_INVISIBLE) == false || HasFlag(target, FLAGS_MARK))
       continue;
 
     AddFlag(target, FLAGS_MARK);
@@ -322,15 +320,15 @@ who_global(struct Client *source, const char *mask, struct WhoQuery *who)
 {
   static uintmax_t last_used = 0;
 
-  if (!HasUMode(source, UMODE_OPER))
+  if (user_mode_has_flag(source, UMODE_OPER) == false)
   {
-    if ((last_used + ConfigGeneral.pace_wait) > event_base->time.sec_monotonic)
+    if ((last_used + ConfigGeneral.pace_wait) > io_time_get(IO_TIME_MONOTONIC_SEC))
     {
       sendto_one_numeric(source, &me, RPL_LOAD2HI, "WHO");
       return;
     }
 
-    last_used = event_base->time.sec_monotonic;
+    last_used = io_time_get(IO_TIME_MONOTONIC_SEC);
   }
 
   /* First, list all matching invisible clients on common channels */
@@ -348,7 +346,7 @@ who_global(struct Client *source, const char *mask, struct WhoQuery *who)
 
     assert(IsClient(target));
 
-    if (HasUMode(target, UMODE_INVISIBLE))
+    if (user_mode_has_flag(target, UMODE_INVISIBLE))
     {
       DelFlag(target, FLAGS_MARK);
       continue;
@@ -375,9 +373,9 @@ who_on_channel(struct Client *source, struct Channel *channel, const struct WhoQ
 {
   bool is_member = false;
 
-  if (HasUMode(source, UMODE_ADMIN) || member_find_link(source, channel))
+  if (user_mode_has_flag(source, UMODE_ADMIN) || member_find_link(source, channel))
     is_member = true;
-  else if (HasCMode(channel, MODE_SECRET))
+  else if (channel_has_mode(channel, MODE_SECRET))
     return;
 
   list_node_t *node;
@@ -386,11 +384,11 @@ who_on_channel(struct Client *source, struct Channel *channel, const struct WhoQ
     struct ChannelMember *member = node->data;
     struct Client *target = member->client;
 
-    if (is_member || !HasUMode(target, UMODE_INVISIBLE))
+    if (is_member || user_mode_has_flag(target, UMODE_INVISIBLE) == false)
     {
       if ((who->bitsel & WHOSELECT_OPER))
-        if (!HasUMode(target, UMODE_OPER) ||
-            (HasUMode(target, UMODE_HIDDEN) && !HasUMode(source, UMODE_OPER)))
+        if (user_mode_has_flag(target, UMODE_OPER) == false ||
+            (user_mode_has_flag(target, UMODE_HIDDEN) && user_mode_has_flag(source, UMODE_OPER) == false))
           continue;
 
       who_send(source, target, member, who);
@@ -421,7 +419,7 @@ m_who(struct Client *source, int parc, char *parv[])
   char *token = NULL;
   struct WhoQuery w = { .maxmatches = WHO_MAX_REPLIES }, *who = &w;
 
-  if (!EmptyString(options))
+  if (!string_is_empty(options))
   {
     char ch;
     char *p = options;
@@ -566,8 +564,8 @@ m_who(struct Client *source, int parc, char *parv[])
     if (target)
     {
       if (!(who->bitsel & WHOSELECT_OPER) ||
-          (HasUMode(target, UMODE_OPER) &&
-           (!HasUMode(target, UMODE_HIDDEN) || HasUMode(source, UMODE_OPER))))
+          (user_mode_has_flag(target, UMODE_OPER) &&
+           (user_mode_has_flag(target, UMODE_HIDDEN) == false || user_mode_has_flag(source, UMODE_OPER))))
         who_send(source, target, NULL, who);
 
       sendto_one_numeric(source, &me, RPL_ENDOFWHO, mask);
@@ -587,7 +585,7 @@ m_who(struct Client *source, int parc, char *parv[])
 
   if (who->maxmatches == 0)
     sendto_one_numeric(source, &me, ERR_WHOLIMEXCEED, WHO_MAX_REPLIES, "WHO");
-  sendto_one_numeric(source, &me, RPL_ENDOFWHO, EmptyString(mask) ? "*" : mask);
+  sendto_one_numeric(source, &me, RPL_ENDOFWHO, string_default(mask, "*"));
 }
 
 static struct Command command_table =
@@ -604,7 +602,7 @@ static void
 init_handler(void)
 {
   command_add(&command_table);
-  isupport_add("WHOX", NULL, -1);
+  isupport_add("WHOX", NULL);
 }
 
 static void
