@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,20 +25,21 @@
 
 #include "stdinc.h"
 #include "list.h"
+#include "misc.h"
+#include "module.h"
+#include "io_string.h"
+#include "io_time.h"
 #include "whowas.h"
 #include "client.h"
 #include "hash.h"
-#include "irc_string.h"
 #include "ircd.h"
 #include "numeric.h"
-#include "misc.h"
 #include "server.h"
 #include "send.h"
+#include "user_mode.h"
 #include "conf.h"
 #include "conf_service.h"
 #include "parse.h"
-#include "module.h"
-
 
 /** Maximum number of lines to send in response to a /WHOWAS. Only applies to remote clients. */
 enum { WHOWAS_MAX_REPLIES = 20 };
@@ -49,7 +50,7 @@ whowas_send(struct Client *source, const struct Whowas *whowas)
   sendto_one_numeric(source, &me, RPL_WHOWASUSER,
                      whowas->name, whowas->username, whowas->hostname, whowas->realname);
 
-  if (HasUMode(source, UMODE_OPER))
+  if (user_mode_has_flag(source, UMODE_OPER))
     sendto_one_numeric(source, &me, RPL_WHOISACTUALLY,
                        whowas->name, whowas->username, whowas->realhost, whowas->sockhost);
 
@@ -58,7 +59,7 @@ whowas_send(struct Client *source, const struct Whowas *whowas)
                        whowas->name, whowas->account, "was");
 
   bool server_hidden = false;
-  if (!HasUMode(source, UMODE_OPER))
+  if (user_mode_has_flag(source, UMODE_OPER) == false)
   {
     if (whowas->server_hidden || ConfigServerHide.hide_servers)
       server_hidden = true;
@@ -79,24 +80,24 @@ whowas_do(struct Client *source, char *parv[])
 {
   int count = 0, max = -1;
 
-  if (!EmptyString(parv[2]))
+  if (!string_is_empty(parv[2]))
     max = atoi(parv[2]);
 
   if (!MyConnect(source) && (max <= 0 || max > WHOWAS_MAX_REPLIES))
     max = WHOWAS_MAX_REPLIES;
 
-  list_node_t *node;
-  LIST_FOREACH(node, whowas_get_hash(hash_string(parv[1]))->head)
+  const struct WhowasGroup *group = whowas_group_find(parv[1]);
+  if (group)
   {
-    const struct Whowas *whowas = node->data;
-    if (irccmp(parv[1], whowas->name) == 0)
+    list_node_t *node;
+    LIST_FOREACH(node, group->whowas_records.head)
     {
+      const struct Whowas *whowas = node->data;
       whowas_send(source, whowas);
-      ++count;
-    }
 
-    if (max > 0 && count >= max)
-      break;
+      if (++count == max)
+        break;
+    }
   }
 
   if (count == 0)
@@ -123,22 +124,22 @@ m_whowas(struct Client *source, int parc, char *parv[])
 {
   static uintmax_t last_used = 0;
 
-  if (EmptyString(parv[1]))
+  if (string_is_empty(parv[1]))
   {
     sendto_one_numeric(source, &me, ERR_NONICKNAMEGIVEN);
     return;
   }
 
-  if ((last_used + ConfigGeneral.pace_wait) > event_base->time.sec_monotonic)
+  if ((last_used + ConfigGeneral.pace_wait) > io_time_get(IO_TIME_MONOTONIC_SEC))
   {
     sendto_one_numeric(source, &me, RPL_LOAD2HI, "WHOWAS");
     return;
   }
 
-  last_used = event_base->time.sec_monotonic;
+  last_used = io_time_get(IO_TIME_MONOTONIC_SEC);
 
   if (ConfigServerHide.disable_remote_commands == 0)
-    if (server_hunt(source, ":%s WHOWAS %s %s :%s", 3, parv)->ret != HUNTED_ISME)
+    if (server_route_command(source, ":%s WHOWAS %s %s :%s", 3, parv)->result != SERVER_ROUTE_ISME)
       return;
 
   whowas_do(source, parv);
@@ -160,13 +161,13 @@ m_whowas(struct Client *source, int parc, char *parv[])
 static void
 ms_whowas(struct Client *source, int parc, char *parv[])
 {
-  if (EmptyString(parv[1]))
+  if (string_is_empty(parv[1]))
   {
     sendto_one_numeric(source, &me, ERR_NONICKNAMEGIVEN);
     return;
   }
 
-  if (server_hunt(source, ":%s WHOWAS %s %s :%s", 3, parv)->ret != HUNTED_ISME)
+  if (server_route_command(source, ":%s WHOWAS %s %s :%s", 3, parv)->result != SERVER_ROUTE_ISME)
     return;
 
   whowas_do(source, parv);
