@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,25 +24,27 @@
  */
 
 #include "stdinc.h"
-#include "defaults.h"
+#include "address.h"
 #include "list.h"
+#include "memory.h"
+#include "misc.h"
+#include "io_string.h"
+#include "io_time.h"
+#include "defaults.h"
 #include "channel.h"
 #include "channel_invite.h"
 #include "channel_mode.h"
 #include "client.h"
 #include "hash.h"
+#include "cap.h"
 #include "conf.h"
 #include "conf_resv.h"
-#include "address.h"
-#include "irc_string.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "server.h"
 #include "server_capab.h"
 #include "send.h"
-#include "event.h"
-#include "memory.h"
-#include "misc.h"
+#include "user_mode.h"
 #include "extban.h"
 
 /** Doubly linked list containing a list of all channels. */
@@ -74,7 +76,7 @@ channel_add_user(struct Channel *channel, struct Client *client, unsigned int fl
     if (flood_ctrl)
       ++channel->number_joined;
 
-    channel->number_joined -= (event_base->time.sec_monotonic - channel->last_join_time) *
+    channel->number_joined -= (io_time_get(IO_TIME_MONOTONIC_SEC) - channel->last_join_time) *
       (((float)GlobalSetOptions.joinfloodcount) /
        (float)GlobalSetOptions.joinfloodtime);
 
@@ -90,19 +92,18 @@ channel_add_user(struct Channel *channel, struct Client *client, unsigned int fl
       if (channel->sent_join_flood_notice == false)
       {
         channel->sent_join_flood_notice = true;
-        sendto_realops_flags(UMODE_FLOOD, L_ALL, SEND_NOTICE, "Possible Join Flooder %s on %s target: %s",
-                             client_get_name(client, HIDE_IP), client->servptr->name, channel->name);
+        sendto_clients(UMODE_FLOOD, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "Possible Join Flooder %s on %s target: %s",
+                       client_get_name(client, HIDE_IP), client->servptr->name, channel->name);
       }
     }
 
-    channel->last_join_time = event_base->time.sec_monotonic;
+    channel->last_join_time = io_time_get(IO_TIME_MONOTONIC_SEC);
   }
 
   struct ChannelMember *member = io_calloc(sizeof(*member));
   member->client = client;
   member->channel = channel;
   member->flags = flags;
-
   list_add(member, &member->channode, &channel->members);
 
   if (MyConnect(client))
@@ -289,12 +290,12 @@ channel_send_modes(struct Client *client, const struct Channel *channel)
   if (channel->topic_time)
     sendto_one(client, ":%s TBURST %ju %s %ju %s :%s",
                me.id, channel->creation_time, channel->name, channel->topic_time,
-               channel->topic_info, channel->topic);
+               channel->topic_info, string_or_empty(channel->topic));
 
-  if (IsCapable(client, CAPAB_MLOCK))
+  if (capab_has_flag(client, CAPAB_MLOCK))
     sendto_one(client, ":%s MLOCK %ju %s %ju :%s",
                me.id, channel->creation_time, channel->name, channel->mode_lock_time,
-               channel->mode_lock == NULL ? "" : channel->mode_lock);
+               string_or_empty(channel->mode_lock));
 }
 
 /*! \brief Check channel name for invalid characters
@@ -307,7 +308,7 @@ channel_check_name(const char *name, bool local)
 {
   const char *p = name;
 
-  assert(!EmptyString(p));
+  assert(!string_is_empty(p));
 
   if (!IsChanPrefix(*p))
     return false;
@@ -359,13 +360,13 @@ channel_free_mask_list(list_t *list)
 struct Channel *
 channel_make(const char *name)
 {
-  assert(!EmptyString(name));
+  assert(!string_is_empty(name));
 
   struct Channel *channel = io_calloc(sizeof(*channel));
   channel->hnextch = channel;
   /* Doesn't hurt to set it here */
-  channel->creation_time = event_base->time.sec_real;
-  channel->last_join_time = event_base->time.sec_monotonic;
+  channel->creation_time = io_time_get(IO_TIME_REALTIME_SEC);
+  channel->last_join_time = io_time_get(IO_TIME_MONOTONIC_SEC);
 
   /* Cache channel name length to avoid repetitive strlen() calls. */
   channel->name_len = strlcpy(channel->name, name, sizeof(channel->name));
@@ -423,6 +424,8 @@ channel_free(struct Channel *channel)
   assert(channel->invexlist.head == NULL);
   assert(channel->invexlist.tail == NULL);
 
+  io_free(channel->topic);
+  io_free(channel->topic_info);
   io_free(channel->mode_lock);
   io_free(channel);
 }
@@ -434,9 +437,9 @@ channel_free(struct Channel *channel)
 static const char *
 channel_pub_or_secret(const struct Channel *channel)
 {
-  if (HasCMode(channel, MODE_SECRET))
+  if (channel_has_mode(channel, MODE_SECRET))
     return "@";
-  if (HasCMode(channel, MODE_PRIVATE))
+  if (channel_has_mode(channel, MODE_PRIVATE))
     return "*";
   return "=";
 }
@@ -448,17 +451,16 @@ channel_pub_or_secret(const struct Channel *channel)
 void
 channel_send_namereply(struct Client *client, struct Channel *channel)
 {
-  char buf[IRCD_BUFSIZE];
-  char *bufptr = buf;
-  size_t masklen = 0;
   bool is_member = member_find_link(client, channel) != NULL;
   bool multi_prefix = HasCap(client, CAP_MULTI_PREFIX) != 0;
   bool uhnames = HasCap(client, CAP_UHNAMES) != 0;
 
   assert(IsClient(client));
 
-  if (PubChannel(channel) || is_member)
+  if (channel_is_public(channel) || is_member)
   {
+    char buf[IRCD_BUFSIZE];
+    char *bufptr = buf;
     /* :me.name 353 client->name @ channel->name :+nick1 @nick2 %nick3 ...\r\n */
     /* 1       23456            789             01                        2 3  */
     size_t len = strlen(me.name) + strlen(client->name) + channel->name_len + 13;
@@ -466,9 +468,10 @@ channel_send_namereply(struct Client *client, struct Channel *channel)
     list_node_t *node;
     LIST_FOREACH(node, channel->members.head)
     {
+      size_t masklen = 0;
       const struct ChannelMember *member = node->data;
 
-      if (HasUMode(member->client, UMODE_INVISIBLE) && is_member == false)
+      if (user_mode_has_flag(member->client, UMODE_INVISIBLE) && is_member == false)
         continue;
 
       if (uhnames)
@@ -634,7 +637,7 @@ ban_matches(struct Client *client, struct Channel *channel, struct Ban *ban)
         break;
       case HM_IPV6:
       case HM_IPV4:
-        if (address_compare(&client->addr, &ban->addr, false, false, ban->bits))
+        if (address_match(&client->addr, &ban->addr, false, false, ban->bits))
           return true;
         break;
       default:
@@ -703,16 +706,16 @@ is_banned(struct Channel *channel, struct Client *client, struct Extban *extban)
 static int
 can_join(struct Client *client, struct Channel *channel, const char *key)
 {
-  if (HasCMode(channel, MODE_SECUREONLY) && !HasUMode(client, UMODE_SECURE))
+  if (channel_has_mode(channel, MODE_SECUREONLY) && user_mode_has_flag(client, UMODE_SECURE) == false)
     return ERR_SECUREONLYCHAN;
 
-  if (HasCMode(channel, MODE_REGONLY) && !HasUMode(client, UMODE_REGISTERED))
+  if (channel_has_mode(channel, MODE_REGONLY) && user_mode_has_flag(client, UMODE_REGISTERED) == false)
     return ERR_NEEDREGGEDNICK;
 
-  if (HasCMode(channel, MODE_OPERONLY) && !HasUMode(client, UMODE_OPER))
+  if (channel_has_mode(channel, MODE_OPERONLY) && user_mode_has_flag(client, UMODE_OPER) == false)
     return ERR_OPERONLYCHAN;
 
-  if (HasCMode(channel, MODE_INVITEONLY))
+  if (channel_has_mode(channel, MODE_INVITEONLY))
     if (invite_find(channel, client) == NULL)
       if (find_bmask(client, channel, &channel->invexlist, NULL) == false)
         return ERR_INVITEONLYCHAN;
@@ -730,14 +733,6 @@ can_join(struct Client *client, struct Channel *channel, const char *key)
   return 0;
 }
 
-bool
-member_has_flags(const struct ChannelMember *member, const unsigned int flags)
-{
-  if (member && (member->flags & flags))
-    return true;
-  return false;
-}
-
 struct ChannelMember *
 member_find_link(const struct Client *client, const struct Channel *channel)
 {
@@ -803,67 +798,73 @@ msg_has_ctrls(const char *message)
  *         CAN_SEND_NONOP if can send to channel but is not an op\n
  *         CAN_SEND_NO if they cannot send to channel\n
  */
-int
-can_send(struct Channel *channel, struct Client *client, struct ChannelMember *member,
-         const char *message, bool notice, const char **error)
+channel_send_perm_t
+channel_send_qualifies(struct Channel *channel, struct Client *client, struct ChannelMember *member,
+                       unsigned int statusmsg, const char *message, bool notice, const char **error)
 {
   if (IsServer(client) || HasFlag(client, FLAGS_SERVICE))
-    return CAN_SEND_OPV;
+    return CHANNEL_SEND_PERM_ELEVATED;
 
   if (MyConnect(client) && !HasFlag(client, FLAGS_EXEMPTRESV))
   {
-    if (!(HasUMode(client, UMODE_OPER) && HasOFlag(client, OPER_FLAG_JOIN_RESV)))
+    if (!(user_mode_has_flag(client, UMODE_OPER) && HasOFlag(client, OPER_FLAG_JOIN_RESV)))
     {
       const struct ResvItem *const resv = resv_find(channel->name, match);
       if (resv && resv_exempt_find(client, resv) == false)
       {
         *error = "channel is reserved";
-        return CAN_SEND_NO;
+        return CHANNEL_SEND_PERM_FORBIDDEN;
       }
     }
   }
 
-  if (HasCMode(channel, MODE_NOCTRL) && msg_has_ctrls(message))
+  if (channel_has_mode(channel, MODE_NOCTRL) && msg_has_ctrls(message))
   {
     *error = "control codes are not permitted";
-    return CAN_SEND_NO;
+    return CHANNEL_SEND_PERM_FORBIDDEN;
   }
 
-  if (HasCMode(channel, MODE_NOCTCP))
+  if (channel_has_mode(channel, MODE_NOCTCP))
   {
     if (*message == '\001' && strncmp(message + 1, "ACTION ", 7))
     {
       *error = "CTCPs are not permitted";
-      return CAN_SEND_NO;
+      return CHANNEL_SEND_PERM_FORBIDDEN;
     }
   }
 
   if (member || (member = member_find_link(client, channel)))
     if (member_highest_rank(member) > CHACCESS_PEON)
-      return CAN_SEND_OPV;
+      return CHANNEL_SEND_PERM_ELEVATED;
 
-  if (member == NULL && HasCMode(channel, MODE_NOPRIVMSGS))
+  if (statusmsg)
+  {
+    *error = "STATUSMSG requires elevated privileges";
+    return CHANNEL_SEND_PERM_FORBIDDEN;
+  }
+
+  if (member == NULL && channel_has_mode(channel, MODE_NOPRIVMSGS))
   {
     *error = "external messages are not permitted";
-    return CAN_SEND_NO;
+    return CHANNEL_SEND_PERM_FORBIDDEN;
   }
 
-  if (HasCMode(channel, MODE_MODERATED))
+  if (channel_has_mode(channel, MODE_MODERATED))
   {
     *error = "channel is moderated (+m)";
-    return CAN_SEND_NO;
+    return CHANNEL_SEND_PERM_FORBIDDEN;
   }
 
-  if (HasCMode(channel, MODE_MODREG) && !HasUMode(client, UMODE_REGISTERED))
+  if (channel_has_mode(channel, MODE_MODREG) && user_mode_has_flag(client, UMODE_REGISTERED) == false)
   {
     *error = "you need to identify to a registered nick";
-    return CAN_SEND_NO;
+    return CHANNEL_SEND_PERM_FORBIDDEN;
   }
 
-  if (HasCMode(channel, MODE_NONOTICE) && notice)
+  if (channel_has_mode(channel, MODE_NONOTICE) && notice)
   {
     *error = "NOTICEs are not permitted";
-    return CAN_SEND_NO;
+    return CHANNEL_SEND_PERM_FORBIDDEN;
   }
 
   *error = "you are banned (+b)";
@@ -873,24 +874,24 @@ can_send(struct Channel *channel, struct Client *client, struct ChannelMember *m
     if (member)
     {
       if (member->flags & CHFL_BAN_SILENCED)
-        return CAN_SEND_NO;
+        return CHANNEL_SEND_PERM_FORBIDDEN;
 
       if (!(member->flags & CHFL_BAN_CHECKED))
       {
         if (is_banned(channel, client, NULL) || is_banned(channel, client, &extban_mute))
         {
           member->flags |= (CHFL_BAN_CHECKED | CHFL_BAN_SILENCED);
-          return CAN_SEND_NO;
+          return CHANNEL_SEND_PERM_FORBIDDEN;
         }
 
         member->flags |= CHFL_BAN_CHECKED;
       }
     }
     else if (is_banned(channel, client, NULL) || is_banned(channel, client, &extban_mute))
-      return CAN_SEND_NO;
+      return CHANNEL_SEND_PERM_FORBIDDEN;
   }
 
-  return CAN_SEND_NONOP;
+  return CHANNEL_SEND_PERM_STANDARD;
 }
 
 /*! \brief Updates the client's oper_warn_count_down, warns the
@@ -913,14 +914,14 @@ channel_check_spambot_warning(struct Client *client, const char *name)
       client->connection->oper_warn_count_down = OPER_SPAM_COUNTDOWN;
 
       /* It's already known as a possible spambot */
-      sendto_realops_flags(UMODE_FLOOD, L_ALL, SEND_NOTICE,
-                           "User %s (%s@%s) trying to join %s is a possible spambot",
-                           client->name, client->username, client->host, name);
+      sendto_clients(UMODE_FLOOD, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                     "User %s (%s@%s) trying to join %s is a possible spambot",
+                     client->name, client->username, client->host, name);
     }
   }
   else
   {
-    unsigned int t_delta = event_base->time.sec_monotonic - client->connection->last_leave_time;
+    unsigned int t_delta = io_time_get(IO_TIME_MONOTONIC_SEC) - client->connection->last_leave_time;
     if (t_delta > JOIN_LEAVE_COUNT_EXPIRE_TIME)
     {
       unsigned int decrement_count = (t_delta / JOIN_LEAVE_COUNT_EXPIRE_TIME);
@@ -931,14 +932,14 @@ channel_check_spambot_warning(struct Client *client, const char *name)
     }
     else
     {
-      if ((event_base->time.sec_monotonic - client->connection->last_join_time) < GlobalSetOptions.spam_time)
+      if ((io_time_get(IO_TIME_MONOTONIC_SEC) - client->connection->last_join_time) < GlobalSetOptions.spam_time)
         ++client->connection->join_leave_count;  /* It's a possible spambot */
     }
 
     if (name)
-      client->connection->last_join_time = event_base->time.sec_monotonic;
+      client->connection->last_join_time = io_time_get(IO_TIME_MONOTONIC_SEC);
     else
-      client->connection->last_leave_time = event_base->time.sec_monotonic;
+      client->connection->last_leave_time = io_time_get(IO_TIME_MONOTONIC_SEC);
   }
 }
 
@@ -952,12 +953,20 @@ channel_check_spambot_warning(struct Client *client, const char *name)
 void
 channel_set_topic(struct Channel *channel, const char *topic, const char *topic_info, uintmax_t topicts, bool local)
 {
-  if (local)
-    strlcpy(channel->topic, topic, IO_MIN(sizeof(channel->topic), ConfigServerInfo.max_topic_length + 1));
-  else
-    strlcpy(channel->topic, topic, sizeof(channel->topic));
+  io_free(channel->topic);
+  channel->topic = NULL;
+  io_free(channel->topic_info);
+  channel->topic_info = NULL;
+
+  if (!string_is_empty(topic))
+  {
+    size_t max_length = local ? ConfigServerInfo.max_topic_length : TOPICLEN;
+    channel->topic = io_strndup(topic, max_length);
+  }
+
+  if (!string_is_empty(topic_info))
+    channel->topic_info = io_strdup(topic_info);
 
-  strlcpy(channel->topic_info, topic_info, sizeof(channel->topic_info));
   channel->topic_time = topicts;
 }
 
@@ -972,7 +981,7 @@ channel_set_mode_lock(struct Client *client, struct Channel *channel, const char
   io_free(channel->mode_lock);
   channel->mode_lock = NULL;
 
-  if (!EmptyString(mode_lock))
+  if (!string_is_empty(mode_lock))
     channel->mode_lock = io_strdup(mode_lock);
 }
 
@@ -992,7 +1001,7 @@ channel_join_list(struct Client *client, char *chan_list, char *key_list)
     unsigned int flags = 0;
 
     /* If we have any more keys, take the first for this channel. */
-    if (!EmptyString(key_list) && (key_list = strchr(key = key_list, ',')))
+    if (!string_is_empty(key_list) && (key_list = strchr(key = key_list, ',')))
       *key_list++ = '\0';
 
     /* Empty keys are the same as no keys. */
@@ -1006,12 +1015,12 @@ channel_join_list(struct Client *client, char *chan_list, char *key_list)
     }
 
     if (!HasFlag(client, FLAGS_EXEMPTRESV) &&
-        !(HasUMode(client, UMODE_OPER) && HasOFlag(client, OPER_FLAG_JOIN_RESV)) &&
+        !(user_mode_has_flag(client, UMODE_OPER) && HasOFlag(client, OPER_FLAG_JOIN_RESV)) &&
         ((resv = resv_find(name, match)) && resv_exempt_find(client, resv) == false))
     {
       sendto_one_numeric(client, &me, ERR_CHANBANREASON, name, resv->reason);
-      sendto_realops_flags(UMODE_REJ, L_ALL, SEND_NOTICE, "Forbidding reserved channel %s from user %s",
-                           name, client_get_name(client, HIDE_IP));
+      sendto_clients(UMODE_REJ, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "Forbidding reserved channel %s from user %s",
+                     name, client_get_name(client, HIDE_IP));
       continue;
     }
 
@@ -1042,21 +1051,21 @@ channel_join_list(struct Client *client, char *chan_list, char *key_list)
       channel = channel_make(name);
     }
 
-    if (!HasUMode(client, UMODE_OPER))
+    if (user_mode_has_flag(client, UMODE_OPER) == false)
       channel_check_spambot_warning(client, channel->name);
 
     channel_add_user(channel, client, flags, true);
+    client->connection->last_join_time = io_time_get(IO_TIME_MONOTONIC_SEC);
 
     /*
      * Set channel modes if appropriate, and propagate
      */
     if (flags == CHFL_CHANOP)
     {
-      AddCMode(channel, MODE_TOPICLIMIT);
-      AddCMode(channel, MODE_NOPRIVMSGS);
+      channel_set_mode(channel, MODE_TOPICLIMIT | MODE_NOPRIVMSGS);
 
-      sendto_server(NULL, 0, 0, ":%s SJOIN %ju %s +nt :@%s",
-                    me.id, channel->creation_time, channel->name, client->id);
+      sendto_servers(NULL, 0, 0, ":%s SJOIN %ju %s +nt :@%s",
+                     me.id, channel->creation_time, channel->name, client->id);
 
       /*
        * Notify all other users on the new channel
@@ -1070,8 +1079,8 @@ channel_join_list(struct Client *client, char *chan_list, char *key_list)
     }
     else
     {
-      sendto_server(NULL, 0, 0, ":%s JOIN %ju %s +",
-                    client->id, channel->creation_time, channel->name);
+      sendto_servers(NULL, 0, 0, ":%s JOIN %ju %s +",
+                     client->id, channel->creation_time, channel->name);
 
       sendto_channel_local(NULL, channel, 0, CAP_EXTENDED_JOIN, 0, ":%s!%s@%s JOIN %s %s :%s",
                            client->name, client->username, client->host, channel->name, client->account, client->info);
@@ -1079,7 +1088,7 @@ channel_join_list(struct Client *client, char *chan_list, char *key_list)
                            client->name, client->username, client->host, channel->name);
     }
 
-    if (client->away[0])
+    if (client->away)
       sendto_channel_local(client, channel, 0, CAP_AWAY_NOTIFY, 0, ":%s!%s@%s AWAY :%s",
                            client->name, client->username, client->host, client->away);
 
@@ -1087,7 +1096,7 @@ channel_join_list(struct Client *client, char *chan_list, char *key_list)
     if (invite)
       invite_del(invite);
 
-    if (channel->topic[0])
+    if (channel->topic)
     {
       sendto_one_numeric(client, &me, RPL_TOPIC, channel->name, channel->topic);
       sendto_one_numeric(client, &me, RPL_TOPICWHOTIME,
@@ -1095,8 +1104,6 @@ channel_join_list(struct Client *client, char *chan_list, char *key_list)
     }
 
     channel_send_namereply(client, channel);
-
-    client->connection->last_join_time = event_base->time.sec_monotonic;
   }
 }
 
@@ -1122,36 +1129,36 @@ channel_part_one(struct Client *client, const char *name, const char *reason)
     return;
   }
 
-  if (MyConnect(client) && !HasUMode(client, UMODE_OPER))
+  if (MyConnect(client) && user_mode_has_flag(client, UMODE_OPER) == false)
     channel_check_spambot_warning(client, NULL);
 
   /*
    * Remove user from the old channel (if any). Only allow /part reasons in -m chans.
    */
   bool show_reason = true;
-  if (EmptyString(reason))
+  if (string_is_empty(reason))
     show_reason = false;
   else if (MyConnect(client))
   {
     const char *error;
     if ((client->connection->created_monotonic + ConfigGeneral.anti_spam_exit_message_time)
-          >= event_base->time.sec_monotonic)
+          >= io_time_get(IO_TIME_MONOTONIC_SEC))
       show_reason = false;
-    else if (can_send(channel, client, member, reason, false, &error) == CAN_SEND_NO)
+    else if (channel_send_qualifies(channel, client, member, 0, reason, false, &error) == CHANNEL_SEND_PERM_FORBIDDEN)
       show_reason = false;
   }
 
   if (show_reason)
   {
-    sendto_server(client, 0, 0, ":%s PART %s :%.*s",
-                  client->id, channel->name, KICKLEN, reason);
+    sendto_servers(client, 0, 0, ":%s PART %s :%.*s",
+                   client->id, channel->name, KICKLEN, reason);
     sendto_channel_local(NULL, channel, 0, 0, 0, ":%s!%s@%s PART %s :%.*s",
                          client->name, client->username, client->host, channel->name, KICKLEN, reason);
   }
   else
   {
-    sendto_server(client, 0, 0, ":%s PART %s",
-                  client->id, channel->name);
+    sendto_servers(client, 0, 0, ":%s PART %s",
+                   client->id, channel->name);
     sendto_channel_local(NULL, channel, 0, 0, 0, ":%s!%s@%s PART %s",
                          client->name, client->username, client->host, channel->name);
   }
