@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,18 +24,21 @@
  */
 
 #include "stdinc.h"
+#include "address.h"
 #include "list.h"
+#include "memory.h"
+#include "io_string.h"
+#include "io_time.h"
 #include "channel.h"
 #include "channel_mode.h"
 #include "client.h"
 #include "conf.h"
-#include "address.h"
-#include "irc_string.h"
 #include "ircd.h"
+#include "nuh.h"
 #include "numeric.h"
 #include "server.h"
 #include "send.h"
-#include "memory.h"
+#include "user_mode.h"
 #include "parse.h"
 #include "extban.h"
 #include "isupport.h"
@@ -76,7 +79,7 @@ check_string(char *s)
     }
   }
 
-  if (EmptyString(str))
+  if (string_is_empty(str))
     strcpy(str, "*");
 }
 
@@ -120,8 +123,7 @@ add_id(struct Client *client, struct Channel *channel, const char *banid, list_t
                             list_length(&channel->invexlist);
 
     /* Don't let local clients overflow the b/e/I lists */
-    if (num_mask >= ((HasCMode(channel, MODE_EXTLIMIT)) ? ConfigChannel.max_bans_large :
-                                                          ConfigChannel.max_bans))
+    if (num_mask >= ((channel_has_mode(channel, MODE_EXTLIMIT)) ? ConfigChannel.max_bans_large : ConfigChannel.max_bans))
     {
       sendto_one_numeric(client, &me, ERR_BANLISTFULL, channel->name, banid);
       return NULL;
@@ -171,12 +173,12 @@ add_id(struct Client *client, struct Channel *channel, const char *banid, list_t
   }
 
   /* Don't allow empty bans */
-  if (EmptyString(maskptr))
+  if (string_is_empty(maskptr))
     return NULL;
 
   struct Ban *ban = io_calloc(sizeof(*ban));
   ban->extban = extbans;
-  ban->when = event_base->time.sec_real;
+  ban->when = io_time_get(IO_TIME_REALTIME_SEC);
 
   check_string(maskptr);
 
@@ -185,7 +187,7 @@ add_id(struct Client *client, struct Channel *channel, const char *banid, list_t
     strlcpy(ban->host, maskptr, sizeof(ban->host));
   else
   {
-    struct split_nuh_item nuh =
+    struct nuh_split nuh =
     {
       .nuhmask = maskptr,
       .nickptr = ban->name,
@@ -196,9 +198,9 @@ add_id(struct Client *client, struct Channel *channel, const char *banid, list_t
       .hostsize = sizeof(ban->host)
     };
 
-    split_nuh(&nuh);
+    nuh_split(&nuh);
 
-    ban->type = parse_netmask(ban->host, &ban->addr, &ban->bits);
+    ban->type = address_parse_netmask(ban->host, &ban->addr, &ban->bits);
   }
 
   if (MyClient(client))
@@ -287,7 +289,7 @@ channel_modes(const struct Channel *channel, const struct Client *client, bool p
   *bufptr++ = '+';
 
   for (const struct chan_mode *tab = cmode_tab; tab->letter; ++tab)
-    if (tab->mode && HasCMode(channel, tab->mode))
+    if (tab->mode && channel_has_mode(channel, tab->mode))
       *bufptr++ = tab->letter;
 
   if (channel->mode.limit)
@@ -389,7 +391,7 @@ channel_mode_can_change(struct Client *client, struct Channel *channel, int *err
 
   if (mode->only_opers)
   {
-    if (!HasUMode(client, UMODE_OPER))
+    if (user_mode_has_flag(client, UMODE_OPER) == false)
     {
       if (!(*errors & SM_ERR_NOTOPER))
         sendto_one_numeric(client, &me, ERR_NOPRIVILEGES);
@@ -461,17 +463,17 @@ chm_simple(struct Client *client, struct Channel *channel, int parc, int *parn,
 
   if (dir == MODE_ADD)  /* setting + */
   {
-    if (MyClient(client) && HasCMode(channel, mode->mode))
+    if (MyClient(client) && channel_has_mode(channel, mode->mode))
       return;
 
-    AddCMode(channel, mode->mode);
+    channel_set_mode(channel, mode->mode);
   }
   else if (dir == MODE_DEL)  /* setting - */
   {
-    if (MyClient(client) && !HasCMode(channel, mode->mode))
+    if (MyClient(client) && !channel_has_mode(channel, mode->mode))
       return;
 
-    DelCMode(channel, mode->mode);
+    channel_unset_mode(channel, mode->mode);
   }
 
   mode_changes[mode_count].letter = mode->letter;
@@ -616,14 +618,14 @@ chm_flag(struct Client *client, struct Channel *channel, int parc, int *parn, ch
     if (member_has_flags(member, mode->flag))
       return;  /* No redundant mode changes */
 
-    AddMemberFlag(member, mode->flag);
+    member_set_flags(member, mode->flag);
   }
   else if (dir == MODE_DEL)  /* setting - */
   {
     if (member_has_flags(member, mode->flag) == false)
       return;  /* No redundant mode changes */
 
-    DelMemberFlag(member, mode->flag);
+    member_unset_flags(member, mode->flag);
   }
 
   mode_changes[mode_count].letter = mode->letter;
@@ -644,7 +646,7 @@ chm_limit(struct Client *client, struct Channel *channel, int parc, int *parn, c
     char *const lstr = parv[(*parn)++];
     int limit = 0;
 
-    if (EmptyString(lstr) || (limit = atoi(lstr)) <= 0)
+    if (string_is_empty(lstr) || (limit = atoi(lstr)) <= 0)
       return;
 
     sprintf(lstr, "%d", limit);
@@ -686,7 +688,7 @@ chm_key(struct Client *client, struct Channel *channel, int parc, int *parn, cha
   {
     char *const key = fix_key(parv[(*parn)++]);
 
-    if (EmptyString(key))
+    if (string_is_empty(key))
       return;
 
     assert(*key != ' ');
@@ -762,7 +764,7 @@ send_mode_changes_server(struct Client *client, struct Channel *channel)
     if ((paracount == MAXMODEPARAMS) || ((arglen + mbl + pbl + 2 /* +2 for /r/n */ ) > sizeof(modebuf)))
     {
       if (modecount)
-        sendto_server(client, 0, 0, paracount == 0 ? "%s" : "%s %s", modebuf, parabuf);
+        sendto_servers(client, 0, 0, paracount == 0 ? "%s" : "%s %s", modebuf, parabuf);
 
       modecount = 0;
       paracount = 0;
@@ -778,7 +780,7 @@ send_mode_changes_server(struct Client *client, struct Channel *channel)
 
     if (dir != mode_changes[i].dir)
     {
-      modebuf[mbl++] = (mode_changes[i].dir == MODE_ADD) ? '+' : '-';
+      modebuf[mbl++] = mode_changes[i].dir == MODE_ADD ? '+' : '-';
       dir = mode_changes[i].dir;
     }
 
@@ -788,7 +790,7 @@ send_mode_changes_server(struct Client *client, struct Channel *channel)
 
     if (arg)
     {
-      int len = sprintf(parptr, (pbl == 0) ? "%s" : " %s", arg);
+      int len = sprintf(parptr, pbl == 0 ? "%s" : " %s", arg);
       pbl += len;
       parptr += len;
       ++paracount;
@@ -796,7 +798,7 @@ send_mode_changes_server(struct Client *client, struct Channel *channel)
   }
 
   if (modecount)
-    sendto_server(client, 0, 0, paracount == 0 ? "%s" : "%s %s", modebuf, parabuf);
+    sendto_servers(client, 0, 0, paracount == 0 ? "%s" : "%s %s", modebuf, parabuf);
 }
 
 /* void send_mode_changes(struct Client *client,
@@ -859,7 +861,7 @@ send_mode_changes_client(struct Client *client, struct Channel *channel)
 
     if (dir != mode_changes[i].dir)
     {
-      modebuf[mbl++] = (mode_changes[i].dir == MODE_ADD) ? '+' : '-';
+      modebuf[mbl++] = mode_changes[i].dir == MODE_ADD ? '+' : '-';
       dir = mode_changes[i].dir;
     }
 
@@ -869,7 +871,7 @@ send_mode_changes_client(struct Client *client, struct Channel *channel)
 
     if (arg)
     {
-      int len = sprintf(parptr, (pbl == 0) ? "%s" : " %s", arg);
+      int len = sprintf(parptr, pbl == 0 ? "%s" : " %s", arg);
       pbl += len;
       parptr += len;
       ++paracount;
@@ -970,16 +972,15 @@ channel_mode_init(void)
       strlcat(cmode_class[cmode->class], str, sizeof(cmode_class[cmode->class]));
   }
 
-  char buf[IRCD_BUFSIZE];
-  snprintf(buf, sizeof(buf), "%s,%s,%s,%s",
-           cmode_class[MODE_CLASS_A], cmode_class[MODE_CLASS_B],
-           cmode_class[MODE_CLASS_C], cmode_class[MODE_CLASS_D]);
-  isupport_add("CHANMODES", buf, -1);
+  isupport_add("CHANMODES", "%s,%s,%s,%s",
+               cmode_class[MODE_CLASS_A], cmode_class[MODE_CLASS_B],
+               cmode_class[MODE_CLASS_C], cmode_class[MODE_CLASS_D]);
 
-
-  snprintf(buf, sizeof(buf), "(%s)%s", letter, prefix);
-  isupport_add("PREFIX", buf, -1);
-  isupport_add("STATUSMSG", prefix, -1);
+  isupport_add("PREFIX", "(%s)%s", letter, prefix);
+  isupport_add("STATUSMSG", "%s", prefix);
+  isupport_add("MODES", "%d", MAXMODEPARAMS);
+  isupport_add("EXCEPTS", NULL);
+  isupport_add("INVEX", NULL);
 
   capab_add("QOP", CAPAB_QOP, ConfigChannel.enable_owner != 0);
   capab_add("AOP", CAPAB_AOP, ConfigChannel.enable_admin != 0);
