@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,34 +24,65 @@
  */
 
 #include "stdinc.h"
+#include "comm.h"
+#include "event.h"
 #include "list.h"
+#include "log.h"
+#include "memory.h"
+#include "misc.h"
+#include "rng_mt.h"
+#include "io_string.h"
+#include "io_time.h"
 #include "client.h"
 #include "client_svstag.h"
-#include "event.h"
 #include "hash.h"
-#include "irc_string.h"
 #include "ircd.h"
+#include "ircd_hook.h"
 #include "numeric.h"
-#include "auth.h"
-#include "comm.h"
+#include "lookup.h"
 #include "conf.h"
 #include "conf_gecos.h"
-#include "log.h"
-#include "misc.h"
 #include "server.h"
 #include "send.h"
 #include "whowas.h"
-#include "user.h"
-#include "memory.h"
+#include "user_mode.h"
 #include "listener.h"
 #include "monitor.h"
-#include "rng_mt.h"
 #include "parse.h"
 #include "ipcache.h"
 #include "channel.h"
 #include "channel_invite.h"
 #include "accept.h"
 
+uint64_t UMODE_BOT;
+uint64_t UMODE_DEAF;
+uint64_t UMODE_FARCONNECT;
+uint64_t UMODE_SOFTCALLERID;
+uint64_t UMODE_HIDDEN;
+uint64_t UMODE_EXPIRATION;
+uint64_t UMODE_REGONLY;
+uint64_t UMODE_SECURE;
+uint64_t UMODE_WEBIRC;
+uint64_t UMODE_SECUREONLY;
+uint64_t UMODE_ADMIN;
+uint64_t UMODE_CCONN;
+uint64_t UMODE_EXTERNAL;
+uint64_t UMODE_FLOOD;
+uint64_t UMODE_CALLERID;
+uint64_t UMODE_INVISIBLE;
+uint64_t UMODE_REJ;
+uint64_t UMODE_SKILL;
+uint64_t UMODE_LOCOPS;
+uint64_t UMODE_NCHANGE;
+uint64_t UMODE_OPER;
+uint64_t UMODE_HIDECHANS;
+uint64_t UMODE_HIDEIDLE;
+uint64_t UMODE_REGISTERED;
+uint64_t UMODE_SERVNOTICE;
+uint64_t UMODE_WALLOP;
+uint64_t UMODE_CLOAK;
+uint64_t UMODE_SPY;
+
 list_t listing_client_list;
 list_t unknown_list;
 list_t local_client_list;
@@ -87,10 +118,10 @@ client_make(struct Client *from)
   {
     client->from = client;  /* 'from' of local client is self! */
     client->connection = io_calloc(sizeof(*client->connection));
-    client->connection->last_data = event_base->time.sec_monotonic;
-    client->connection->last_ping = event_base->time.sec_monotonic;
-    client->connection->created_real = event_base->time.sec_real;
-    client->connection->created_monotonic = event_base->time.sec_monotonic;
+    client->connection->last_data = \
+    client->connection->last_ping = \
+    client->connection->created_monotonic = io_time_get(IO_TIME_MONOTONIC_SEC);
+    client->connection->created_real = io_time_get(IO_TIME_REALTIME_SEC);
     client->connection->registration = REG_INIT;
 
     /* as good a place as any... */
@@ -139,10 +170,12 @@ client_free(struct Client *client)
   assert(client->svstags.head == NULL);
   assert(client->svstags.tail == NULL);
 
-
+  if (client->serv)
+    io_free(client->serv->initiator_name);
   io_free(client->serv);
   io_free(client->tls_certfp);
   io_free(client->tls_cipher);
+  io_free(client->away);
 
   if (MyConnect(client))
   {
@@ -150,7 +183,7 @@ client_free(struct Client *client)
     assert(client->connection->node.next == NULL);
 
     assert(client->connection->list_task == NULL);
-    assert(client->connection->auth == NULL);
+    assert(client->connection->lookup == NULL);
 
     assert(list_length(&client->connection->acceptlist) == 0);
     assert(client->connection->acceptlist.head == NULL);
@@ -201,7 +234,6 @@ client_free(struct Client *client)
 static void
 check_pings_list(list_t *list)
 {
-  char buf[32];  /* 32 = sizeof("Ping timeout: 999999999 seconds") */
   list_node_t *node, *node_next;
 
   LIST_FOREACH_SAFE(node, node_next, list->head)
@@ -213,7 +245,7 @@ check_pings_list(list_t *list)
       continue;  /* Ignore it, it's been exited already */
 
     unsigned int ping = class_get_ping_freq(&client->connection->confs);
-    if (ping < event_base->time.sec_monotonic - client->connection->last_ping)
+    if (ping < io_time_get(IO_TIME_MONOTONIC_SEC) - client->connection->last_ping)
     {
       if (!HasFlag(client, FLAGS_PINGSENT))
       {
@@ -223,12 +255,12 @@ check_pings_list(list_t *list)
          * it is still alive.
          */
         AddFlag(client, FLAGS_PINGSENT);
-        client->connection->last_ping = event_base->time.sec_monotonic - ping;
+        client->connection->last_ping = io_time_get(IO_TIME_MONOTONIC_SEC) - ping;
         sendto_one(client, "PING :%s", ID_or_name(&me, client));
       }
       else
       {
-        if (event_base->time.sec_monotonic - client->connection->last_ping >= 2 * ping)
+        if (io_time_get(IO_TIME_MONOTONIC_SEC) - client->connection->last_ping >= 2 * ping)
         {
           /*
            * If the client/server hasn't talked to us in 2*ping seconds
@@ -236,19 +268,18 @@ check_pings_list(list_t *list)
            */
           if (IsServer(client))
           {
-            sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE,
-                                 "No response from %s, closing link",
-                                 client_get_name(client, SHOW_IP));
-            sendto_realops_flags(UMODE_SERVNOTICE, L_OPER, SEND_NOTICE,
-                                 "No response from %s, closing link",
-                                 client_get_name(client, MASK_IP));
+            sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE,
+                           "No response from %s, closing link",
+                           client_get_name(client, SHOW_IP));
+            sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER, SEND_TYPE_NOTICE,
+                           "No response from %s, closing link",
+                           client_get_name(client, MASK_IP));
             log_write(LOG_TYPE_IRCD, "No response from %s, closing link",
                       client_get_name(client, SHOW_IP));
           }
 
-          snprintf(buf, sizeof(buf), "Ping timeout: %ju seconds",
-                   (event_base->time.sec_monotonic - client->connection->last_ping));
-          exit_client(client, buf);
+          client_exit_fmt(client, "Ping timeout: %ju seconds",
+                          (io_time_get(IO_TIME_MONOTONIC_SEC) - client->connection->last_ping));
         }
       }
     }
@@ -275,26 +306,26 @@ check_unknowns_list(void)
      * Check UNKNOWN connections - if they have been in this state
      * for > 30s, close them.
      */
-    if ((event_base->time.sec_monotonic - client->connection->created_monotonic) <= 30)
+    if ((io_time_get(IO_TIME_MONOTONIC_SEC) - client->connection->created_monotonic) <= 30)
       continue;
 
     if (IsHandshake(client))
     {
-      sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE,
-                           "No response from %s during handshake, closing link",
-                           client_get_name(client, SHOW_IP));
-      sendto_realops_flags(UMODE_SERVNOTICE, L_OPER, SEND_NOTICE,
-                           "No response from %s during handshake, closing link",
-                           client_get_name(client, MASK_IP));
+      sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE,
+                     "No response from %s during handshake, closing link",
+                     client_get_name(client, SHOW_IP));
+      sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER, SEND_TYPE_NOTICE,
+                     "No response from %s during handshake, closing link",
+                     client_get_name(client, MASK_IP));
       log_write(LOG_TYPE_IRCD, "No response from %s during handshake, closing link",
                 client_get_name(client, SHOW_IP));
       exit = true;
     }
-    else if (HasFlag(client, FLAGS_FINISHED_AUTH))
+    else if (HasFlag(client, FLAGS_LOOKUP_DONE))
       exit = true;
 
     if (exit)
-      exit_client(client, "Registration timed out");
+      client_exit(client, "Registration timed out");
   }
 }
 
@@ -411,9 +442,9 @@ conf_try_ban(struct Client *client, int type, const char *reason)
     case CLIENT_BAN_KLINE:
       if (HasFlag(client, FLAGS_EXEMPTKLINE))
       {
-        sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                             "KLINE over-ruled for %s, client is kline_exempt",
-                             client_get_name(client, HIDE_IP));
+        sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                       "KLINE over-ruled for %s, client is kline_exempt",
+                       client_get_name(client, HIDE_IP));
         return;
       }
 
@@ -427,9 +458,9 @@ conf_try_ban(struct Client *client, int type, const char *reason)
     case CLIENT_BAN_XLINE:
       if (HasFlag(client, FLAGS_EXEMPTXLINE))
       {
-        sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                             "XLINE over-ruled for %s, client is xline_exempt",
-                             client_get_name(client, HIDE_IP));
+        sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                       "XLINE over-ruled for %s, client is xline_exempt",
+                       client_get_name(client, HIDE_IP));
         return;
       }
 
@@ -440,13 +471,13 @@ conf_try_ban(struct Client *client, int type, const char *reason)
       break;
   }
 
-  sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE, "%c-line active for %s",
-                       ban_type, client_get_name(client, HIDE_IP));
+  sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "%c-line active for %s",
+                 ban_type, client_get_name(client, HIDE_IP));
 
   if (IsClient(client))
     sendto_one_numeric(client, &me, ERR_YOUREBANNEDCREEP, reason);
 
-  exit_client(client, reason);
+  client_exit(client, reason);
 }
 
 /* find_person()
@@ -529,7 +560,7 @@ client_get_name(const struct Client *client, enum addr_mask_type type)
                client->name, client->username, client->sockhost);
       break;
     case MASK_IP:
-      if (client->addr.ss.ss_family == AF_INET)
+      if (address_is_ipv4(&client->addr))
         snprintf(buf, sizeof(buf), "%s[%s@255.255.255.255]",
                  client->name, client->username);
       else
@@ -584,14 +615,14 @@ client_close_connection(struct Client *client)
     ++ServerStats.is_cl;
     ServerStats.is_cbs += client->connection->send.bytes;
     ServerStats.is_cbr += client->connection->recv.bytes;
-    ServerStats.is_cti += event_base->time.sec_monotonic - client->connection->created_monotonic;
+    ServerStats.is_cti += io_time_get(IO_TIME_MONOTONIC_SEC) - client->connection->created_monotonic;
   }
   else if (IsServer(client))
   {
     ++ServerStats.is_sv;
     ServerStats.is_sbs += client->connection->send.bytes;
     ServerStats.is_sbr += client->connection->recv.bytes;
-    ServerStats.is_sti += event_base->time.sec_monotonic - client->connection->created_monotonic;
+    ServerStats.is_sti += io_time_get(IO_TIME_MONOTONIC_SEC) - client->connection->created_monotonic;
 
     list_node_t *node;
     LIST_FOREACH(node, connect_items.head)
@@ -603,7 +634,7 @@ client_close_connection(struct Client *client)
        * this servername.
        */
       if (irccmp(conf->name, client->name) == 0)
-        conf->until = event_base->time.sec_monotonic + conf->class->con_freq;
+        conf->until = io_time_get(IO_TIME_MONOTONIC_SEC) + conf->class->con_freq;
     }
   }
   else
@@ -641,9 +672,9 @@ exit_one_client(struct Client *client, const char *comment)
 
   if (IsClient(client))
   {
-    if (HasUMode(client, UMODE_OPER))
+    if (user_mode_has_flag(client, UMODE_OPER))
       --Count.oper;
-    if (HasUMode(client, UMODE_INVISIBLE))
+    if (user_mode_has_flag(client, UMODE_INVISIBLE))
       --Count.invisi;
 
     list_remove(&client->lnode, &client->servptr->serv->client_list);
@@ -671,8 +702,8 @@ exit_one_client(struct Client *client, const char *comment)
   }
   else if (IsServer(client))
   {
-    sendto_realops_flags(UMODE_EXTERNAL, L_ALL, SEND_NOTICE, "Server %s split from %s",
-                         client->name, client->servptr->name);
+    sendto_clients(UMODE_EXTERNAL, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "Server %s split from %s",
+                   client->name, client->servptr->name);
 
     list_remove(&client->lnode, &client->servptr->serv->server_list);
     list_remove(&client->node, &global_server_list);
@@ -737,7 +768,7 @@ recurse_remove_clients(struct Client *client, const char *comment)
  *               Client memory is scheduled to be freed
  */
 void
-exit_client(struct Client *client, const char *comment)
+client_exit(struct Client *client, const char *comment)
 {
   assert(!IsMe(client));
   assert(client != &me);
@@ -754,15 +785,17 @@ exit_client(struct Client *client, const char *comment)
 
     AddFlag(client, FLAGS_CLOSING);
 
-    if (client->connection->auth)
+    hook_dispatch(ircd_hook_client_exit_local, &(ircd_hook_client_exit_ctx){ .client = client, .comment = comment });
+
+    if (client->connection->lookup)
     {
-      auth_delete(client->connection->auth);
-      client->connection->auth = NULL;
+      lookup_delete(client->connection->lookup);
+      client->connection->lookup = NULL;
     }
 
     if (IsClient(client))
     {
-      if (HasUMode(client, UMODE_OPER))
+      if (user_mode_has_flag(client, UMODE_OPER))
       {
         list_node_t *node = list_find_remove(&oper_list, client);
         if (node)
@@ -781,13 +814,9 @@ exit_client(struct Client *client, const char *comment)
 
       monitor_clear_list(client);
 
-      sendto_realops_flags(UMODE_CCONN, L_ALL, SEND_NOTICE, "Client exiting: %s (%s@%s) [%s] [%s]",
-                           client->name, client->username, client->realhost,
-                           client->sockhost, comment);
-
       log_write(LOG_TYPE_USER, "%s (%ju): %s!%s@%s %s %s %ju/%ju :%s",
                 date_ctime(client->connection->created_real),
-                event_base->time.sec_monotonic - client->connection->created_monotonic,
+                io_time_get(IO_TIME_MONOTONIC_SEC) - client->connection->created_monotonic,
                 client->name, client->username, client->host,
                 client->sockhost, client->account,
                 client->connection->send.bytes >> 10,
@@ -812,11 +841,8 @@ exit_client(struct Client *client, const char *comment)
 
     client_close_connection(client);
   }
-  else if (IsClient(client) && HasFlag(client->servptr, FLAGS_EOB))
-    sendto_realops_flags(UMODE_FARCONNECT, L_ALL, SEND_NOTICE,
-                         "Client exiting at %s: %s (%s@%s) [%s] [%s]",
-                         client->servptr->name, client->name,
-                         client->username, client->realhost, client->sockhost, comment);
+  else
+    hook_dispatch(ircd_hook_client_exit_remote, &(ircd_hook_client_exit_ctx){ .client = client, .comment = comment });
 
   if (IsServer(client))
   {
@@ -837,26 +863,26 @@ exit_client(struct Client *client, const char *comment)
 
     /* Send SQUIT for 'client' in every direction. 'client' is already off of local_server_list here */
     if (!HasFlag(client, FLAGS_SQUIT))
-      sendto_server(NULL, 0, 0, "SQUIT %s :%s", client->id, comment);
+      sendto_servers(NULL, 0, 0, "SQUIT %s :%s", client->id, comment);
 
     /* Now exit the clients internally */
     recurse_remove_clients(client, splitstr);
 
     if (MyConnect(client))
     {
-      sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                           "%s was connected for %s. %ju/%ju sendK/recvK.",
-                           client->name, time_format_duration(event_base->time.sec_monotonic - client->connection->created_monotonic),
-                           client->connection->send.bytes >> 10,
-                           client->connection->recv.bytes >> 10);
+      sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                     "%s was connected for %s. %ju/%ju sendK/recvK.",
+                     client->name, time_format_duration(io_time_get(IO_TIME_MONOTONIC_SEC) - client->connection->created_monotonic),
+                     client->connection->send.bytes >> 10,
+                     client->connection->recv.bytes >> 10);
       log_write(LOG_TYPE_IRCD, "%s was connected for %s. %ju/%ju sendK/recvK.",
-                client->name, time_format_duration(event_base->time.sec_monotonic - client->connection->created_monotonic),
+                client->name, time_format_duration(io_time_get(IO_TIME_MONOTONIC_SEC) - client->connection->created_monotonic),
                 client->connection->send.bytes >> 10,
                 client->connection->recv.bytes >> 10);
     }
   }
   else if (IsClient(client) && !HasFlag(client, FLAGS_KILLED))
-    sendto_server(client->from, 0, 0, ":%s QUIT :%s", client->id, comment);
+    sendto_servers(client->from, 0, 0, ":%s QUIT :%s", client->id, comment);
 
   /* The client *better* be off all of the lists */
   assert(list_find(&unknown_list, client) == NULL);
@@ -869,6 +895,19 @@ exit_client(struct Client *client, const char *comment)
   exit_one_client(client, comment);
 }
 
+void
+client_exit_fmt(struct Client *client, const char *format, ...)
+{
+  char buf[IRCD_BUFSIZE];
+  va_list args;
+
+  va_start(args, format);
+  vsnprintf(buf, sizeof(buf), format, args);
+  va_end(args);
+
+  client_exit(client, buf);
+}
+
 /*
  * dead_link_on_write - report a write error if not already dead,
  *			mark it as dead then exit it
@@ -900,7 +939,6 @@ dead_link_on_write(struct Client *client, int ierrno)
 void
 dead_link_on_read(struct Client *client, int error)
 {
-  char errmsg[IRCD_BUFSIZE];
   int current_error;
 
   if (IsDefunct(client))
@@ -915,37 +953,35 @@ dead_link_on_read(struct Client *client, int error)
   {
     if (error == 0)
     {
-      sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE,
-                           "Server %s closed the connection",
-                           client_get_name(client, SHOW_IP));
-      sendto_realops_flags(UMODE_SERVNOTICE, L_OPER, SEND_NOTICE,
-                           "Server %s closed the connection",
-                           client_get_name(client, MASK_IP));
+      sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE,
+                     "Server %s closed the connection",
+                     client_get_name(client, SHOW_IP));
+      sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER, SEND_TYPE_NOTICE,
+                     "Server %s closed the connection",
+                     client_get_name(client, MASK_IP));
       log_write(LOG_TYPE_IRCD, "Server %s closed the connection",
                 client_get_name(client, SHOW_IP));
     }
     else
     {
-      sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE,
-                           "Lost connection to %s: %s",
-                           client_get_name(client, SHOW_IP), strerror(current_error));
-      sendto_realops_flags(UMODE_SERVNOTICE, L_OPER, SEND_NOTICE,
-                           "Lost connection to %s: %s",
-                           client_get_name(client, MASK_IP), strerror(current_error));
+      sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE,
+                     "Lost connection to %s: %s",
+                     client_get_name(client, SHOW_IP), strerror(current_error));
+      sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER, SEND_TYPE_NOTICE,
+                     "Lost connection to %s: %s",
+                     client_get_name(client, MASK_IP), strerror(current_error));
       log_write(LOG_TYPE_IRCD, "Lost connection to %s: %s",
                 client_get_name(client, SHOW_IP), strerror(current_error));
     }
 
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE, "%s was connected for %s",
-                         client->name, time_format_duration(event_base->time.sec_monotonic - client->connection->created_monotonic));
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "%s was connected for %s",
+                   client->name, time_format_duration(io_time_get(IO_TIME_MONOTONIC_SEC) - client->connection->created_monotonic));
   }
 
   if (error == 0)
-    strlcpy(errmsg, "Remote host closed the connection", sizeof(errmsg));
+    client_exit(client, "Remote host closed the connection");
   else
-    snprintf(errmsg, sizeof(errmsg), "Read error: %s", strerror(current_error));
-
-  exit_client(client, errmsg);
+    client_exit_fmt(client, "Read error: %s", strerror(current_error));
 }
 
 void
@@ -964,8 +1000,8 @@ exit_aborted_clients(void)
 
     if (client == NULL)
     {
-      sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                           "Warning: null client on abort_list!");
+      sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                     "Warning: null client on abort_list!");
       continue;
     }
 
@@ -974,7 +1010,7 @@ exit_aborted_clients(void)
     else
       notice = "Write error: connection closed";
 
-    exit_client(client, notice);
+    client_exit(client, notice);
   }
 }
 
@@ -1000,11 +1036,11 @@ client_get_idle_time(const struct Client *source,
   const struct ClassItem *const class = class_get_ptr(&target->connection->confs);
 
   if (!(class->flags & CLASS_FLAGS_FAKE_IDLE) || target == source)
-    return event_base->time.sec_monotonic - target->connection->last_privmsg;
+    return io_time_get(IO_TIME_MONOTONIC_SEC) - target->connection->last_privmsg;
 
-  if (HasUMode(source, UMODE_OPER) &&
+  if (user_mode_has_flag(source, UMODE_OPER) &&
       !(class->flags & CLASS_FLAGS_HIDE_IDLE_FROM_OPERS))
-    return event_base->time.sec_monotonic - target->connection->last_privmsg;
+    return io_time_get(IO_TIME_MONOTONIC_SEC) - target->connection->last_privmsg;
 
   const unsigned int min_idle = class->min_idle;
   const unsigned int max_idle = class->max_idle;
@@ -1015,7 +1051,7 @@ client_get_idle_time(const struct Client *source,
   if (class->flags & CLASS_FLAGS_RANDOM_IDLE)
     idle = genrand_int32();
   else
-    idle = event_base->time.sec_monotonic - target->connection->last_privmsg;
+    idle = io_time_get(IO_TIME_MONOTONIC_SEC) - target->connection->last_privmsg;
 
   if (max_idle)
     idle %= max_idle;
