@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,43 +24,44 @@
  */
 
 #include "stdinc.h"
-#include "defaults.h"
+#include "address.h"
+#include "fdlist.h"
+#include "io_string.h"
+#include "io_time.h"
 #include "list.h"
+#include "log.h"
+#include "memory.h"
+#include "misc.h"
+#include "module.h"
+#include "res.h"
+#include "reslib.h"
+#include "defaults.h"
 #include "ircd_defs.h"
 #include "parse.h"
+#include "channel.h"
+#include "channel_mode.h"
+#include "cloak.h"
 #include "conf.h"
+#include "conf_class.h"
 #include "conf_cluster.h"
+#include "conf_db.h"
 #include "conf_gecos.h"
+#include "conf_parser.h"
 #include "conf_pseudo.h"
 #include "conf_resv.h"
 #include "conf_service.h"
 #include "conf_shared.h"
-#include "server.h"
-#include "channel.h"
 #include "client.h"
-#include "event.h"
-#include "irc_string.h"
-#include "ircd.h"
-#include "listener.h"
-#include "address.h"
-#include "module.h"
-#include "numeric.h"
-#include "fdlist.h"
-#include "log.h"
-#include "send.h"
-#include "memory.h"
-#include "reslib.h"
-#include "res.h"
-#include "user.h"
-#include "channel_mode.h"
-#include "misc.h"
-#include "conf_db.h"
-#include "conf_class.h"
-#include "motd.h"
 #include "ipcache.h"
+#include "ircd.h"
 #include "isupport.h"
+#include "listener.h"
+#include "motd.h"
+#include "numeric.h"
+#include "send.h"
+#include "server.h"
+#include "user.h"
 #include "whowas.h"
-#include "cloak.h"
 
 /* Hashtable stuff...now external as it's used in m_stats.c */
 list_t atable[ADDRESS_HASHSIZE];
@@ -77,10 +78,9 @@ struct config_serverinfo_entry ConfigServerInfo;
 struct config_admin_entry ConfigAdminInfo;
 struct conf_parser_context conf_parser_ctx;
 
-extern unsigned int lineno;
-extern char linebuf[];
-extern char conffilebuf[IRCD_BUFSIZE];
-extern int yyparse(); /* defined in y.tab.c */
+extern unsigned int conf_line_number;
+extern char conf_line_text[];
+extern char conf_file_name[IRCD_BUFSIZE];
 
 /* struct MaskItem *find_conf_by_address(const char *, struct io_addr *,
  *                                         int type, int fam, const char *username)
@@ -105,7 +105,7 @@ find_conf_by_address(const char *name, const struct io_addr *addr, unsigned int
   if (addr)
   {
     /* Check for IPV6 matches... */
-    if (addr->ss.ss_family == AF_INET6)
+    if (address_is_ipv6(addr))
     {
       for (int b = 128; b >= 0; b -= 16)
       {
@@ -116,11 +116,11 @@ find_conf_by_address(const char *name, const struct io_addr *addr, unsigned int
           if ((arec->type == type) &&
               arec->precedence > hprecv &&
               arec->masktype == HM_IPV6 &&
-              match_ipv6(addr, &arec->Mask.ipa.addr,
+              address_match_ipv6(addr, &arec->Mask.ipa.addr,
                          arec->Mask.ipa.bits) &&
               (!username || !cmpfunc(arec->username, username)) &&
               (IsNeedPassword(arec->conf) || arec->conf->passwd == NULL ||
-               match_conf_password(password, arec->conf)))
+               conf_match_password(password, arec->conf)))
           {
             hprecv = arec->precedence;
             hprec = arec->conf;
@@ -128,7 +128,7 @@ find_conf_by_address(const char *name, const struct io_addr *addr, unsigned int
         }
       }
     }
-    else if (addr->ss.ss_family == AF_INET)
+    else if (address_is_ipv4(addr))
     {
       for (int b = 32; b >= 0; b -= 8)
       {
@@ -139,11 +139,11 @@ find_conf_by_address(const char *name, const struct io_addr *addr, unsigned int
           if ((arec->type == type) &&
               arec->precedence > hprecv &&
               arec->masktype == HM_IPV4 &&
-              match_ipv4(addr, &arec->Mask.ipa.addr,
+              address_match_ipv4(addr, &arec->Mask.ipa.addr,
                          arec->Mask.ipa.bits) &&
               (!username || !cmpfunc(arec->username, username)) &&
               (IsNeedPassword(arec->conf) || arec->conf->passwd == NULL ||
-               match_conf_password(password, arec->conf)))
+               conf_match_password(password, arec->conf)))
           {
             hprecv = arec->precedence;
             hprec = arec->conf;
@@ -168,7 +168,7 @@ find_conf_by_address(const char *name, const struct io_addr *addr, unsigned int
             !cmpfunc(arec->Mask.hostname, name) &&
             (!username || !cmpfunc(arec->username, username)) &&
             (IsNeedPassword(arec->conf) || arec->conf->passwd == NULL ||
-             match_conf_password(password, arec->conf)))
+             conf_match_password(password, arec->conf)))
         {
           hprecv = arec->precedence;
           hprec = arec->conf;
@@ -190,7 +190,7 @@ find_conf_by_address(const char *name, const struct io_addr *addr, unsigned int
           !cmpfunc(arec->Mask.hostname, name) &&
           (!username || !cmpfunc(arec->username, username)) &&
           (IsNeedPassword(arec->conf) || arec->conf->passwd == NULL ||
-           match_conf_password(password, arec->conf)))
+           conf_match_password(password, arec->conf)))
       {
         hprecv = arec->precedence;
         hprec = arec->conf;
@@ -264,10 +264,10 @@ add_conf_by_address(const unsigned int type, struct MaskItem *conf)
   static unsigned int prec_value = UINT_MAX;
   int bits = 0;
 
-  assert(type && !EmptyString(hostname));
+  assert(type && !string_is_empty(hostname));
 
   struct AddressRec *arec = io_calloc(sizeof(*arec));
-  arec->masktype = parse_netmask(hostname, &arec->Mask.ipa.addr, &bits);
+  arec->masktype = address_parse_netmask(hostname, &arec->Mask.ipa.addr, &bits);
   arec->Mask.ipa.bits = bits;
   arec->username = username;
   arec->conf = conf;
@@ -309,7 +309,7 @@ delete_one_address_conf(const char *address, struct MaskItem *conf)
   list_node_t *node;
   struct io_addr addr;
 
-  switch (parse_netmask(address, &addr, &bits))
+  switch (address_parse_netmask(address, &addr, &bits))
   {
     case HM_IPV4:
       /* We have to do this, since we do not re-hash for every bit -A1kmm. */
@@ -393,10 +393,10 @@ hostmask_send_expiration(const struct AddressRec *const arec)
     default: break;
   }
 
-  sendto_realops_flags(UMODE_EXPIRATION, L_ALL, SEND_NOTICE,
-                       "Temporary %c-line for [%s@%s] expired", ban_type,
-                       (arec->conf->user) ? arec->conf->user : "*",
-                       (arec->conf->host) ? arec->conf->host : "*");
+  sendto_clients(UMODE_EXPIRATION, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                 "Temporary %c-line for [%s@%s] expired", ban_type,
+                 (arec->conf->user) ? arec->conf->user : "*",
+                 (arec->conf->host) ? arec->conf->host : "*");
 }
 
 static void
@@ -410,7 +410,7 @@ hostmask_expire_temporary(void)
     {
       struct AddressRec *arec = node->data;
 
-      if (arec->conf->until == 0 || arec->conf->until > event_base->time.sec_real)
+      if (arec->conf->until == 0 || arec->conf->until > io_time_get(IO_TIME_REALTIME_SEC))
         continue;
 
       switch (arec->type)
@@ -447,7 +447,7 @@ conf_dns_callback(void *vptr, const struct io_addr *addr, const char *name, size
   conf->dns_pending = false;
 
   if (addr)
-    *conf->addr = *addr;
+    address_copy(conf->addr, addr);
   else
     conf->dns_failed = true;
 }
@@ -460,44 +460,22 @@ conf_dns_callback(void *vptr, const struct io_addr *addr, const char *name, size
 void
 conf_dns_lookup(struct MaskItem *conf)
 {
-  struct addrinfo hints, *res;
+  if (address_from_string(conf->host, conf->addr))
+    return;
 
   /*
-   * Do name lookup now on hostnames given and store the
-   * ip numbers in conf structure.
+   * By this point conf->host possibly is not a numerical network address. Do a nameserver
+   * lookup of the conf host. If the conf entry is currently doing a ns lookup do nothing.
    */
-  memset(&hints, 0, sizeof(hints));
-
-  hints.ai_family = AF_UNSPEC;
-  hints.ai_socktype = SOCK_STREAM;
-
-  /* Get us ready for a bind() and don't bother doing dns lookup */
-  hints.ai_flags = AI_PASSIVE | AI_NUMERICHOST;
-
-  if (getaddrinfo(conf->host, NULL, &hints, &res))
-  {
-    /*
-     * By this point conf->host possibly is not a numerical network address. Do a nameserver
-     * lookup of the conf host. If the conf entry is currently doing a ns lookup do nothing.
-     */
-    if (conf->dns_pending)
-      return;
-
-    conf->dns_pending = true;
-
-    if (conf->aftype == AF_INET)
-      gethost_byname_type(conf_dns_callback, conf, conf->host, T_A);
-    else
-      gethost_byname_type(conf_dns_callback, conf, conf->host, T_AAAA);
+  if (conf->dns_pending)
     return;
-  }
 
-  assert(res);
+  conf->dns_pending = true;
 
-  memcpy(conf->addr, res->ai_addr, res->ai_addrlen);
-  conf->addr->ss_len = res->ai_addrlen;
-
-  freeaddrinfo(res);
+  if (conf->aftype == AF_INET)
+    gethost_byname_type(conf_dns_callback, conf, conf->host, T_A);
+  else
+    gethost_byname_type(conf_dns_callback, conf, conf->host, T_AAAA);
 }
 
 /* map_to_list()
@@ -558,6 +536,7 @@ conf_free(struct MaskItem *conf)
   io_free(conf->reason);
   io_free(conf->certfp);
   io_free(conf->whois);
+  io_free(conf->modes);
   io_free(conf->user);
   io_free(conf->host);
   io_free(conf->addr);
@@ -582,7 +561,7 @@ conf_free(struct MaskItem *conf)
   io_free(conf);
 }
 
-/* attach_iline()
+/* conf_attach_auth()
  *
  * inputs       - client pointer
  *              - conf pointer
@@ -590,7 +569,7 @@ conf_free(struct MaskItem *conf)
  * side effects - do actual attach
  */
 static int
-attach_iline(struct Client *client, struct MaskItem *conf)
+conf_attach_auth(struct Client *client, struct MaskItem *conf)
 {
   const struct ClassItem *const class = conf->class;
   bool a_limit_reached = false;
@@ -648,8 +627,7 @@ verify_access(struct Client *client)
   if (IsConfRedir(conf))
   {
     sendto_one_numeric(client, &me, RPL_REDIR,
-                       conf->name ? conf->name : "",
-                       conf->port);
+                       string_or_empty(conf->name), conf->port);
     return NOT_AUTHORIZED;
   }
 
@@ -665,7 +643,7 @@ verify_access(struct Client *client)
     AddFlag(client, FLAGS_SPOOF);
   }
 
-  return attach_iline(client, conf);
+  return conf_attach_auth(client, conf);
 }
 
 /* check_client()
@@ -706,18 +684,15 @@ conf_check_client(struct Client *client)
 
   if (error)
   {
-    char buf[REASONLEN + 1];
-    snprintf(buf, sizeof(buf), "Connection rejected - %s", error);
-
     ++ServerStats.is_ref;
 
     if (warn)
-      sendto_realops_flags(UMODE_REJ, L_ALL, SEND_NOTICE, "Rejecting client connection from %s: %s",
-                           client_get_name(client, SHOW_IP), error);
+      sendto_clients(UMODE_REJ, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "Rejecting client connection from %s: %s",
+                     client_get_name(client, SHOW_IP), error);
 
     log_write(LOG_TYPE_IRCD, "Rejecting client connection from %s: %s",
          client_get_name(client, SHOW_IP), error);
-    exit_client(client, buf);
+    client_exit_fmt(client, "Connection rejected - %s", error);
     return false;
   }
 
@@ -868,7 +843,7 @@ operator_find(const struct Client *client, const char *name)
             break;
           case HM_IPV6:
           case HM_IPV4:
-            if (address_compare(&client->addr, conf->addr, false, false, conf->bits))
+            if (address_match(&client->addr, conf->addr, false, false, conf->bits))
               return conf;
             break;
           default:
@@ -893,17 +868,14 @@ operator_find(const struct Client *client, const char *name)
 static void
 conf_set_defaults(void)
 {
-  /*
-   * Verify init_class() ran, this should be an unnecessary check
-   * but it's not much work.
-   */
-  assert(class_default == class_get_list()->tail->data);
+  /* Verify class_init() ran. */
+  assert(class_default == list_peek_tail(class_get_list()));
 
   ConfigServerInfo.network_name = io_strdup(NETWORK_NAME_DEFAULT);
   ConfigServerInfo.network_description = io_strdup(NETWORK_DESCRIPTION_DEFAULT);
   ConfigServerInfo.default_max_clients = MAXCLIENTS_MAX;
   ConfigServerInfo.max_nick_length = 9;
-  ConfigServerInfo.max_topic_length = 80;
+  ConfigServerInfo.max_topic_length = TOPICLEN;
   ConfigServerInfo.hub = 0;
 
   ConfigLog.use_logging = 1;
@@ -921,6 +893,7 @@ conf_set_defaults(void)
   ConfigChannel.max_invites = 20;
   ConfigChannel.max_bans = 100;
   ConfigChannel.max_bans_large = 500;
+  ConfigChannel.max_kick_length = KICKLEN;
   ConfigChannel.default_join_flood_count = 18;
   ConfigChannel.default_join_flood_time = 6;
 
@@ -944,6 +917,7 @@ conf_set_defaults(void)
   ConfigGeneral.kline_min_cidr6 = 48;
   ConfigGeneral.invisible_on_connect = 1;
   ConfigGeneral.disable_auth = 0;
+  ConfigGeneral.disable_dns = 0;
   ConfigGeneral.kill_chase_time_limit = 90;
   ConfigGeneral.default_floodcount = 8;
   ConfigGeneral.default_floodtime = 1;
@@ -952,6 +926,7 @@ conf_set_defaults(void)
   ConfigGeneral.min_nonwildcard = 4;
   ConfigGeneral.min_nonwildcard_simple = 3;
   ConfigGeneral.max_accept = 50;
+  ConfigGeneral.max_away_length = AWAYLEN;
   ConfigGeneral.anti_nick_flood = 0;
   ConfigGeneral.max_nick_time = 20;
   ConfigGeneral.max_nick_changes = 5;
@@ -974,12 +949,10 @@ conf_set_defaults(void)
   ConfigGeneral.ping_cookie = 0;
   ConfigGeneral.no_oper_flood = 0;
   ConfigGeneral.max_targets = 4;
-  ConfigGeneral.oper_only_umodes = UMODE_LOCOPS | UMODE_HIDDEN | UMODE_FARCONNECT |
-                                   UMODE_EXTERNAL | UMODE_FLOOD | UMODE_NCHANGE |
-                                   UMODE_SPY | UMODE_SKILL | UMODE_REJ | UMODE_CCONN;
-  ConfigGeneral.oper_umodes = UMODE_FLOOD | UMODE_LOCOPS | UMODE_SERVNOTICE | UMODE_WALLOP;
+  ConfigGeneral.oper_umodes = io_strdup("+flsw");
   ConfigGeneral.throttle_count = 1;
   ConfigGeneral.throttle_time = 1;
+  ConfigGeneral.ident_timeout = 4;
 
   ConfigGeneral.cloak_enabled = false;
   cloak_set_disabled();
@@ -998,10 +971,10 @@ conf_set_defaults(void)
 static void
 conf_validate(void)
 {
-  if (EmptyString(ConfigServerInfo.network_name))
+  if (string_is_empty(ConfigServerInfo.network_name))
     ConfigServerInfo.network_name = io_strdup(NETWORK_NAME_DEFAULT);
 
-  if (EmptyString(ConfigServerInfo.network_description))
+  if (string_is_empty(ConfigServerInfo.network_description))
     ConfigServerInfo.network_description = io_strdup(NETWORK_DESCRIPTION_DEFAULT);
 }
 
@@ -1014,7 +987,7 @@ conf_validate(void)
 static void
 conf_read(FILE *file)
 {
-  lineno = 1;
+  conf_line_number = 1;
 
   conf_set_defaults();  /* Set default values prior to conf parsing */
 
@@ -1043,8 +1016,8 @@ conf_rehash(bool sig)
 {
   if (sig)
   {
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                         "Got signal SIGHUP, reloading configuration file(s)");
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                   "Got signal SIGHUP, reloading configuration file(s)");
     log_write(LOG_TYPE_IRCD, "Got signal SIGHUP, reloading configuration file(s)");
   }
 
@@ -1080,7 +1053,7 @@ conf_connect_allowed(struct io_addr *addr)
   }
 
   struct ip_entry *ip_found = ipcache_record_find_or_add(addr);
-  if ((event_base->time.sec_monotonic - ip_found->last_attempt) < ConfigGeneral.throttle_time)
+  if ((io_time_get(IO_TIME_MONOTONIC_SEC) - ip_found->last_attempt) < ConfigGeneral.throttle_time)
   {
     if (ip_found->connection_count >= ConfigGeneral.throttle_count)
       return TOO_FAST;
@@ -1090,7 +1063,7 @@ conf_connect_allowed(struct io_addr *addr)
   else
     ip_found->connection_count = 1;
 
-  ip_found->last_attempt = event_base->time.sec_monotonic;
+  ip_found->last_attempt = io_time_get(IO_TIME_MONOTONIC_SEC);
   return 0;
 }
 
@@ -1125,18 +1098,12 @@ get_oper_name(const struct Client *client)
 
   if (MyConnect(client))
   {
-    const list_node_t *const node = client->connection->confs.head;
-
-    if (node)
+    const struct MaskItem *conf = list_peek_head(&client->connection->confs);
+    if (conf && (conf->type == CONF_OPER))
     {
-      const struct MaskItem *const conf = node->data;
-
-      if (conf->type == CONF_OPER)
-      {
-        snprintf(buf, sizeof(buf), "%s!%s@%s{%s}", client->name,
-                 client->username, client->host, conf->name);
-        return buf;
-      }
+      snprintf(buf, sizeof(buf), "%s!%s@%s{%s}",
+               client->name, client->username, client->host, conf->name);
+      return buf;
     }
 
     /*
@@ -1228,6 +1195,8 @@ conf_clear(void)
   ConfigServerInfo.tls_cipher_suites = NULL;
   io_free(ConfigServerInfo.tls_message_digest_algorithm);
   ConfigServerInfo.tls_message_digest_algorithm = NULL;
+  io_free(ConfigServerInfo.motd_file);
+  ConfigServerInfo.motd_file = NULL;
 
   /* Clean out ConfigAdminInfo */
   io_free(ConfigAdminInfo.name);
@@ -1248,6 +1217,8 @@ conf_clear(void)
   ConfigGeneral.cloak_secret = NULL;
   io_free(ConfigGeneral.cloak_suffix);
   ConfigGeneral.cloak_suffix = NULL;
+  io_free(ConfigGeneral.oper_umodes);
+  ConfigGeneral.oper_umodes = NULL;
 
   /* Clean out listeners */
   listener_close_marked();
@@ -1266,8 +1237,8 @@ conf_handle_tls(bool cold)
     else
     {
       /* Failed to load new settings/certs, old ones remain active */
-      sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                           "Error reloading TLS settings, check the ircd log"); // report_crypto_errors logs this
+      sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                     "Error reloading TLS settings, check the ircd log"); // report_crypto_errors logs this
     }
   }
 }
@@ -1294,9 +1265,9 @@ conf_read_files(bool cold)
     }
     else
     {
-      sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE,
-                           "Unable to read configuration file '%s': %s",
-                           ConfigGeneral.configfile, strerror(errno));
+      sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE,
+                     "Unable to read configuration file '%s': %s",
+                     ConfigGeneral.configfile, strerror(errno));
       return;
     }
   }
@@ -1309,7 +1280,7 @@ conf_read_files(bool cold)
    *
    *  - Gozem 2002-07-21
    */
-  strlcpy(conffilebuf, ConfigGeneral.configfile, sizeof(conffilebuf));
+  strlcpy(conf_file_name, ConfigGeneral.configfile, sizeof(conf_file_name));
 
   if (cold == false)
     conf_clear();
@@ -1319,44 +1290,42 @@ conf_read_files(bool cold)
 
   conf_handle_tls(cold);
 
-  isupport_add("NICKLEN", NULL, ConfigServerInfo.max_nick_length);
-  isupport_add("NETWORK", ConfigServerInfo.network_name, -1);
+  motd_init();
 
-  char buf[IRCD_BUFSIZE];
-  snprintf(buf, sizeof(buf), "beI:%u", ConfigChannel.max_bans);
-  isupport_add("MAXLIST", buf, -1);
-
-  isupport_add("MAXTARGETS", NULL, ConfigGeneral.max_targets);
-  isupport_add("CHANTYPES", "#", -1);
-
-  snprintf(buf, sizeof(buf), "#:%u", ConfigChannel.max_channels);
-  isupport_add("CHANLIMIT", buf, -1);
-
-  isupport_add("CHANNELLEN", NULL, CHANNELLEN);
-  isupport_add("TOPICLEN", NULL, ConfigServerInfo.max_topic_length);
+  isupport_add("CASEMAPPING", "%s", "ascii");
+  isupport_add("AWAYLEN", "%d", ConfigGeneral.max_away_length);
+  isupport_add("KICKLEN", "%d", ConfigChannel.max_kick_length);
+  isupport_add("NICKLEN", "%d", ConfigServerInfo.max_nick_length);
+  isupport_add("NETWORK", "%s", ConfigServerInfo.network_name);
+  isupport_add("MAXLIST", "beI:%u", ConfigChannel.max_bans);
+  isupport_add("MAXTARGETS", "%d", ConfigGeneral.max_targets);
+  isupport_add("CHANTYPES", "%s", "#");
+  isupport_add("CHANLIMIT", "#:%u", ConfigChannel.max_channels);
+  isupport_add("CHANNELLEN", "%d", CHANNELLEN);
+  isupport_add("TOPICLEN", "%d", ConfigServerInfo.max_topic_length);
 }
 
-/* conf_add_class_to_conf()
+/* conf_assign_class()
  *
  * inputs       - pointer to config item
  * output       - NONE
  * side effects - Add a class pointer to a conf
  */
 void
-conf_add_class_to_conf(struct MaskItem *conf, const char *name)
+conf_assign_class(struct MaskItem *conf, const char *name)
 {
-  if (EmptyString(name) || (conf->class = class_find(name, true)) == NULL)
+  if (string_is_empty(name) || (conf->class = class_find(name, true)) == NULL)
   {
     conf->class = class_default;
 
     if (conf->type == CONF_CLIENT || conf->type == CONF_OPER)
-      sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE,
-                           "Warning *** Defaulting to default class for %s@%s",
-                           conf->user, conf->host);
+      sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE,
+                     "Warning *** Defaulting to default class for %s@%s",
+                     conf->user, conf->host);
     else
-      sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE,
-                           "Warning *** Defaulting to default class for %s",
-                           conf->name);
+      sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE,
+                     "Warning *** Defaulting to default class for %s",
+                     conf->name);
   }
 }
 
@@ -1372,24 +1341,24 @@ yyerror(const char *msg)
   if (conf_parser_ctx.pass != 1)
     return;
 
-  const char *p = stripws(linebuf);
-  sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE, "\"%s\", line %u: %s: %s",
-                       conffilebuf, lineno, msg, p);
+  const char *p = stripws(conf_line_text);
+  sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE, "\"%s\", line %u: %s: %s",
+                 conf_file_name, conf_line_number, msg, p);
   log_write(LOG_TYPE_IRCD, "\"%s\", line %u: %s: %s",
-            conffilebuf, lineno, msg, p);
+            conf_file_name, conf_line_number, msg, p);
 }
 
 void
 conf_error_report(const char *msg)
 {
-  const char *p = stripws(linebuf);
-  sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE, "\"%s\", line %u: %s: %s",
-                       conffilebuf, lineno, msg, p);
+  const char *p = stripws(conf_line_text);
+  sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE, "\"%s\", line %u: %s: %s",
+                 conf_file_name, conf_line_number, msg, p);
   log_write(LOG_TYPE_IRCD, "\"%s\", line %u: %s: %s",
-            conffilebuf, lineno, msg, p);
+            conf_file_name, conf_line_number, msg, p);
 }
 
-/* match_conf_password()
+/* conf_match_password()
  *
  * inputs       - pointer to given password
  *              - pointer to Conf
@@ -1397,9 +1366,9 @@ conf_error_report(const char *msg)
  * side effects - none
  */
 bool
-match_conf_password(const char *password, const struct MaskItem *conf)
+conf_match_password(const char *password, const struct MaskItem *conf)
 {
-  if (EmptyString(password) || EmptyString(conf->passwd))
+  if (string_is_empty(password) || string_is_empty(conf->passwd))
     return false;
 
   const char *encr;
@@ -1410,93 +1379,3 @@ match_conf_password(const char *password, const struct MaskItem *conf)
 
   return encr && strcmp(encr, conf->passwd) == 0;
 }
-
-/*
- * split_nuh
- *
- * inputs	- pointer to original mask (modified in place)
- *		- pointer to pointer where nick should go
- *		- pointer to pointer where user should go
- *		- pointer to pointer where host should go
- * output	- NONE
- * side effects	- mask is modified in place
- *		  If nick pointer is NULL, ignore writing to it
- *		  this allows us to use this function elsewhere.
- *
- * mask				nick	user	host
- * ----------------------	------- ------- ------
- * Dianora!db@db.net		Dianora	db	db.net
- * Dianora			Dianora	*	*
- * db.net                       *       *       db.net
- * OR if nick pointer is NULL
- * Dianora			-	*	Dianora
- * Dianora!			Dianora	*	*
- * Dianora!@			Dianora	*	*
- * Dianora!db			Dianora	db	*
- * Dianora!@db.net		Dianora	*	db.net
- * db@db.net			*	db	db.net
- * !@				*	*	*
- * @				*	*	*
- * !				*	*	*
- */
-void
-split_nuh(struct split_nuh_item *const iptr)
-{
-  char *p = NULL, *q = NULL;
-
-  if (iptr->nickptr)
-    strlcpy(iptr->nickptr, "*", iptr->nicksize);
-
-  if (iptr->userptr)
-    strlcpy(iptr->userptr, "*", iptr->usersize);
-
-  if (iptr->hostptr)
-    strlcpy(iptr->hostptr, "*", iptr->hostsize);
-
-  if ((p = strchr(iptr->nuhmask, '!')))
-  {
-    *p = '\0';
-
-    if (iptr->nickptr && *iptr->nuhmask)
-      strlcpy(iptr->nickptr, iptr->nuhmask, iptr->nicksize);
-
-    if ((q = strchr(++p, '@')))
-    {
-      *q++ = '\0';
-
-      if (*p)
-        strlcpy(iptr->userptr, p, iptr->usersize);
-
-      if (*q)
-        strlcpy(iptr->hostptr, q, iptr->hostsize);
-    }
-    else
-    {
-      if (*p)
-        strlcpy(iptr->userptr, p, iptr->usersize);
-    }
-  }
-  else
-  {
-    /* No ! found so lets look for a user@host */
-    if ((p = strchr(iptr->nuhmask, '@')))
-    {
-      /* if found a @ */
-      *p++ = '\0';
-
-      if (*iptr->nuhmask)
-        strlcpy(iptr->userptr, iptr->nuhmask, iptr->usersize);
-
-      if (*p)
-        strlcpy(iptr->hostptr, p, iptr->hostsize);
-    }
-    else
-    {
-      /* No @ found */
-      if (iptr->nickptr == NULL || strpbrk(iptr->nuhmask, ".:"))
-        strlcpy(iptr->hostptr, iptr->nuhmask, iptr->hostsize);
-      else
-        strlcpy(iptr->nickptr, iptr->nuhmask, iptr->nicksize);
-    }
-  }
-}
