@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,10 +26,10 @@
 #include "stdinc.h"
 #include "defaults.h"
 #include "list.h"
-#include "conf.h"
-#include "irc_string.h"
+#include "io_string.h"
 #include "memory.h"
 #include "patricia.h"
+#include "conf.h"
 
 static list_t class_list;
 
@@ -52,7 +52,6 @@ class_make(void)
   class->max_recvq = DEFAULT_RECVQ;
   class->ip_tree_v6 = patricia_new(128);
   class->ip_tree_v4 = patricia_new( 32);
-
   list_add(class, &class->node, &class_list);
 
   return class;
@@ -85,19 +84,14 @@ class_init(void)
 const struct ClassItem *
 class_get_ptr(const list_t *const list)
 {
-  const list_node_t *const node = list->head;
+  const struct MaskItem *conf = list_peek_head(list);
+  if (conf == NULL)
+    return class_default;
 
-  if (node)
-  {
-    const struct MaskItem *const conf = node->data;
+  assert(conf->class);
+  assert(conf->type & (CONF_OPER | CONF_CLIENT | CONF_SERVER));
 
-    assert(conf->class);
-    assert(conf->type & (CONF_OPER | CONF_CLIENT | CONF_SERVER));
-
-    return conf->class;
-  }
-
-  return class_default;
+  return conf->class;
 }
 
 const char *
@@ -137,7 +131,6 @@ class_find(const char *name, bool active)
   LIST_FOREACH(node, class_list.head)
   {
     struct ClassItem *class = node->data;
-
     if (irccmp(class->name, name) == 0)
     {
       if (active && class->active == false)
@@ -174,13 +167,12 @@ class_delete_marked(void)
   LIST_FOREACH_SAFE(node, node_next, class_list.head)
   {
     struct ClassItem *class = node->data;
-
     if (class->active == false && class->ref_count == 0)
       class_free(class);
   }
 }
 
-static void *
+static patricia_tree_t *
 class_ip_limit_trie(struct ClassItem *class, void *addr)
 {
   if (((struct sockaddr *)addr)->sa_family == AF_INET6)
@@ -259,7 +251,7 @@ class_ip_limit_rebuild(struct ClassItem *class)
   LIST_FOREACH(node, local_client_list.head)
   {
     struct Client *client = node->data;
-    struct MaskItem *conf = client->connection->confs.tail->data;
+    struct MaskItem *conf = list_peek_tail(&client->connection->confs);
 
     if (conf->type == CONF_CLIENT)
       if (conf->class == class)
