@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1998-2024 ircd-hybrid development team
+ *  Copyright (c) 1998-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -31,15 +31,15 @@ struct GecosItem
   list_node_t node;
   char *mask;
   char *reason;
-  uintmax_t expire;
-  uintmax_t setat;
+  uintmax_t created_at;
+  uintmax_t expires_at;
   bool in_database;
 };
 
-extern const list_t *gecos_get_list(void);
 extern void gecos_delete(struct GecosItem *, bool);
-extern struct GecosItem *gecos_make(void);
-extern struct GecosItem *gecos_find(const char *, int (*)(const char *, const char *));
 extern void gecos_clear(void);
 extern void gecos_expire(void);
+extern struct GecosItem *gecos_make(void);
+extern struct GecosItem *gecos_find(const char *, int (*)(const char *, const char *));
+extern const list_t *gecos_get_list(void);
 #endif  /* INCLUDED_conf_gecos_h */
