@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 2001-2024 ircd-hybrid development team
+ *  Copyright (c) 2001-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,18 +24,19 @@
  */
 
 #include "stdinc.h"
-#include "irc_string.h"
+#include "address.h"
 #include "list.h"
+#include "memory.h"
+#include "misc.h"
+#include "io_string.h"
+#include "io_time.h"
 #include "send.h"
 #include "client.h"
-#include "memory.h"
 #include "ircd.h"
-#include "event.h"
 #include "ircd_defs.h"
-#include "misc.h"
+#include "nuh.h"
 #include "conf.h"
 #include "conf_resv.h"
-#include "address.h"
 
 static list_t resv_chan_list;
 static list_t resv_nick_list;
@@ -56,13 +57,12 @@ void
 resv_delete(struct ResvItem *resv, bool expired)
 {
   if (expired)
-    sendto_realops_flags(UMODE_EXPIRATION, L_ALL, SEND_NOTICE, "Temporary RESV for [%s] expired",
-                         resv->mask);
+    sendto_clients(UMODE_EXPIRATION, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "Temporary RESV for [%s] expired",
+                   resv->mask);
 
   while (resv->exempt_list.head)
   {
     struct ResvExemptItem *exempt = resv->exempt_list.head->data;
-
     list_remove(&exempt->node, &resv->exempt_list);
     io_free(exempt->name);
     io_free(exempt->user);
@@ -97,7 +97,7 @@ resv_make(const char *mask, const char *reason, const list_t *elist)
   struct ResvItem *resv = io_calloc(sizeof(*resv));
   resv->list = list;
   resv->mask = io_strdup(mask);
-  resv->reason = io_strndup(reason, IO_MIN(strlen(reason), REASONLEN));
+  resv->reason = io_strndup(reason, REASONLEN);
   list_add(resv, &resv->node, resv->list);
 
   if (elist)
@@ -110,7 +110,7 @@ resv_make(const char *mask, const char *reason, const list_t *elist)
       char nick[NICKLEN + 1];
       char user[USERLEN + 1];
       char host[HOSTLEN + 1];
-      struct split_nuh_item nuh =
+      struct nuh_split nuh =
       {
         .nuhmask = s,
         .nickptr = nick,
@@ -121,13 +121,13 @@ resv_make(const char *mask, const char *reason, const list_t *elist)
         .hostsize = sizeof(host)
       };
 
-      split_nuh(&nuh);
+      nuh_split(&nuh);
 
       struct ResvExemptItem *exempt = io_calloc(sizeof(*exempt));
       exempt->name = io_strdup(nick);
       exempt->user = io_strdup(user);
       exempt->host = io_strdup(host);
-      exempt->type = parse_netmask(host, &exempt->addr, &exempt->bits);
+      exempt->type = address_parse_netmask(host, &exempt->addr, &exempt->bits);
       list_add(exempt, &exempt->node, &resv->exempt_list);
     }
   }
@@ -149,9 +149,8 @@ resv_find(const char *name, int (*compare)(const char *, const char *))
   LIST_FOREACH_SAFE(node, node_next, list->head)
   {
     struct ResvItem *resv = node->data;
-
-    if (resv->expire &&
-        (resv->expire <= event_base->time.sec_real))
+    if (resv->expires_at &&
+        (resv->expires_at <= io_time_get(IO_TIME_REALTIME_SEC)))
       resv_delete(resv, true);
     else if (compare(resv->mask, name) == 0)
       return resv;
@@ -168,7 +167,6 @@ resv_exempt_find(const struct Client *client, const struct ResvItem *resv)
   LIST_FOREACH(node, resv->exempt_list.head)
   {
     const struct ResvExemptItem *exempt = node->data;
-
     if (match(exempt->name, client->name) == 0 && match(exempt->user, client->username) == 0)
     {
       switch (exempt->type)
@@ -180,7 +178,7 @@ resv_exempt_find(const struct Client *client, const struct ResvItem *resv)
           break;
         case HM_IPV6:
         case HM_IPV4:
-          if (address_compare(&client->addr, &exempt->addr, false, false, exempt->bits))
+          if (address_match(&client->addr, &exempt->addr, false, false, exempt->bits))
             return true;
           break;
         default:
@@ -204,7 +202,6 @@ resv_clear(void)
     LIST_FOREACH_SAFE(node, node_next, (*list)->head)
     {
       struct ResvItem *resv = node->data;
-
       if (resv->in_database == false)
         resv_delete(resv, false);
     }
@@ -223,9 +220,8 @@ resv_expire(void)
     LIST_FOREACH_SAFE(node, node_next, (*list)->head)
     {
       struct ResvItem *resv = node->data;
-
-      if (resv->expire &&
-          (resv->expire <= event_base->time.sec_real))
+      if (resv->expires_at &&
+          (resv->expires_at <= io_time_get(IO_TIME_REALTIME_SEC)))
         resv_delete(resv, true);
     }
   }
