@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
 
 #include "stdinc.h"
 #include "list.h"
-#include "irc_string.h"
+#include "io_string.h"
 #include "memory.h"
 #include "conf_shared.h"
 
@@ -43,7 +43,6 @@ shared_clear(void)
   while (shared_list.head)
   {
     struct SharedItem *shared = shared_list.head->data;
-
     list_remove(&shared->node, &shared_list);
     io_free(shared->server);
     io_free(shared->user);
@@ -69,11 +68,9 @@ shared_find(unsigned int type, const char *server, const char *user, const char
   LIST_FOREACH(node, shared_list.head)
   {
     const struct SharedItem *shared = node->data;
-
     if (shared->type & type)
       if (match(shared->server, server) == 0)
-        if (match(shared->user, user) == 0 &&
-            match(shared->host, host) == 0)
+        if (match(shared->user, user) == 0 && match(shared->host, host) == 0)
           return shared;
   }
 
