@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 2019-2024 ircd-hybrid development team
+ *  Copyright (c) 2019-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
 
 #include "stdinc.h"
 #include "list.h"
-#include "irc_string.h"
+#include "io_string.h"
 #include "channel.h"
 #include "channel_mode.h"
 #include "client.h"
