@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,7 +29,10 @@
  */
 
 #include "stdinc.h"
+#include "dbuf.h"
 #include "list.h"
+#include "memory.h"
+#include "io_string.h"
 #include "conf.h"
 #include "channel.h"
 #include "channel_mode.h"
@@ -37,12 +40,10 @@
 #include "hash.h"
 #include "id.h"
 #include "rng_mt.h"
-#include "irc_string.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "send.h"
-#include "memory.h"
-#include "dbuf.h"
+#include "user_mode.h"
 
 /**
  * @var static struct Client *idTable[HASHSIZE]
@@ -87,10 +88,9 @@ unsigned int
 hash_string(const char *name)
 {
   static unsigned int hashf_xor_key = 0;
-  const unsigned char *p = (const unsigned char *)name;
   unsigned int hval = FNV1_32_INIT;
 
-  if (EmptyString(p))
+  if (string_is_empty(name))
     return 0;
 
   if (hashf_xor_key == 0)
@@ -98,7 +98,7 @@ hash_string(const char *name)
       hashf_xor_key = genrand_int32() % 256;  /* better than nothing --adx */
     while (hashf_xor_key == 0);
 
-  for (; *p; ++p)
+  for (const unsigned char *p = (const unsigned char *)name; *p; ++p)
   {
     hval += (hval << 1) + (hval << 4) +
             (hval << 7) + (hval << 8) + (hval << 24);
@@ -495,6 +495,7 @@ free_list_task(struct Client *client)
     list_free_node(node);
   }
 
+  io_free(lt->topic);
   io_free(lt);
   client->connection->list_task = NULL;
 }
@@ -536,19 +537,17 @@ list_allow_channel(const char *name, const struct ListTask *lt)
 static void
 list_one_channel(struct Client *client, struct Channel *channel)
 {
-  const struct ListTask *const lt = client->connection->list_task;
-  char buf[MODEBUFLEN];
-
-  if (HasCMode(channel, MODE_SECRET) &&
-      !(HasUMode(client, UMODE_ADMIN) || member_find_link(client, channel)))
+  if (channel_has_mode(channel, MODE_SECRET) &&
+      !(user_mode_has_flag(client, UMODE_ADMIN) || member_find_link(client, channel)))
     return;
 
-  /* Check the number of users in the channel */
+  const struct ListTask *const lt = client->connection->list_task;
+  /* Check the number of users in the channel. */
   if (list_length(&channel->members) < lt->users_min ||
       list_length(&channel->members) > lt->users_max)
     return;
 
-  /* Check the creation time of the channel */
+  /* Check the creation time of the channel. */
   if (channel->creation_time &&
       ((unsigned int)channel->creation_time < lt->created_min ||
        (unsigned int)channel->creation_time > lt->created_max))
@@ -556,26 +555,22 @@ list_one_channel(struct Client *client, struct Channel *channel)
 
   /* Check the topic time of the channel */
   if ((unsigned int)channel->topic_time < lt->topicts_min ||
-    /* If channel->topic_time is zero, treat it as UINT_MAX to ensure it is considered greater than lt->topicts_max */
+    /* If channel->topic_time is zero, treat it as UINT_MAX to ensure it is considered greater than lt->topicts_max. */
       (channel->topic_time ? (unsigned int)channel->topic_time : UINT_MAX) > lt->topicts_max)
     return;
 
-  if (lt->topic[0] && match(lt->topic, channel->topic))
+  if (lt->topic && match(lt->topic, channel->topic))
     return;
 
   if (list_allow_channel(channel->name, lt) == false)
     return;
 
-  if (channel->topic[0])
-    snprintf(buf, sizeof(buf), "[%s] ",
-             channel_modes(channel, client, false));
-  else
-    snprintf(buf, sizeof(buf), "[%s]",
-             channel_modes(channel, client, false));
+  char buf[MODEBUFLEN];
+  snprintf(buf, sizeof(buf), channel->topic ? "[%s] " : "[%s]",
+           channel_modes(channel, client, false));
 
-  sendto_one_numeric(client, &me, RPL_LIST, channel->name,
-                     list_length(&channel->members),
-                     buf, channel->topic);
+  sendto_one_numeric(client, &me, RPL_LIST,
+                     channel->name, list_length(&channel->members), buf, string_or_empty(channel->topic));
 }
 
 /**
