@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,15 +28,20 @@
  * components, sets up event handlers, and manages server configurations.
  */
 
+#include <jansson.h>
 #include "stdinc.h"
 #include "io_getopt.h"
+#include "io_pidfile.h"
+#include "io_rlimit.h"
+#include "io_string.h"
+#include "io_time.h"
 #include "list.h"
+#include "log.h"
 #include "memory.h"
-#include "irc_string.h"
+#include "module.h"
 #include "event.h"
 #include "fdlist.h"
 #include "comm.h"
-#include "log.h"
 #include "res.h"
 #include "rng_mt.h"
 #include "defaults.h"
@@ -55,13 +60,12 @@
 #include "server.h"
 #include "server_capab.h"
 #include "send.h"
-#include "module.h"
 #include "conf_db.h"
 #include "conf_class.h"
 #include "ipcache.h"
-#include "isupport.h"
 #include "extban.h"
-#include "restart.h"
+#include "ircd_exit.h"
+#include "ircd_hook.h"
 #include "flatten_links.h"
 
 /**
@@ -140,24 +144,6 @@ const char *logFileName = LPATH;
  */
 const char *pidFileName = PPATH;
 
-/**
- * @var bool dorehash
- * @brief Flag indicating whether to reload ircd configuration files.
- *
- * This flag is set to true when the ircd receives a SIGHUP signal,
- * indicating the need to reload the configuration files.
- */
-bool dorehash;
-
-/**
- * @var bool doremotd
- * @brief Flag indicating whether to reload the Message of the Day (MOTD) files.
- *
- * This flag is set to true when the ircd receives a SIGUSR1 signal,
- * indicating the need to reload the MOTD files.
- */
-bool doremotd;
-
 /**
  * @var bool printVersion
  * @brief Flag indicating whether to print the version and exit.
@@ -190,10 +176,10 @@ static struct event event_cleanup_tklines =
   .when = 30
 };
 
-static struct event event_try_connections =
+static struct event event_server_connect_auto =
 {
-  .name = "try_connections",
-  .handler = try_connections,
+  .name = "server_connect_auto",
+  .handler = server_connect_auto,
   .when = 15
 };
 
@@ -240,15 +226,15 @@ io_loop(void)
     if (dorehash)
     {
       conf_rehash(true);
-      dorehash = false;
+      dorehash = 0;
     }
 
     if (doremotd)
     {
       motd_recache();
-      sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                           "Got signal SIGUSR1, reloading motd file(s)");
-      doremotd = false;
+      sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                     "Got signal SIGUSR1, reloading motd file(s)");
+      doremotd = 0;
     }
   }
 }
@@ -277,144 +263,32 @@ initialize_global_set_options(void)
  * @brief Handles out-of-memory conditions.
  *
  * This function reports an out-of-memory condition and restarts the program.
- * If it is called more than once, it aborts the program execution.
  */
 static void
 ircd_oom(void)
 {
-  static bool was_here = false;
-
-  if (was_here == false)
-    was_here = true;
-  else
-    abort();
-
-  server_die("out of memory", true);
+  ircd_exit(IRCD_EXIT_RESTART, "out of memory");
 }
 
 /**
- * @brief Writes the process ID to the specified file.
+ * @brief Handles time provider API error conditions.
  *
- * @param filename Path to the process ID file.
+ * This function reports an error condition encountered in the time provider API using the provided
+ * error message. It exits with EXIT_FAILURE if the error code is IO_TIME_ERR_INIT, otherwise, it restarts the program.
  *
- * This function opens the specified file and writes the process ID of
- * the ircd server to it.
+ * @param error_code The error code indicating the type of error in the time provider API.
+ * @param message The error message providing details about the time provider API error.
  */
 static void
-write_pidfile(const char *filename)
+ircd_time_failure(enum io_time_error_code error_code, const char *message)
 {
-  FILE *fb = fopen(filename, "w");
-
-  if (fb)
+  if (error_code == IO_TIME_ERR_INIT)
   {
-    char buf[IRCD_BUFSIZE];
-    unsigned int pid = (unsigned int)getpid();
-
-    snprintf(buf, sizeof(buf), "%u\n", pid);
-
-    if (fputs(buf, fb) == -1)
-      log_write(LOG_TYPE_IRCD, "Error writing to pid file %s: %s",
-                filename, strerror(errno));
-
-    fclose(fb);
-  }
-  else
-    log_write(LOG_TYPE_IRCD, "Error opening pid file %s: %s",
-              filename, strerror(errno));
-}
-
-/**
- * @brief Checks if the daemon is already running using the process ID file.
- *
- * This function reads the process ID from the specified file and checks
- * if the ircd server is already running. If it is, the program exits
- * to prevent multiple instances.
- *
- * @param filename Path to the process ID file.
- */
-static void
-check_pidfile(const char *filename)
-{
-  FILE *fb = fopen(filename, "r");
-
-  if (fb)
-  {
-    char buf[IRCD_BUFSIZE];
-
-    if (fgets(buf, 20, fb) == NULL)
-      log_write(LOG_TYPE_IRCD, "Error reading from pid file %s: %s",
-                filename, strerror(errno));
-    else
-    {
-      pid_t pid = atoi(buf);
-
-      if (kill(pid, 0) == 0)
-      {
-        /* log(L_ERROR, "Server is already running"); */
-        printf("ircd: daemon is already running\n");
-        exit(EXIT_FAILURE);
-      }
-    }
-
-    fclose(fb);
-  }
-  else if (errno != ENOENT)
-    log_write(LOG_TYPE_IRCD, "Error opening pid file %s: %s",
-              filename, strerror(errno));
-}
-
-/**
- * @brief Sets up the core file size to system limits.
- *
- * This function sets the core file size to the maximum allowed by the system.
- */
-static void
-setup_corefile(void)
-{
-  struct rlimit rlim; /* resource limits */
-
-  /* Set corefilesize to maximum */
-  if (getrlimit(RLIMIT_CORE, &rlim) == 0)
-  {
-    rlim.rlim_cur = rlim.rlim_max;
-    setrlimit(RLIMIT_CORE, &rlim);
-  }
-}
-
-/**
- * @brief Sets up the maximum number of file descriptors.
- *
- * This function sets up the maximum number of file descriptors that
- * the ircd server is allowed to use.
- */
-static void
-setup_fdlimit(void)
-{
-  struct rlimit rlim;
-
-  if (getrlimit(RLIMIT_NOFILE, &rlim))
-  {
-    fprintf(stderr, "getrlimit: couldn't get maximum number of file descriptors: %s\n",
-            strerror(errno));
-    exit(EXIT_FAILURE);
+    fprintf(stderr, "ERROR: %s\n", message);
+    exit(EXIT_FAILURE);  /* Exit with failure if initialization error. */
   }
 
-#ifdef __APPLE__
-  if (rlim.rlim_max > OPEN_MAX)
-    rlim.rlim_max = OPEN_MAX;
-#endif
-  if (rlim.rlim_max > 0xFFFF)
-    rlim.rlim_max = 0xFFFF;
-  rlim.rlim_cur = rlim.rlim_max;
-
-  if (setrlimit(RLIMIT_NOFILE, &rlim))
-  {
-    fprintf(stderr, "setrlimit: couldn't set maximum number of file descriptors: %s\n",
-            strerror(errno));
-    exit(EXIT_FAILURE);
-  }
-
-  hard_fdlimit = rlim.rlim_cur;
+  ircd_exit(IRCD_EXIT_RESTART, message);  /* Restart the program for other errors. */
 }
 
 /**
@@ -430,8 +304,8 @@ print_startup(int pid)
 {
   printf("ircd: version %s\n", IRCD_VERSION);
   printf("ircd: pid %d\n", pid);
-  printf("ircd: running in %s mode from %s\n", !server_state.foreground ? "background"
-         : "foreground", ConfigGeneral.dpath);
+  printf("ircd: running in %s mode from %s\n",
+         server_state.foreground ? "foreground": "background", ConfigGeneral.dpath);
 }
 
 /**
@@ -446,8 +320,7 @@ static void
 make_daemon(void)
 {
   int pid = fork();
-
-  if (pid < 0)
+  if (pid == -1)
   {
     perror("fork");
     exit(EXIT_FAILURE);
@@ -466,7 +339,7 @@ make_daemon(void)
 
   /* Connect stdin, stdout, and stderr to /dev/null */
   int fd = open("/dev/null", O_RDWR);
-  if (fd < 0)
+  if (fd == -1)
   {
     perror("open");
     exit(EXIT_FAILURE);
@@ -503,22 +376,34 @@ main(int argc, char *argv[])
 
   io_set_oom_handler(ircd_oom);
 
-  /* Setup corefile size immediately after boot -kre */
-  setup_corefile();
+  io_rlimit_set_max_core();
 
-  setup_fdlimit();
+  io_rlimit_set_max_nofile();
 
-  /* Save server boot time right away, so getrusage works correctly */
-  event_time_set();
+  io_time_set_error_callback(ircd_time_failure);
+  io_time_init();
+
+  /* Configure libjansson to use io_calloc and io_free. */
+  json_set_alloc_funcs(io_calloc, io_free);
+
+  /*
+   * Calculate the seed using multiple sources of entropy
+   * - Real-time and monotonic clocks provide high-resolution time values
+   * - Process ID adds process-specific uniqueness
+   * This combination aims to create a seed that is harder to predict
+   */
+  const uint32_t seed = (uint32_t)(
+     (io_time_get(IO_TIME_REALTIME_SEC) % UINT32_MAX) ^  /* Real-time seconds (modulo to fit in 32 bits) */
+      io_time_get(IO_TIME_REALTIME_NSEC) ^  /* Real-time nanoseconds */
+    ((io_time_get(IO_TIME_MONOTONIC_SEC) % UINT32_MAX) ^  /* Monotonic seconds (modulo to fit in 32 bits) */
+      io_time_get(IO_TIME_MONOTONIC_NSEC) ^  /* Monotonic nanoseconds */
+    ((uint32_t)(getpid() & 0xFFFF) << 16))  /* Masked and shifted PID for added entropy */
+  );
 
-  /* It's not random, but it ought to be a little harder to guess */
-  uint32_t seed = (uint32_t)(event_base->time.sec_real ^
-                             (event_base->time.sec_monotonic | (getpid() << 16)));
   init_genrand(seed);
 
   ConfigGeneral.dpath      = DPATH;
   ConfigGeneral.spath      = SPATH;
-  ConfigGeneral.mpath      = MPATH;
   ConfigGeneral.configfile = CPATH;    /* Server configuration file */
   ConfigGeneral.klinefile  = KPATH;    /* Server kline file         */
   ConfigGeneral.xlinefile  = XPATH;    /* Server xline file         */
@@ -547,7 +432,7 @@ main(int argc, char *argv[])
   else
     make_daemon();
 
-  setup_signals();
+  ircd_signal_init();
 
   /* We need this to initialise the fd array before anything else */
   fdlist_init();
@@ -557,9 +442,10 @@ main(int argc, char *argv[])
   tls_init();
 
   /* Check if there is pidfile and daemon already running */
-  check_pidfile(pidFileName);
+  if (io_pidfile_create(pidFileName))
+    exit(EXIT_FAILURE);
 
-  isupport_init();
+  ircd_hook_init();
   ipcache_init();
   client_init();
   class_init();
@@ -571,11 +457,9 @@ main(int argc, char *argv[])
   extban_init();
   capab_init();  /* Set up default_server_capabs */
   initialize_global_set_options();  /* Has to be called after conf_read_files() */
-  motd_init();
-  user_modes_init();
   flatten_links_init();
 
-  if (EmptyString(ConfigServerInfo.name))
+  if (string_is_empty(ConfigServerInfo.name))
   {
     log_write(LOG_TYPE_IRCD, "ERROR: No server name specified in serverinfo block.");
     exit(EXIT_FAILURE);
@@ -584,7 +468,7 @@ main(int argc, char *argv[])
   strlcpy(me.name, ConfigServerInfo.name, sizeof(me.name));
 
   /* serverinfo {} description must exist.  If not, error out.*/
-  if (EmptyString(ConfigServerInfo.description))
+  if (string_is_empty(ConfigServerInfo.description))
   {
     log_write(LOG_TYPE_IRCD, "ERROR: No server description specified in serverinfo block.");
     exit(EXIT_FAILURE);
@@ -592,7 +476,7 @@ main(int argc, char *argv[])
 
   strlcpy(me.info, ConfigServerInfo.description, sizeof(me.info));
 
-  if (EmptyString(ConfigServerInfo.sid))
+  if (string_is_empty(ConfigServerInfo.sid))
   {
     log_write(LOG_TYPE_IRCD, "Generating server ID");
     generate_sid();
@@ -605,8 +489,8 @@ main(int argc, char *argv[])
 
   me.from = &me;
   me.servptr = &me;
-  me.connection->created_real = event_base->time.sec_real;
-  me.connection->created_monotonic = event_base->time.sec_monotonic;
+  me.connection->created_real = io_time_get(IO_TIME_REALTIME_SEC);
+  me.connection->created_monotonic = io_time_get(IO_TIME_MONOTONIC_SEC);
 
   SetMe(&me);
   server_make(&me);
@@ -623,13 +507,11 @@ main(int argc, char *argv[])
 
   module_load_all(NULL);
 
-  write_pidfile(pidFileName);
-
   event_addish(&event_cleanup_tklines, NULL);
 
-  /* We want try_connections to be called as soon as possible now! -- adrian */
+  /* We want server_connect_auto to be called as soon as possible now! -- adrian */
   /* No, 'cause after a restart it would cause all sorts of nick collides */
-  event_addish(&event_try_connections, NULL);
+  event_addish(&event_server_connect_auto, NULL);
 
   /* Setup the timeout check. I'll shift it later :)  -- adrian */
   event_add(&event_comm_checktimeouts, NULL);
