@@ -2,7 +2,7 @@
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
  *  Copyright (c) 1997 Jukka Santala (Donwulff)
- *  Copyright (c) 2005-2024 ircd-hybrid development team
+ *  Copyright (c) 2005-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,9 +27,9 @@
 #include "stdinc.h"
 #include "list.h"
 #include "memory.h"
+#include "io_string.h"
 #include "client.h"
 #include "hash.h"
-#include "irc_string.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "send.h"
@@ -81,7 +81,6 @@ monitor_find_hash(const char *name)
   LIST_FOREACH(node, monitor_hash[hash_string(name)].head)
   {
     struct Monitor *monitor = node->data;
-
     if (irccmp(monitor->name, name) == 0)
       return monitor;
   }
@@ -141,7 +140,6 @@ monitor_free(struct Monitor *monitor)
   assert(list_find(&monitor_hash[monitor->hash_value], monitor));
 
   list_remove(&monitor->node, &monitor_hash[monitor->hash_value]);
-
   io_free(monitor->name);
   io_free(monitor);
 }
@@ -163,7 +161,6 @@ monitor_add_to_hash_table(const char *name, struct Client *client)
     monitor = io_calloc(sizeof(*monitor));
     monitor->name = io_strdup(name);
     monitor->hash_value = hash_string(monitor->name);
-
     list_add(monitor, &monitor->node, &monitor_hash[monitor->hash_value]);
   }
   else
