@@ -2,7 +2,7 @@
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
  *  Copyright (c) 2000 Kevin L. Mitchell <klmitch@mit.edu>
- *  Copyright (c) 2013-2024 ircd-hybrid development team
+ *  Copyright (c) 2013-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,19 +25,19 @@
  */
 
 #include "stdinc.h"
-#include "defaults.h"
+#include "address.h"
 #include "list.h"
+#include "log.h"
+#include "memory.h"
+#include "misc.h"
+#include "io_string.h"
+#include "defaults.h"
 #include "ircd.h"
 #include "conf.h"
 #include "send.h"
 #include "numeric.h"
 #include "client.h"
-#include "irc_string.h"
-#include "memory.h"
-#include "log.h"
 #include "motd.h"
-#include "address.h"
-#include "misc.h"
 
 /** Global list of messages of the day. */
 static struct
@@ -58,13 +58,13 @@ motd_create(const char *mask, const char *path)
 {
   struct Motd *motd = io_calloc(sizeof(*motd));
 
-  if (EmptyString(mask))
+  if (string_is_empty(mask))
     motd->type = MOTD_UNIVERSAL;
   else if (class_find(mask, true))
     motd->type = MOTD_CLASS;
   else
   {
-    switch (parse_netmask(mask, &motd->address, &motd->addrbits))
+    switch (address_parse_netmask(mask, &motd->address, &motd->addrbits))
     {
       case HM_IPV4:
         motd->type = MOTD_IPMASKV4;
@@ -78,9 +78,7 @@ motd_create(const char *mask, const char *path)
     }
   }
 
-  if (mask)
-    motd->mask = io_strdup(mask);
-
+  motd->mask = mask ? io_strdup(mask) : NULL;
   motd->path = io_strdup(path);
   motd->maxcount = MOTD_MAXLINES;
 
@@ -96,13 +94,8 @@ motd_create(const char *mask, const char *path)
 static struct MotdCache *
 motd_cache(struct Motd *motd)
 {
-  struct stat sb;
-  char line[MOTD_LINESIZE + 2];  /* +2 for \r\n */
-  char *tmp = NULL;
-  unsigned int i = 0;
-
-  assert(motd);
-  assert(motd->path);
+  if (motd == NULL)
+    return NULL;
 
   if (motd->cache)
     return motd->cache;
@@ -121,19 +114,29 @@ motd_cache(struct Motd *motd)
     }
   }
 
-  /* Need the file's modification time */
-  if (stat(motd->path, &sb) == -1)
+  int fd = open(motd->path, O_RDONLY);
+  if (fd == -1)
   {
-    log_write(LOG_TYPE_IRCD, "Couldn't stat \"%s\": %s", motd->path, strerror(errno));
-    return 0;
+    log_write(LOG_TYPE_IRCD, "Couldn't open \"%s\": %s", motd->path, strerror(errno));
+    return NULL;
+  }
+
+  /* Need the file's modification time */
+  struct stat sb;
+  if (fstat(fd, &sb))
+  {
+    log_write(LOG_TYPE_IRCD, "Couldn't fstat \"%s\": %s", motd->path, strerror(errno));
+    close(fd);
+    return NULL;
   }
 
   /* Gotta read in the file, now */
-  FILE *file = fopen(motd->path, "r");
+  FILE *file = fdopen(fd, "r");
   if (file == NULL)
   {
-    log_write(LOG_TYPE_IRCD, "Couldn't open \"%s\": %s", motd->path, strerror(errno));
-    return 0;
+    log_write(LOG_TYPE_IRCD, "Couldn't fdopen \"%s\": %s", motd->path, strerror(errno));
+    close(fd);
+    return NULL;
   }
 
   /* Ok, allocate a structure; we'll realloc later to trim memory */
@@ -143,13 +146,15 @@ motd_cache(struct Motd *motd)
   cache->maxcount = motd->maxcount;
   cache->modtime = sb.st_mtime;  /* Store modtime */
 
+  char line[MOTD_LINESIZE + 2];  /* +2 for \r\n */
   while (cache->count < cache->maxcount && fgets(line, sizeof(line), file))
   {
+    unsigned int i = 0;
     /* Copy over line, stopping when we overflow or hit line end */
-    for (tmp = line, i = 0; i < (MOTD_LINESIZE - 1) && *tmp && *tmp != '\r' && *tmp != '\n'; ++tmp, ++i)
+    for (char *tmp = line; i < (MOTD_LINESIZE - 1) && *tmp && *tmp != '\r' && *tmp != '\n'; ++tmp, ++i)
       cache->motd[cache->count][i] = *tmp;
-    cache->motd[cache->count][i] = '\0';
 
+    cache->motd[cache->count][i] = '\0';
     cache->count++;
   }
 
@@ -173,7 +178,8 @@ motd_cache(struct Motd *motd)
 static void
 motd_decache(struct Motd *motd)
 {
-  assert(motd);
+  if (motd == NULL)
+    return;
 
   struct MotdCache *cache = motd->cache;
   if (cache == NULL)  /* We can be called for records with no cache */
@@ -196,7 +202,8 @@ motd_decache(struct Motd *motd)
 static void
 motd_destroy(struct Motd *motd)
 {
-  assert(motd);
+  if (motd == NULL)
+    return;
 
   if (motd->cache)  /* Drop the cache */
     motd_decache(motd);
@@ -245,7 +252,7 @@ motd_lookup(const struct Client *client)
         break;
       case MOTD_IPMASKV6:
       case MOTD_IPMASKV4:
-        if (address_compare(&client->addr, &motd->address, false, false, motd->addrbits))
+        if (address_match(&client->addr, &motd->address, false, false, motd->addrbits))
           return motd;
         break;
       default: break;
@@ -319,8 +326,11 @@ motd_signon(struct Client *client)
 void
 motd_recache(void)
 {
-  motd_decache(MotdList.local);  /* Decache local and remote MOTDs */
-  motd_decache(MotdList.remote);
+  if (MotdList.local)
+    motd_decache(MotdList.local);  /* Decache local and remote MOTDs */
+
+  if (MotdList.remote)
+    motd_decache(MotdList.remote);
 
   list_node_t *node;
   LIST_FOREACH(node, MotdList.other.head)  /* Now all the others */
@@ -338,15 +348,25 @@ void
 motd_init(void)
 {
   if (MotdList.local)  /* Destroy old local MOTD */
+  {
     motd_destroy(MotdList.local);
-
-  MotdList.local = motd_create(NULL, MPATH);
-  motd_cache(MotdList.local);  /* Initialize local MOTD and cache it */
+    MotdList.local = NULL;
+  }
 
   if (MotdList.remote)  /* Destroy old remote MOTD */
+  {
     motd_destroy(MotdList.remote);
+    MotdList.remote = NULL;
+  }
 
-  MotdList.remote = motd_create(NULL, MPATH);
+  const char *const path = ConfigServerInfo.motd_file;
+  if (string_is_empty(path))
+    return;
+
+  MotdList.local = motd_create(NULL, path);
+  motd_cache(MotdList.local);  /* Initialize local MOTD and cache it */
+
+  MotdList.remote = motd_create(NULL, path);
   motd_cache(MotdList.remote);  /* Initialize remote MOTD and cache it */
 }
 
@@ -358,7 +378,6 @@ void
 motd_add(const char *mask, const char *path)
 {
   struct Motd *motd = motd_create(mask, path);  /* Create the motd */
-
   list_add(motd, &motd->node, &MotdList.other);
 }
 
@@ -406,43 +425,44 @@ motd_report(struct Client *client, int parc, char *parv[])
 void
 motd_memory_count(struct Client *client)
 {
-  unsigned int mt  = 0;  /* Motd count */
-  unsigned int mtc = 0;  /* Motd cache count */
-  size_t mtm  = 0;  /* Memory consumed by motd */
-  size_t mtcm = 0;  /* Memory consumed by motd cache */
+  unsigned int motd_count = 0;
+  size_t motd_bytes = 0;
 
   if (MotdList.local)
   {
-    ++mt;
-    mtm += sizeof(struct Motd);
-    mtm += MotdList.local->path ? (strlen(MotdList.local->path) + 1) : 0;
+    ++motd_count;
+    motd_bytes += sizeof(struct Motd);
+    motd_bytes += MotdList.local->path ? strlen(MotdList.local->path) + 1 : 0;
   }
 
   if (MotdList.remote)
   {
-    ++mt;
-    mtm += sizeof(struct Motd);
-    mtm += MotdList.remote->path ? (strlen(MotdList.remote->path) + 1) : 0;
+    ++motd_count;
+    motd_bytes += sizeof(struct Motd);
+    motd_bytes += MotdList.remote->path ? strlen(MotdList.remote->path) + 1 : 0;
   }
 
   list_node_t *node;
   LIST_FOREACH(node, MotdList.other.head)
   {
     const struct Motd *motd = node->data;
-    ++mt;
-    mtm += sizeof(struct Motd);
-    mtm += motd->path ? (strlen(motd->path) + 1) : 0;
-    mtm += motd->mask ? (strlen(motd->mask) + 1) : 0;
+    ++motd_count;
+    motd_bytes += sizeof(struct Motd);
+    motd_bytes += motd->path ? strlen(motd->path) + 1 : 0;
+    motd_bytes += motd->mask ? strlen(motd->mask) + 1 : 0;
   }
 
+  unsigned int motd_cache_count = 0;
+  size_t motd_cache_bytes = 0;
+
   LIST_FOREACH(node, MotdList.cachelist.head)
   {
     const struct MotdCache *cache = node->data;
-    ++mtc;
-    mtcm += sizeof(struct MotdCache) + (MOTD_LINESIZE * (cache->count - 1));
-    mtcm += cache->path ? (strlen(cache->path) + 1) : 0;
+    ++motd_cache_count;
+    motd_cache_bytes += sizeof(struct MotdCache) + (MOTD_LINESIZE * (cache->count - 1));
+    motd_cache_bytes += cache->path ? strlen(cache->path) + 1 : 0;
   }
 
   sendto_one_numeric(client, &me, RPL_STATSDEBUG | SND_EXPLICIT, "z :Motds %u(%zu) Cache %u(%zu)",
-                     mt, mtm, mtc, mtcm);
+                     motd_count, motd_bytes, motd_cache_count, motd_cache_bytes);
 }
