@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,15 +24,16 @@
  */
 
 #include "stdinc.h"
+#include "log.h"
+#include "io_string.h"
 #include "client.h"
 #include "channel.h"
 #include "hash.h"
 #include "id.h"
-#include "irc_string.h"
 #include "ircd.h"
 #include "numeric.h"
-#include "log.h"
 #include "send.h"
+#include "user_mode.h"
 #include "conf.h"
 #include "packet.h"
 #include "parse.h"
@@ -133,7 +134,7 @@ parse_handle_numeric(unsigned int numeric, struct Client *source, int parc, char
 
     /* Fake it for server hiding, if it's our client */
     if ((ConfigServerHide.hide_servers || IsHidden(source)) && MyConnect(target) &&
-        !HasUMode(target, UMODE_OPER))
+        user_mode_has_flag(target, UMODE_OPER) == false)
       sendto_one_numeric(target, &me, numeric | SND_EXPLICIT, "%s", parv[2]);
     else
       sendto_one_numeric(target, source, numeric | SND_EXPLICIT, "%s", parv[2]);
@@ -166,7 +167,7 @@ parse_handle_command(struct Command *command, struct Client *source,
   /* Check right amount of parameters is passed... --is */
   if (handler->args_min &&
       ((i < handler->args_min) ||
-       (handler->empty_last_arg != true && EmptyString(para[handler->args_min - 1]))))
+       (handler->empty_last_arg != true && string_is_empty(para[handler->args_min - 1]))))
     sendto_one_numeric(source, &me, ERR_NEEDMOREPARAMS, command->name);
   else
     handler->handler(source, i, para);
