@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,19 +24,21 @@
  */
 
 #include "stdinc.h"
+#include "address.h"
+#include "comm.h"
+#include "dbuf.h"
 #include "list.h"
+#include "log.h"
+#include "io_string.h"
+#include "io_time.h"
 #include "send.h"
+#include "cap.h"
 #include "channel.h"
 #include "client.h"
-#include "dbuf.h"
-#include "irc_string.h"
-#include "event.h"
-#include "comm.h"
 #include "server_capab.h"
+#include "user_mode.h"
 #include "conf_class.h"
-#include "log.h"
 #include "conf.h"
-#include "address.h"
 
 static uintmax_t send_marker;
 
@@ -44,13 +46,13 @@ static uintmax_t send_marker;
  *
  * inputs
  *		- buffer
- *		- format pattern to use
+ *		- format format to use
  *		- var args
  * output	- number of bytes formatted output
  * side effects	- modifies sendbuf
  */
 static void
-send_format(struct dbuf_block *buffer, const char *pattern, va_list args)
+send_format(struct dbuf_block *buffer, const char *format, va_list args)
 {
   /*
    * from rfc1459
@@ -64,7 +66,7 @@ send_format(struct dbuf_block *buffer, const char *pattern, va_list args)
    * continuation message lines.  See section 7 for more details about
    * current implementations.
    */
-  dbuf_put_args(buffer, pattern, args);
+  dbuf_put_args(buffer, format, args);
 
   if (buffer->size > IRCD_BUFSIZE - 2)
     buffer->size = IRCD_BUFSIZE - 2;
@@ -79,7 +81,7 @@ send_format(struct dbuf_block *buffer, const char *pattern, va_list args)
  **      sendq.
  */
 static void
-send_message(struct Client *to, struct dbuf_block *buffer)
+sendto_one_buffer(struct Client *to, struct dbuf_block *buffer)
 {
   assert(!IsMe(to));
   assert(to != &me);
@@ -88,11 +90,11 @@ send_message(struct Client *to, struct dbuf_block *buffer)
   if (dbuf_length(&to->connection->buf_sendq) + buffer->size > class_get_sendq(&to->connection->confs))
   {
     if (IsServer(to))
-      sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                           "Max SendQ limit exceeded for %s: %zu > %u",
-                           client_get_name(to, HIDE_IP),
-                           (dbuf_length(&to->connection->buf_sendq) + buffer->size),
-                           class_get_sendq(&to->connection->confs));
+      sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                     "Max SendQ limit exceeded for %s: %zu > %u",
+                     client_get_name(to, HIDE_IP),
+                     (dbuf_length(&to->connection->buf_sendq) + buffer->size),
+                     class_get_sendq(&to->connection->confs));
 
     if (IsClient(to))
       AddFlag(to, FLAGS_SENDQEX);
@@ -125,7 +127,7 @@ send_message(struct Client *to, struct dbuf_block *buffer)
  *
  */
 static void
-send_message_remote(struct Client *to, const struct Client *from, struct dbuf_block *buffer)
+sendto_one_buffer_remote(struct Client *to, const struct Client *from, struct dbuf_block *buffer)
 {
   assert(MyConnect(to));
   assert(IsServer(to));
@@ -133,10 +135,10 @@ send_message_remote(struct Client *to, const struct Client *from, struct dbuf_bl
   assert(to->from == to);
 
   if (to == from->from)
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE, "Send message to %s dropped from %s (Fake Dir)",
-                         to->name, from->name);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "Send message to %s dropped from %s (Fake Dir)",
+                   to->name, from->name);
   else
-    send_message(to, buffer);
+    sendto_one_buffer(to, buffer);
 }
 
 /*
@@ -221,7 +223,7 @@ send_queued_write(struct Client *to)
  * side effects	- send message to single client
  */
 void
-sendto_one(struct Client *to, const char *pattern, ...)
+sendto_one(struct Client *to, const char *format, ...)
 {
   if (IsDead(to->from))
     return;  /* This socket has already been marked as dead */
@@ -229,11 +231,11 @@ sendto_one(struct Client *to, const char *pattern, ...)
   struct dbuf_block *buffer = dbuf_alloc();
 
   va_list args;
-  va_start(args, pattern);
-  send_format(buffer, pattern, args);
+  va_start(args, format);
+  send_format(buffer, format, args);
   va_end(args);
 
-  send_message(to->from, buffer);
+  sendto_one_buffer(to->from, buffer);
 
   dbuf_ref_free(buffer);
 }
@@ -241,12 +243,11 @@ sendto_one(struct Client *to, const char *pattern, ...)
 void
 sendto_one_numeric(struct Client *to, const struct Client *from, enum irc_numerics numeric, ...)
 {
-
   if (IsDead(to->from))
     return;  /* This socket has already been marked as dead */
 
   const char *dest = ID_or_name(to, to);
-  if (EmptyString(dest))
+  if (string_is_empty(dest))
     dest = "*";
 
   struct dbuf_block *buffer = dbuf_alloc();
@@ -264,103 +265,285 @@ sendto_one_numeric(struct Client *to, const struct Client *from, enum irc_numeri
   send_format(buffer, numstr, args);
   va_end(args);
 
-  send_message(to->from, buffer);
+  sendto_one_buffer(to->from, buffer);
 
   dbuf_ref_free(buffer);
 }
 
 void
-sendto_one_notice(struct Client *to, const struct Client *from, const char *pattern, ...)
+sendto_one_notice(struct Client *to, const struct Client *from, const char *format, ...)
 {
   if (IsDead(to->from))
     return;  /* This socket has already been marked as dead */
 
   const char *dest = ID_or_name(to, to);
-  if (EmptyString(dest))
+  if (string_is_empty(dest))
     dest = "*";
 
   struct dbuf_block *buffer = dbuf_alloc();
   dbuf_put_fmt(buffer, ":%s NOTICE %s ", ID_or_name(from, to), dest);
 
   va_list args;
-  va_start(args, pattern);
-  send_format(buffer, pattern, args);
+  va_start(args, format);
+  send_format(buffer, format, args);
   va_end(args);
 
-  send_message(to->from, buffer);
+  sendto_one_buffer(to->from, buffer);
 
   dbuf_ref_free(buffer);
 }
 
-/* sendto_channel_butone()
+/* sendto_anywhere()
  *
- * inputs	- pointer to client(server) to NOT send message to
- *		- pointer to client that is sending this message
- *		- pointer to channel being sent to
- *		- vargs message
+ * inputs	- pointer to dest client
+ * 		- pointer to from client
+ * 		- varags
  * output	- NONE
- * side effects	- message as given is sent to given channel members.
- *
- * WARNING - +D clients are ignored
+ * side effects	- less efficient than sendto_remote and sendto_one
+ * 		  but useful when one does not know where target "lives"
  */
 void
-sendto_channel_butone(struct Client *one, const struct Client *from,
-                      struct Channel *channel, int rank,
-                      const char *pattern, ...)
+sendto_one_anywhere(struct Client *to, const struct Client *from, const char *command, const char *format, ...)
+{
+  if (IsDead(to->from))
+    return;
+
+  struct dbuf_block *buffer = dbuf_alloc();
+  if (MyClient(to) && IsClient(from))
+    dbuf_put_fmt(buffer, ":%s!%s@%s %s %s ",
+                 from->name, from->username, from->host, command, to->name);
+  else
+    dbuf_put_fmt(buffer, ":%s %s %s ",
+                 ID_or_name(from, to), command, ID_or_name(to, to));
+
+  va_list args;
+  va_start(args, format);
+  send_format(buffer, format, args);
+  va_end(args);
+
+  if (MyConnect(to))
+    sendto_one_buffer(to, buffer);
+  else
+    sendto_one_buffer_remote(to->from, from, buffer);
+
+  dbuf_ref_free(buffer);
+}
+
+static bool
+sendto_clients_qualifies(const struct Client *client, uint64_t flags, send_recipient_t recipient)
+{
+  if (flags && user_mode_has_flag(client, flags) == false)
+    return false;
+
+  switch (recipient)
+  {
+    case SEND_RECIPIENT_ADMIN:
+      return user_mode_has_flag(client, UMODE_ADMIN);
+    case SEND_RECIPIENT_OPER:
+      return user_mode_has_flag(client, UMODE_OPER) && user_mode_has_flag(client, UMODE_ADMIN) == false;
+    case SEND_RECIPIENT_OPER_ALL:
+      return user_mode_has_flag(client, UMODE_OPER);
+    case SEND_RECIPIENT_CLIENT:
+      return true;
+    default:
+      return false;
+  }
+}
+
+/* sendto_clients()
+ *
+ * inputs	- flag types of messages to show to real opers
+ *		- flag indicating opers/admins
+ *		- var args input message
+ * output	- NONE
+ * side effects	- Send to *local* ops only but NOT +s nonopers.
+ */
+void
+sendto_clients(uint64_t flags, send_recipient_t recipient, send_type_t type, const char *format, ...)
+{
+  const char *ntype = "???";
+
+  switch (type)
+  {
+    case SEND_TYPE_NOTICE:
+      ntype = "Notice";
+      break;
+    case SEND_TYPE_GLOBAL:
+      ntype = "Global";
+      break;
+    case SEND_TYPE_LOCOPS:
+      ntype = "LocOps";
+      break;
+    default:
+      assert(0);
+  }
+
+  struct dbuf_block *buffer = dbuf_alloc();
+  dbuf_put_fmt(buffer, ":%s NOTICE * :*** %s -- ", me.name, ntype);
+
+  va_list args;
+  va_start(args, format);
+  send_format(buffer, format, args);
+  va_end(args);
+
+  list_t *list = (recipient == SEND_RECIPIENT_CLIENT) ? &local_client_list : &oper_list;
+  list_node_t *node;
+  LIST_FOREACH(node, list->head)
+  {
+    struct Client *client = node->data;
+    if (IsDead(client))
+      continue;
+
+    if (sendto_clients_qualifies(client, flags, recipient) == false)
+      continue;
+
+    sendto_one_buffer(client, buffer);
+  }
+
+  dbuf_ref_free(buffer);
+}
+
+/* ts_warn()
+ *
+ * inputs       - var args message
+ * output       - NONE
+ * side effects - Call sendto_clients, with some flood checking
+ *                (at most 5 warnings every 5 seconds)
+ */
+void
+sendto_clients_ratelimited(uintmax_t *rate, const char *format, ...)
+{
+  if ((io_time_get(IO_TIME_MONOTONIC_SEC) - *rate) < 20)
+    return;
+
+  *rate = io_time_get(IO_TIME_MONOTONIC_SEC);
+
+  char buffer[IRCD_BUFSIZE];
+  va_list args;
+  va_start(args, format);
+  vsnprintf(buffer, sizeof(buffer), format, args);
+  va_end(args);
+
+  sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "%s", buffer);
+  log_write(LOG_TYPE_IRCD, "%s", buffer);
+}
+
+/*
+ ** match_it() and sendto_match_butone() ARE only used
+ ** to send a msg to all ppl on servers/hosts that match a specified mask
+ ** (used for enhanced PRIVMSGs) for opers
+ **
+ ** addition -- Armin, 8jun90 (gruner@informatik.tu-muenchen.de)
+ **
+ */
+
+/* match_it()
+ *
+ * inputs	- client pointer to match on
+ *		- actual mask to match
+ *		- what to match on, HOST or SERVER
+ * output	- 1 or 0 if match or not
+ * side effects	- NONE
+ */
+static bool
+sendto_match_butone_qualifies(const struct Client *one, const char *mask, send_match_type_t type)
+{
+  if (type == SEND_MATCH_HOST)
+  {
+    struct io_addr addr;
+    int bits = 0;
+    const int ret = address_parse_netmask(mask, &addr, &bits);
+
+    if (ret == HM_IPV4 || ret == HM_IPV6)
+      if (address_match(&one->addr, &addr, false, false, bits))
+        return true;
+    return match(mask, one->realhost) == 0;
+  }
+
+  return match(mask, one->servptr->name) == 0;
+}
+
+/* sendto_match_butone()
+ *
+ * Send to all clients which match the mask in a way defined on 'what';
+ * either by user hostname or user servername.
+ *
+ * ugh. ONLY used by m_message.c to send an "oper magic" message. ugh.
+ */
+void
+sendto_match_butone(const struct Client *one, const struct Client *from, const char *mask,
+                    send_match_type_t type, const char *format, ...)
 {
   struct dbuf_block *buffer_l = dbuf_alloc();
   struct dbuf_block *buffer_r = dbuf_alloc();
 
-  if (IsClient(from))
-    dbuf_put_fmt(buffer_l, ":%s!%s@%s ", from->name, from->username, from->host);
-  else
-    dbuf_put_fmt(buffer_l, ":%s ", from->name);
-
+  dbuf_put_fmt(buffer_l, ":%s!%s@%s ", from->name, from->username, from->host);
   dbuf_put_fmt(buffer_r, ":%s ", from->id);
 
   va_list args_l, args_r;
-  va_start(args_l, pattern);
-  va_start(args_r, pattern);
-  send_format(buffer_l, pattern, args_l);
-  send_format(buffer_r, pattern, args_r);
+  va_start(args_l, format);
+  va_start(args_r, format);
+  send_format(buffer_l, format, args_l);
+  send_format(buffer_r, format, args_r);
   va_end(args_l);
   va_end(args_r);
 
-  ++send_marker;
-
+  /* Scan the local clients */
   list_node_t *node;
-  LIST_FOREACH(node, channel->members.head)
+  LIST_FOREACH(node, local_client_list.head)
   {
-    struct ChannelMember *member = node->data;
-    struct Client *target = member->client;
+    struct Client *client = node->data;
 
-    assert(IsClient(target));
-
-    if (IsDead(target->from))
+    if (IsDead(client))
       continue;
 
-    if (one && (target->from == one->from))
+    if (one && (client == one->from))
       continue;
 
-    if (rank && member_highest_rank(member) < rank)
+    if (sendto_match_butone_qualifies(client, mask, type) == false)
       continue;
 
-    if (HasUMode(target, UMODE_DEAF))
+    sendto_one_buffer(client, buffer_l);
+  }
+
+  /* Now scan servers */
+  LIST_FOREACH(node, local_server_list.head)
+  {
+    struct Client *client = node->data;
+
+    /*
+     * The old code looped through every client on the network for each
+     * server to check if the server (client) has at least 1 client
+     * matching the mask, using something like:
+     *
+     * for (target = GlobalClientList; target; target = target->next)
+     *        if (IsRegisteredUser(target) &&
+     *                        match_it(target, mask, what) &&
+     *                        (target->from == client))
+     *   vsendto_prefix_one(client, from, format, args);
+     *
+     * That way, we wouldn't send the message to a server who didn't have
+     * a matching client. However, on a network such as EFNet, that code
+     * would have looped through about 50 servers, and in each loop, loop
+     * through about 50k clients as well, calling match() in each nested
+     * loop. That is a very bad thing cpu wise - just send the message to
+     * every connected server and let that server deal with it.
+     * -wnder
+     */
+    if (IsDead(client))
       continue;
 
-    if (MyConnect(target))
-      send_message(target, buffer_l);
-    else if (target->from->connection->send_marker != send_marker)
-      send_message_remote(target->from, from, buffer_r);
+    if (one && (client == one->from))
+      continue;
 
-    target->from->connection->send_marker = send_marker;
+    sendto_one_buffer_remote(client, from, buffer_r);
   }
 
   dbuf_ref_free(buffer_l);
   dbuf_ref_free(buffer_r);
 }
 
-/* sendto_server()
+/* sendto_servers()
  *
  * inputs       - pointer to client to NOT send to
  *              - pointer to channel
@@ -379,10 +562,8 @@ sendto_channel_butone(struct Client *one, const struct Client *from,
  * -davidt
  */
 void
-sendto_server(const struct Client *one,
-              const unsigned int capab,
-              const unsigned int nocapab,
-              const char *format, ...)
+sendto_servers(const struct Client *one, const unsigned int capab,
+               const unsigned int nocapab, const char *format, ...)
 {
   struct dbuf_block *buffer = dbuf_alloc();
 
@@ -405,14 +586,69 @@ sendto_server(const struct Client *one,
       continue;
 
     /* check we have required capabs */
-    if (IsCapable(client, capab) != capab)
+    if (capab_has_flag(client, capab) != capab)
       continue;
 
     /* check we don't have any forbidden capabs */
-    if (IsCapable(client, nocapab))
+    if (capab_has_flag(client, nocapab))
       continue;
 
-    send_message(client, buffer);
+    sendto_one_buffer(client, buffer);
+  }
+
+  dbuf_ref_free(buffer);
+}
+
+/* sendto_match_servs()
+ *
+ * inputs       - source client
+ *              - mask to send to
+ *              - capab needed
+ *              - data
+ * outputs	- none
+ * side effects	- data sent to servers matching with capab
+ */
+void
+sendto_match_servs(const struct Client *source, const char *mask, unsigned int capab, const char *format, ...)
+{
+  struct dbuf_block *buffer = dbuf_alloc();
+
+  dbuf_put_fmt(buffer, ":%s ", source->id);
+
+  va_list args;
+  va_start(args, format);
+  send_format(buffer, format, args);
+  va_end(args);
+
+  ++send_marker;
+
+  list_node_t *node;
+  LIST_FOREACH(node, global_server_list.head)
+  {
+    struct Client *target = node->data;
+
+    if (IsDead(target->from))
+      continue;
+
+    /* Do not attempt to send to ourselves ... */
+    if (IsMe(target))
+      continue;
+
+    /* ... or the source */
+    if (target->from == source->from)
+      continue;
+
+    if (target->from->connection->send_marker == send_marker)
+      continue;
+
+    if (capab_has_flag(target->from, capab) != capab)
+      continue;
+
+    if (match(mask, target->name))
+      continue;
+
+    target->from->connection->send_marker = send_marker;
+    sendto_one_buffer_remote(target->from, source, buffer);
   }
 
   dbuf_ref_free(buffer);
@@ -421,7 +657,7 @@ sendto_server(const struct Client *one,
 /* sendto_common_channels_local()
  *
  * inputs	- pointer to client
- *		- pattern to send
+ *		- format to send
  * output	- NONE
  * side effects	- Sends a message to all people on local server who are
  * 		  in same channel with user.
@@ -429,13 +665,13 @@ sendto_server(const struct Client *one,
  */
 void
 sendto_common_channels_local(struct Client *user, bool touser, unsigned int poscap,
-                             unsigned int negcap, const char *pattern, ...)
+                             unsigned int negcap, const char *format, ...)
 {
   struct dbuf_block *buffer = dbuf_alloc();
 
   va_list args;
-  va_start(args, pattern);
-  send_format(buffer, pattern, args);
+  va_start(args, format);
+  send_format(buffer, format, args);
   va_end(args);
 
   ++send_marker;
@@ -467,13 +703,13 @@ sendto_common_channels_local(struct Client *user, bool touser, unsigned int posc
         continue;
 
       target->connection->send_marker = send_marker;
-      send_message(target, buffer);
+      sendto_one_buffer(target, buffer);
     }
   }
 
   if (touser && MyConnect(user) && !IsDead(user))
     if (HasCap(user, poscap) == poscap)
-      send_message(user, buffer);
+      sendto_one_buffer(user, buffer);
 
   dbuf_ref_free(buffer);
 }
@@ -484,17 +720,17 @@ sendto_common_channels_local(struct Client *user, bool touser, unsigned int posc
  * \param rank     Minimum channel member rank clients must have
  * \param poscap   Positive client capabilities flags (CAP)
  * \param negcap   Negative client capabilities flags (CAP)
- * \param pattern  Format string for command arguments
+ * \param format  Format string for command arguments
  */
 void
-sendto_channel_local(const struct Client *one, struct Channel *channel, int rank,
-                     unsigned int poscap, unsigned int negcap, const char *pattern, ...)
+sendto_channel_local(const struct Client *one, struct Channel *channel, int rank, unsigned int poscap,
+                     unsigned int negcap, const char *format, ...)
 {
   struct dbuf_block *buffer = dbuf_alloc();
 
   va_list args;
-  va_start(args, pattern);
-  send_format(buffer, pattern, args);
+  va_start(args, format);
+  send_format(buffer, format, args);
   va_end(args);
 
   list_node_t *node;
@@ -518,347 +754,75 @@ sendto_channel_local(const struct Client *one, struct Channel *channel, int rank
     if (negcap && HasCap(target, negcap))
       continue;
 
-    send_message(target, buffer);
+    sendto_one_buffer(target, buffer);
   }
 
   dbuf_ref_free(buffer);
 }
 
-/*
- ** match_it() and sendto_match_butone() ARE only used
- ** to send a msg to all ppl on servers/hosts that match a specified mask
- ** (used for enhanced PRIVMSGs) for opers
- **
- ** addition -- Armin, 8jun90 (gruner@informatik.tu-muenchen.de)
- **
- */
-
-/* match_it()
+/* sendto_channel_butone()
  *
- * inputs	- client pointer to match on
- *		- actual mask to match
- *		- what to match on, HOST or SERVER
- * output	- 1 or 0 if match or not
- * side effects	- NONE
- */
-static bool
-match_it(const struct Client *one, const char *mask, bool host)
-{
-  if (host)
-  {
-    struct io_addr addr;
-    int bits = 0;
-    const int ret = parse_netmask(mask, &addr, &bits);
-
-    if (ret == HM_IPV4 || ret == HM_IPV6)
-      if (address_compare(&one->addr, &addr, false, false, bits))
-        return true;
-    return match(mask, one->realhost) == 0;
-  }
-
-  return match(mask, one->servptr->name) == 0;
-}
-
-/* sendto_match_butone()
+ * inputs	- pointer to client(server) to NOT send message to
+ *		- pointer to client that is sending this message
+ *		- pointer to channel being sent to
+ *		- vargs message
+ * output	- NONE
+ * side effects	- message as given is sent to given channel members.
  *
- * Send to all clients which match the mask in a way defined on 'what';
- * either by user hostname or user servername.
- *
- * ugh. ONLY used by m_message.c to send an "oper magic" message. ugh.
+ * WARNING - +D clients are ignored
  */
 void
-sendto_match_butone(const struct Client *one, const struct Client *from,
-                    const char *mask, bool host, const char *pattern, ...)
+sendto_channel_butone(struct Client *one, const struct Client *from, struct Channel *channel,
+                      int rank, const char *format, ...)
 {
   struct dbuf_block *buffer_l = dbuf_alloc();
   struct dbuf_block *buffer_r = dbuf_alloc();
 
-  dbuf_put_fmt(buffer_l, ":%s!%s@%s ", from->name, from->username, from->host);
+  if (IsClient(from))
+    dbuf_put_fmt(buffer_l, ":%s!%s@%s ", from->name, from->username, from->host);
+  else
+    dbuf_put_fmt(buffer_l, ":%s ", from->name);
+
   dbuf_put_fmt(buffer_r, ":%s ", from->id);
 
   va_list args_l, args_r;
-  va_start(args_l, pattern);
-  va_start(args_r, pattern);
-  send_format(buffer_l, pattern, args_l);
-  send_format(buffer_r, pattern, args_r);
+  va_start(args_l, format);
+  va_start(args_r, format);
+  send_format(buffer_l, format, args_l);
+  send_format(buffer_r, format, args_r);
   va_end(args_l);
   va_end(args_r);
 
-  /* Scan the local clients */
+  ++send_marker;
+
   list_node_t *node;
-  LIST_FOREACH(node, local_client_list.head)
+  LIST_FOREACH(node, channel->members.head)
   {
-    struct Client *client = node->data;
+    struct ChannelMember *member = node->data;
+    struct Client *target = member->client;
 
-    if (IsDead(client))
+    assert(IsClient(target));
+
+    if (IsDead(target->from))
       continue;
 
-    if (one && (client == one->from))
+    if (one && (target->from == one->from))
       continue;
 
-    if (match_it(client, mask, host) == false)
+    if (rank && member_highest_rank(member) < rank)
       continue;
 
-    send_message(client, buffer_l);
-  }
-
-  /* Now scan servers */
-  LIST_FOREACH(node, local_server_list.head)
-  {
-    struct Client *client = node->data;
-
-    /*
-     * The old code looped through every client on the network for each
-     * server to check if the server (client) has at least 1 client
-     * matching the mask, using something like:
-     *
-     * for (target = GlobalClientList; target; target = target->next)
-     *        if (IsRegisteredUser(target) &&
-     *                        match_it(target, mask, what) &&
-     *                        (target->from == client))
-     *   vsendto_prefix_one(client, from, pattern, args);
-     *
-     * That way, we wouldn't send the message to a server who didn't have
-     * a matching client. However, on a network such as EFNet, that code
-     * would have looped through about 50 servers, and in each loop, loop
-     * through about 50k clients as well, calling match() in each nested
-     * loop. That is a very bad thing cpu wise - just send the message to
-     * every connected server and let that server deal with it.
-     * -wnder
-     */
-    if (IsDead(client))
+    if (user_mode_has_flag(target, UMODE_DEAF))
       continue;
 
-    if (one && (client == one->from))
-      continue;
+    if (MyConnect(target))
+      sendto_one_buffer(target, buffer_l);
+    else if (target->from->connection->send_marker != send_marker)
+      sendto_one_buffer_remote(target->from, from, buffer_r);
 
-    send_message_remote(client, from, buffer_r);
+    target->from->connection->send_marker = send_marker;
   }
 
   dbuf_ref_free(buffer_l);
   dbuf_ref_free(buffer_r);
 }
-
-/* sendto_match_servs()
- *
- * inputs       - source client
- *              - mask to send to
- *              - capab needed
- *              - data
- * outputs	- none
- * side effects	- data sent to servers matching with capab
- */
-void
-sendto_match_servs(const struct Client *source, const char *mask, unsigned int capab,
-                   const char *pattern, ...)
-{
-  struct dbuf_block *buffer = dbuf_alloc();
-
-  dbuf_put_fmt(buffer, ":%s ", source->id);
-
-  va_list args;
-  va_start(args, pattern);
-  send_format(buffer, pattern, args);
-  va_end(args);
-
-  ++send_marker;
-
-  list_node_t *node;
-  LIST_FOREACH(node, global_server_list.head)
-  {
-    struct Client *target = node->data;
-
-    if (IsDead(target->from))
-      continue;
-
-    /* Do not attempt to send to ourselves ... */
-    if (IsMe(target))
-      continue;
-
-    /* ... or the source */
-    if (target->from == source->from)
-      continue;
-
-    if (target->from->connection->send_marker == send_marker)
-      continue;
-
-    if (IsCapable(target->from, capab) != capab)
-      continue;
-
-    if (match(mask, target->name))
-      continue;
-
-    target->from->connection->send_marker = send_marker;
-    send_message_remote(target->from, source, buffer);
-  }
-
-  dbuf_ref_free(buffer);
-}
-
-/* sendto_anywhere()
- *
- * inputs	- pointer to dest client
- * 		- pointer to from client
- * 		- varags
- * output	- NONE
- * side effects	- less efficient than sendto_remote and sendto_one
- * 		  but useful when one does not know where target "lives"
- */
-void
-sendto_anywhere(struct Client *to, const struct Client *from,
-                const char *command,
-                const char *pattern, ...)
-{
-  if (IsDead(to->from))
-    return;
-
-  struct dbuf_block *buffer = dbuf_alloc();
-  if (MyClient(to) && IsClient(from))
-    dbuf_put_fmt(buffer, ":%s!%s@%s %s %s ",
-                 from->name, from->username, from->host, command, to->name);
-  else
-    dbuf_put_fmt(buffer, ":%s %s %s ",
-                 ID_or_name(from, to), command, ID_or_name(to, to));
-
-  va_list args;
-  va_start(args, pattern);
-  send_format(buffer, pattern, args);
-  va_end(args);
-
-  if (MyConnect(to))
-    send_message(to, buffer);
-  else
-    send_message_remote(to->from, from, buffer);
-
-  dbuf_ref_free(buffer);
-}
-
-/* sendto_realops_flags()
- *
- * inputs	- flag types of messages to show to real opers
- *		- flag indicating opers/admins
- *		- var args input message
- * output	- NONE
- * side effects	- Send to *local* ops only but NOT +s nonopers.
- */
-void
-sendto_realops_flags(unsigned int flags, int level, int type, const char *pattern, ...)
-{
-  const char *ntype = "???";
-
-  switch (type)
-  {
-    case SEND_NOTICE:
-      ntype = "Notice";
-      break;
-    case SEND_GLOBAL:
-      ntype = "Global";
-      break;
-    case SEND_LOCOPS:
-      ntype = "LocOps";
-      break;
-    default:
-      assert(0);
-  }
-
-  struct dbuf_block *buffer = dbuf_alloc();
-  dbuf_put_fmt(buffer, ":%s NOTICE * :*** %s -- ", me.name, ntype);
-
-  va_list args;
-  va_start(args, pattern);
-  send_format(buffer, pattern, args);
-  va_end(args);
-
-  list_node_t *node;
-  LIST_FOREACH(node, oper_list.head)
-  {
-    struct Client *client = node->data;
-    assert(HasUMode(client, UMODE_OPER));
-
-    if (IsDead(client))
-      continue;
-
-    /*
-     * If we're sending it to opers and they're an admin, skip.
-     * If we're sending it to admins, and they're not, skip.
-     */
-    if (((level == L_ADMIN) && !HasUMode(client, UMODE_ADMIN)) ||
-        ((level == L_OPER) && HasUMode(client, UMODE_ADMIN)))
-      continue;
-
-    if (!HasUMode(client, flags))
-      continue;
-
-    send_message(client, buffer);
-  }
-
-  dbuf_ref_free(buffer);
-}
-
-/* ts_warn()
- *
- * inputs       - var args message
- * output       - NONE
- * side effects - Call sendto_realops_flags, with some flood checking
- *                (at most 5 warnings every 5 seconds)
- */
-void
-sendto_realops_flags_ratelimited(uintmax_t *rate, const char *pattern, ...)
-{
-  char buffer[IRCD_BUFSIZE];
-
-  if ((event_base->time.sec_monotonic - *rate) < 20)
-    return;
-
-  *rate = event_base->time.sec_monotonic;
-
-  va_list args;
-  va_start(args, pattern);
-  vsnprintf(buffer, sizeof(buffer), pattern, args);
-  va_end(args);
-
-  sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE, "%s", buffer);
-  log_write(LOG_TYPE_IRCD, "%s", buffer);
-}
-
-/* sendto_wallops_flags()
- *
- * inputs       - flag types of messages to show to real opers
- *              - client sending request
- *              - var args input message
- * output       - NONE
- * side effects - Send a wallops to local opers
- */
-void
-sendto_wallops_flags(unsigned int flags, const struct Client *source,
-                     const char *pattern, ...)
-{
-  struct dbuf_block *buffer = dbuf_alloc();
-
-  if (IsClient(source))
-    dbuf_put_fmt(buffer, ":%s!%s@%s WALLOPS :", source->name, source->username, source->host);
-  else
-    dbuf_put_fmt(buffer, ":%s WALLOPS :", source->name);
-
-  va_list args;
-  va_start(args, pattern);
-  send_format(buffer, pattern, args);
-  va_end(args);
-
-  list_node_t *node;
-  LIST_FOREACH(node, oper_list.head)
-  {
-    struct Client *client = node->data;
-    assert(client->umodes & UMODE_OPER);
-
-    if (IsDead(client))
-      continue;
-
-    if (!HasUMode(client, flags))
-      continue;
-
-    send_message(client, buffer);
-  }
-
-  dbuf_ref_free(buffer);
-}
