@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,74 +24,67 @@
  */
 
 #include "stdinc.h"
+#include "comm.h"
 #include "list.h"
+#include "log.h"
+#include "memory.h"
+#include "io_string.h"
+#include "io_time.h"
 #include "client.h"
-#include "event.h"
 #include "hash.h"
-#include "irc_string.h"
 #include "ircd.h"
 #include "ircd_defs.h"
-#include "comm.h"
 #include "packet.h"
 #include "conf.h"
 #include "server.h"
 #include "server_capab.h"
-#include "log.h"
 #include "send.h"
-#include "memory.h"
 #include "parse.h"
 
-
-static void server_connect_callback(fde_t *, int, void *);
-
-/* server_hunt()
- *      Do the basic thing in delivering the message (command)
- *      across the relays to the specific server (server) for
- *      actions.
+/**
+ * @brief Routes a command to the appropriate server or client.
  *
- *      Note:   The command is a format string and *MUST* be
- *              of prefixed style (e.g. ":%s COMMAND %s ...").
- *              Command can have only max 8 parameters.
+ * This function attempts to deliver a command to the specified server or client.
+ * The command is a format string that must be prefixed (e.g., ":%s COMMAND %s ...").
+ * The command can have up to 8 parameters.
  *
- *      server  parv[server] is the parameter identifying the
- *              target server.
- *
- *      *WARNING*
- *              parv[server] is replaced with the pointer to the
- *              real servername from the matched client (I'm lazy
- *              now --msa).
- *
- *      returns: (see #defines)
+ * @param client The client issuing the command.
+ * @param command The command to be delivered.
+ * @param server The index in the parv array that identifies the target server.
+ * @param parv The parameter array for the command, where parv[server] is the target server identifier.
+ * @return A pointer to a server_route_t structure containing the result of the routing attempt.
  */
-const struct server_hunt *
-server_hunt(struct Client *client, const char *command, const int server, char *parv[])
+const server_route_t *
+server_route_command(struct Client *client, const char *command, const int server, char *parv[])
 {
-  static struct server_hunt hunt;
-  struct server_hunt *const h = &hunt;
+  static server_route_t r;
+  server_route_t *const route = &r;
   const char *const mask = parv[server];
 
-  /* Assume it's me, if no server */
-  if (EmptyString(mask))
+  /* Assume it's this server if no target is specified. */
+  if (string_is_empty(mask))
   {
-    h->target = &me;
-    h->ret = HUNTED_ISME;
-    return h;
+    route->target = &me;
+    route->result = SERVER_ROUTE_ISME;
+    return route;
   }
 
-  h->target = find_person(client, mask);
-  if (h->target == NULL)
-    h->target = hash_find_server(mask);
+  /* Attempt to find the target as a person or server. */
+  route->target = find_person(client, mask);
+  if (route->target == NULL)
+    route->target = hash_find_server(mask);
 
   /*
-   * These are to pickup matches that would cause the following
-   * message to go in the wrong direction while doing quick fast
-   * non-matching lookups.
+   * Ensure the target is not from the same upstream server as the client,
+   * unless the target is directly connected to this server. This prevents
+   * incorrect routing of the message back upstream.
    */
-  if (h->target)
-    if (h->target->from == client->from && !MyConnect(h->target))
-      h->target = NULL;
+  if (route->target)
+    if (route->target->from == client->from && !MyConnect(route->target))
+      route->target = NULL;
 
-  if (h->target == NULL && has_wildcards(mask))
+  /* Handle wildcard matches if no exact match was found. */
+  if (route->target == NULL && has_wildcards(mask))
   {
     list_node_t *node;
     LIST_FOREACH(node, global_server_list.head)
@@ -104,12 +97,12 @@ server_hunt(struct Client *client, const char *command, const int server, char *
         if (tmp->from == client->from && !MyConnect(tmp))
           continue;
 
-        h->target = tmp;
+        route->target = tmp;
         break;
       }
     }
 
-    if (h->target == NULL)
+    if (route->target == NULL)
     {
       LIST_FOREACH(node, global_client_list.head)
       {
@@ -121,117 +114,35 @@ server_hunt(struct Client *client, const char *command, const int server, char *
           if (tmp->from == client->from && !MyConnect(tmp))
             continue;
 
-          h->target = tmp;
+          route->target = tmp;
           break;
         }
       }
     }
   }
 
-  if (h->target)
+  /* Determine the result of the routing attempt. */
+  if (route->target)
   {
-    assert(IsMe(h->target) || IsServer(h->target) || IsClient(h->target));
-    if (IsMe(h->target) || MyClient(h->target))
+    assert(IsMe(route->target) || IsServer(route->target) || IsClient(route->target));
+    if (IsMe(route->target) || MyClient(route->target))
     {
-      h->ret = HUNTED_ISME;
-      return h;
+      route->result = SERVER_ROUTE_ISME;
+      return route;
     }
 
-    parv[server] = h->target->id;
-    sendto_one(h->target, command, client->id,
+    parv[server] = route->target->id;
+    sendto_one(route->target, command, client->id,
                parv[1], parv[2], parv[3], parv[4],
                parv[5], parv[6], parv[7], parv[8]);
-    h->ret = HUNTED_PASS;
-    return h;
+    route->result = SERVER_ROUTE_PASS;
+    return route;
   }
 
+  /* Handle case where the target was not found. */
   sendto_one_numeric(client, &me, ERR_NOSUCHSERVER, mask);
-  h->ret = HUNTED_NOSUCH;
-  return h;
-}
-
-/* try_connections()
- *
- * inputs	- void pointer which is not used
- * output	- NONE
- * side effects	-
- * scan through configuration and try new connections.
- * Returns the calendar time when the next call to this
- * function should be made latest. (No harm done if this
- * is called earlier or later...)
- */
-void
-try_connections(void *unused)
-{
-  if (GlobalSetOptions.autoconnect == false)
-    return;
-
-  list_node_t *node;
-  LIST_FOREACH(node, connect_items.head)
-  {
-    struct MaskItem *conf = node->data;
-
-    assert(conf->type == CONF_SERVER);
-    assert(conf->class);
-
-    /* Also when already connecting! (update holdtimes) --SRB */
-    if (conf->port == 0 || !IsConfAllowAutoConn(conf))
-      continue;
-
-    /*
-     * Skip this entry if the use of it is still on hold until
-     * future. Otherwise handle this entry (and set it on hold
-     * until next time). Will reset only hold times, if already
-     * made one successfull connection... [this algorithm is
-     * a bit fuzzy... -- msa >;) ]
-     */
-    if (conf->until > event_base->time.sec_monotonic)
-      continue;
-
-    conf->until = event_base->time.sec_monotonic + conf->class->con_freq;
-
-    /*
-     * Found a CONNECT config with port specified, scan clients
-     * and see if this server is already connected?
-     */
-    if (hash_find_server(conf->name))
-      continue;
-
-    if (conf->class->ref_count < conf->class->max_total)
-    {
-      /* Move this entry to the end of the list, if not already last */
-      if (node->next)
-      {
-        list_remove(node, &connect_items);
-        list_add_tail(conf, &conf->node, &connect_items);
-      }
-
-      if (find_servconn_in_progress(conf->name))
-        return;
-
-      /*
-       * We used to only print this if server_connect() actually
-       * succeeded, but since comm_tcp_connect() can call the callback
-       * immediately if there is an error, we were getting error messages
-       * in the wrong order. SO, we just print out the activated line,
-       * and let server_connect() / server_connect_callback() print an
-       * error afterwards if it fails.
-       *   -- adrian
-       */
-      if (ConfigServerHide.hide_server_ips)
-        sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                             "Connection to %s activated.",
-                             conf->name);
-      else
-        sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                             "Connection to %s[%s] activated.",
-                             conf->name, conf->host);
-
-      server_connect(conf, NULL);
-      /* We connect only one at time... */
-      return;
-    }
-  }
+  route->result = SERVER_ROUTE_NOSUCH;
+  return route;
 }
 
 bool
@@ -268,106 +179,6 @@ server_make(struct Client *client)
   return client->serv;
 }
 
-/* server_connect() - initiate a server connection
- *
- * inputs	- pointer to conf
- *		- pointer to client doing the connect
- * output	-
- * side effects	-
- *
- * This code initiates a connection to a server. It first checks to make
- * sure the given server exists. If this is the case, it creates a socket,
- * creates a client, saves the socket information in the client, and
- * initiates a connection to the server through comm_connect_tcp(). The
- * completion of this goes through serv_completed_connection().
- *
- * We return 1 if the connection is attempted, since we don't know whether
- * it suceeded or not, and 0 if it fails in here somewhere.
- */
-bool
-server_connect(struct MaskItem *conf, struct Client *by)
-{
-  assert(conf);
-  assert(conf->type == CONF_SERVER);
-  assert(hash_find_server(conf->name) == NULL);  /* This should have been checked by the caller */
-
-  /* Still processing a DNS lookup? -> exit */
-  if (conf->dns_pending)
-  {
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                         "Error connecting to %s: DNS lookup for connect{} in progress.",
-                         conf->name);
-    return false;
-  }
-
-  if (conf->dns_failed)
-  {
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                         "Error connecting to %s: DNS lookup for connect{} failed.",
-                         conf->name);
-    return false;
-  }
-
-  char buf[HOSTIPLEN + 1];
-  getnameinfo((const struct sockaddr *)conf->addr, conf->addr->ss_len,
-              buf, sizeof(buf), NULL, 0, NI_NUMERICHOST);
-  log_write(LOG_TYPE_IRCD, "Connect to %s[%s] @%s", conf->name, conf->host, buf);
-
-  /* Create a socket for the server connection */
-  int fd = comm_socket(conf->addr->ss.ss_family, SOCK_STREAM, 0);
-  if (fd == -1)
-  {
-    /* Eek, failure to create the socket */
-    log_write(LOG_TYPE_IRCD, "opening stream socket to %s: %s",
-              conf->name, strerror(errno));
-    return false;
-  }
-
-  /* Create a local client */
-  struct Client *client = client_make(NULL);
-
-  /* Copy in the server, hostname, fd */
-  strlcpy(client->name, conf->name, sizeof(client->name));
-  strlcpy(client->host, conf->host, sizeof(client->host));
-
-  /* We already converted the ip once, so lets use it - stu */
-  strlcpy(client->sockhost, buf, sizeof(client->sockhost));
-
-  client->addr = *conf->addr;
-  client->connection->fd = fd_open(fd, true, NULL);
-
-  /* Server names are always guaranteed under HOSTLEN chars */
-  fd_note(client->connection->fd, "Server: %s", client->name);
-
-  /*
-   * Attach config entries to client here rather than in server_connect_callback().
-   * This to avoid null pointer references.
-   */
-  conf_attach(client, conf);
-
-  server_make(client);
-
-  if (by && IsClient(by))
-    strlcpy(client->serv->by, by->name, sizeof(client->serv->by));
-  else
-    strlcpy(client->serv->by, "AutoConn.", sizeof(client->serv->by));
-
-  SetConnecting(client);
-
-  /* Now, initiate the connection */
-  comm_connect_tcp(client->connection->fd, conf->addr, conf->port, conf->bind,
-                   server_connect_callback, client, conf->timeout);
-
-  /*
-   * At this point we have a connection in progress and a connect {} block
-   * attached to the client, the socket info should be saved in the client
-   * and it should either be resolved or have a valid address.
-   *
-   * The socket has been connected or connect is in progress.
-   */
-  return true;
-}
-
 static void
 server_finish_tls_handshake(struct Client *client)
 {
@@ -375,14 +186,14 @@ server_finish_tls_handshake(struct Client *client)
                                                 client->name, CONF_SERVER);
   if (conf == NULL)
   {
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE,
-                         "Lost connect{} block for %s",
-                         client_get_name(client, SHOW_IP));
-    sendto_realops_flags(UMODE_SERVNOTICE, L_OPER, SEND_NOTICE,
-                         "Lost connect{} block for %s",
-                         client_get_name(client, MASK_IP));
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE,
+                   "Lost connect{} block for %s",
+                   client_get_name(client, SHOW_IP));
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER, SEND_TYPE_NOTICE,
+                   "Lost connect{} block for %s",
+                   client_get_name(client, MASK_IP));
 
-    exit_client(client, "Lost connect{} block");
+    client_exit(client, "Lost connect{} block");
     return;
   }
 
@@ -401,9 +212,9 @@ server_finish_tls_handshake(struct Client *client)
 }
 
 static void
-server_tls_handshake(fde_t *F, void *data)
+server_tls_handshake(fde_t *F, void *data_)
 {
-  struct Client *client = data;
+  struct Client *client = data_;
   const char *sslerr = NULL;
 
   assert(client);
@@ -414,9 +225,9 @@ server_tls_handshake(fde_t *F, void *data)
   tls_handshake_status_t ret = tls_handshake(&F->tls, TLS_ROLE_CLIENT, &sslerr);
   if (ret != TLS_HANDSHAKE_DONE)
   {
-    if ((event_base->time.sec_monotonic - client->connection->created_monotonic) > TLS_HANDSHAKE_TIMEOUT)
+    if ((io_time_get(IO_TIME_MONOTONIC_SEC) - client->connection->created_monotonic) > TLS_HANDSHAKE_TIMEOUT)
     {
-      exit_client(client, "Timeout during TLS handshake");
+      client_exit(client, "Timeout during TLS handshake");
       return;
     }
 
@@ -432,10 +243,9 @@ server_tls_handshake(fde_t *F, void *data)
         return;
       default:
       {
-        sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                             "Error connecting to %s: %s", client->name,
-                             sslerr ? sslerr : "unknown TLS error");
-        exit_client(client, "Error during TLS handshake");
+        sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "Error connecting to %s: %s",
+                       client->name, sslerr ? sslerr : "unknown TLS error");
+        client_exit(client, "Error during TLS handshake");
         return;
       }
     }
@@ -444,7 +254,7 @@ server_tls_handshake(fde_t *F, void *data)
   comm_settimeout(F, 0, NULL, NULL);
   comm_setselect(F, COMM_SELECT_WRITE | COMM_SELECT_READ, NULL, NULL, 0);
 
-  if (tls_verify_certificate(&F->tls, ConfigServerInfo.message_digest_algorithm, &client->tls_certfp) == false)
+  if (tls_verify_certificate(&F->tls, &client->tls_certfp) == false)
     log_write(LOG_TYPE_IRCD, "Server %s gave bad TLS client certificate",
               client_get_name(client, MASK_IP));
 
@@ -462,11 +272,11 @@ server_tls_connect_init(struct Client *client, const struct MaskItem *conf, fde_
   if (tls_new(&F->tls, F->fd, TLS_ROLE_CLIENT) == false)
   {
     SetDead(client);
-    exit_client(client, "TLS context initialization failed");
+    client_exit(client, "TLS context initialization failed");
     return;
   }
 
-  if (!EmptyString(conf->cipher_list))
+  if (!string_is_empty(conf->cipher_list))
     tls_set_ciphers(&F->tls, conf->cipher_list);
 
   server_tls_handshake(F, client);
@@ -481,9 +291,9 @@ server_tls_connect_init(struct Client *client, const struct MaskItem *conf, fde_
  * marked for reading.
  */
 static void
-server_connect_callback(fde_t *F, int status, void *data)
+server_connect_callback(fde_t *F, int status, void *data_)
 {
-  struct Client *const client = data;
+  struct Client *const client = data_;
 
   /* First, make sure it's a real client! */
   assert(client);
@@ -495,12 +305,12 @@ server_connect_callback(fde_t *F, int status, void *data)
   if (status != COMM_OK)
   {
     /* We have an error, so report it and quit */
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE,
-                         "Error connecting to %s: %s",
-                         client_get_name(client, SHOW_IP), comm_errstr(status));
-    sendto_realops_flags(UMODE_SERVNOTICE, L_OPER, SEND_NOTICE,
-                         "Error connecting to %s: %s",
-                         client_get_name(client, MASK_IP), comm_errstr(status));
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE,
+                   "Error connecting to %s: %s",
+                   client_get_name(client, SHOW_IP), comm_errstr(status));
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER, SEND_TYPE_NOTICE,
+                   "Error connecting to %s: %s",
+                   client_get_name(client, MASK_IP), comm_errstr(status));
 
     /*
      * If a fd goes bad, call dead_link() the socket is no
@@ -516,12 +326,12 @@ server_connect_callback(fde_t *F, int status, void *data)
                                                 client->name, CONF_SERVER);
   if (conf == NULL)
   {
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE,
-                         "Lost connect{} block for %s", client_get_name(client, SHOW_IP));
-    sendto_realops_flags(UMODE_SERVNOTICE, L_OPER, SEND_NOTICE,
-                         "Lost connect{} block for %s", client_get_name(client, MASK_IP));
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_ADMIN, SEND_TYPE_NOTICE,
+                   "Lost connect{} block for %s", client_get_name(client, SHOW_IP));
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER, SEND_TYPE_NOTICE,
+                   "Lost connect{} block for %s", client_get_name(client, MASK_IP));
 
-    exit_client(client, "Lost connect{} block");
+    client_exit(client, "Lost connect{} block");
     return;
   }
 
@@ -545,6 +355,181 @@ server_connect_callback(fde_t *F, int status, void *data)
   read_packet(client->connection->fd, client);
 }
 
+/* server_connect() - initiate a server connection
+ *
+ * inputs	- pointer to conf
+ *		- pointer to client doing the connect
+ * output	-
+ * side effects	-
+ *
+ * This code initiates a connection to a server. It first checks to make
+ * sure the given server exists. If this is the case, it creates a socket,
+ * creates a client, saves the socket information in the client, and
+ * initiates a connection to the server through comm_connect_tcp(). The
+ * completion of this goes through serv_completed_connection().
+ *
+ * We return 1 if the connection is attempted, since we don't know whether
+ * it suceeded or not, and 0 if it fails in here somewhere.
+ */
+bool
+server_connect(struct MaskItem *conf, struct Client *initiator)
+{
+  assert(conf);
+  assert(conf->type == CONF_SERVER);
+  assert(hash_find_server(conf->name) == NULL);  /* This should have been checked by the caller */
+
+  /* Still processing a DNS lookup? -> exit */
+  if (conf->dns_pending)
+  {
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                   "Error connecting to %s: DNS lookup for connect{} in progress.",
+                   conf->name);
+    return false;
+  }
+
+  if (conf->dns_failed)
+  {
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                   "Error connecting to %s: DNS lookup for connect{} failed.",
+                   conf->name);
+    return false;
+  }
+
+  char buf[HOSTIPLEN + 1];
+  address_to_string(conf->addr, buf, sizeof(buf));
+  log_write(LOG_TYPE_IRCD, "Connect to %s[%s] @%s", conf->name, conf->host, buf);
+
+  /* Create a socket for the server connection */
+  int fd = comm_socket(address_get_family(conf->addr), SOCK_STREAM, 0);
+  if (fd == -1)
+  {
+    /* Eek, failure to create the socket */
+    log_write(LOG_TYPE_IRCD, "opening stream socket to %s: %s",
+              conf->name, strerror(errno));
+    return false;
+  }
+
+  /* Create a local client */
+  struct Client *client = client_make(NULL);
+
+  /* Copy in the server, hostname, fd */
+  strlcpy(client->name, conf->name, sizeof(client->name));
+  strlcpy(client->host, conf->host, sizeof(client->host));
+
+  /* We already converted the ip once, so lets use it - stu */
+  strlcpy(client->sockhost, buf, sizeof(client->sockhost));
+
+  address_copy(&client->addr, conf->addr);
+  client->connection->fd = fd_open(fd, true, NULL);
+
+  /* Server names are always guaranteed under HOSTLEN chars */
+  fd_note(client->connection->fd, "Server: %s", client->name);
+
+  /*
+   * Attach config entries to client here rather than in server_connect_callback().
+   * This to avoid null pointer references.
+   */
+  conf_attach(client, conf);
+
+  server_make(client);
+
+  const char *initiator_name = initiator ? initiator->name : "AutoConn.";
+  client->serv->initiator_name = io_strdup(initiator_name);
+
+  SetConnecting(client);
+
+  /* Now, initiate the connection */
+  comm_connect_tcp(client->connection->fd, conf->addr, conf->port, conf->bind,
+                   server_connect_callback, client, conf->timeout);
+
+  /*
+   * At this point we have a connection in progress and a connect {} block
+   * attached to the client, the socket info should be saved in the client
+   * and it should either be resolved or have a valid address.
+   *
+   * The socket has been connected or connect is in progress.
+   */
+  return true;
+}
+
+/* server_connect_auto()
+ *
+ * inputs	- void pointer which is not used
+ * output	- NONE
+ * side effects	-
+ * scan through configuration and try new connections.
+ * Returns the calendar time when the next call to this
+ * function should be made latest. (No harm done if this
+ * is called earlier or later...)
+ */
+void
+server_connect_auto(void *unused)
+{
+  if (GlobalSetOptions.autoconnect == false)
+    return;
+
+  list_node_t *node;
+  LIST_FOREACH(node, connect_items.head)
+  {
+    struct MaskItem *conf = node->data;
+
+    assert(conf->type == CONF_SERVER);
+    assert(conf->class);
+
+    /* Also when already connecting! (update holdtimes) --SRB */
+    if (conf->port == 0 || !IsConfAllowAutoConn(conf))
+      continue;
+
+    /*
+     * Skip this entry if the use of it is still on hold until
+     * future. Otherwise handle this entry (and set it on hold
+     * until next time). Will reset only hold times, if already
+     * made one successfull connection... [this algorithm is
+     * a bit fuzzy... -- msa >;) ]
+     */
+    if (conf->until > io_time_get(IO_TIME_MONOTONIC_SEC))
+      continue;
+
+    conf->until = io_time_get(IO_TIME_MONOTONIC_SEC) + conf->class->con_freq;
+
+    if (conf->class->ref_count >= conf->class->max_total)
+      continue;
+
+    /*
+     * Found a CONNECT config with port specified, scan clients
+     * and see if this server is already connected?
+     */
+    if (hash_find_server(conf->name))
+      continue;
+
+    if (find_servconn_in_progress(conf->name))
+      continue;
+
+    /* Move this entry to the end of the list, if not already last */
+    if (node->next)
+    {
+      list_remove(node, &connect_items);
+      list_add_tail(conf, &conf->node, &connect_items);
+    }
+
+    /*
+     * We used to only print this if server_connect() actually
+     * succeeded, but since comm_tcp_connect() can call the callback
+     * immediately if there is an error, we were getting error messages
+     * in the wrong order. SO, we just print out the activated line,
+     * and let server_connect() / server_connect_callback() print an
+     * error afterwards if it fails.
+     *   -- adrian
+     */
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                   ConfigServerHide.hide_server_ips ? "Connection to %s activated." : "Connection to %s[%s] activated.",
+                   conf->name, conf->host);
+
+    server_connect(conf, NULL);
+    return;  /* We connect only one at time... */
+  }
+}
+
 struct Client *
 find_servconn_in_progress(const char *name)
 {
