@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2024 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,83 +24,36 @@
  */
 
 #include "stdinc.h"
+#include "io_string.h"
+#include "io_time.h"
 #include "defaults.h"
 #include "list.h"
-#include "user.h"
+#include "log.h"
+#include "memory.h"
+#include "misc.h"
+#include "rng_mt.h"
+#include "tls.h"
+#include "cap.h"
 #include "channel.h"
 #include "channel_mode.h"
 #include "client.h"
 #include "cloak.h"
 #include "hash.h"
 #include "id.h"
-#include "irc_string.h"
 #include "ircd.h"
+#include "ircd_hook.h"
 #include "listener.h"
 #include "motd.h"
 #include "numeric.h"
 #include "conf.h"
 #include "conf_gecos.h"
-#include "log.h"
 #include "server.h"
 #include "send.h"
-#include "memory.h"
-#include "rng_mt.h"
-#include "misc.h"
 #include "parse.h"
 #include "monitor.h"
 #include "isupport.h"
-#include "tls.h"
-
-static char umode_buffer[UMODE_MAX_STR];
-
-const struct user_modes *umode_map[256];
-const struct user_modes  umode_tab[] =
-{
-  { 'B', UMODE_BOT          },
-  { 'D', UMODE_DEAF         },
-  { 'F', UMODE_FARCONNECT   },
-  { 'G', UMODE_SOFTCALLERID },
-  { 'H', UMODE_HIDDEN       },
-  { 'X', UMODE_EXPIRATION   },
-  { 'R', UMODE_REGONLY      },
-  { 'S', UMODE_SECURE       },
-  { 'W', UMODE_WEBIRC       },
-  { 'Z', UMODE_SECUREONLY   },
-  { 'a', UMODE_ADMIN        },
-  { 'c', UMODE_CCONN        },
-  { 'e', UMODE_EXTERNAL     },
-  { 'f', UMODE_FLOOD        },
-  { 'g', UMODE_CALLERID     },
-  { 'i', UMODE_INVISIBLE    },
-  { 'j', UMODE_REJ          },
-  { 'k', UMODE_SKILL        },
-  { 'l', UMODE_LOCOPS       },
-  { 'n', UMODE_NCHANGE      },
-  { 'o', UMODE_OPER         },
-  { 'p', UMODE_HIDECHANS    },
-  { 'q', UMODE_HIDEIDLE     },
-  { 'r', UMODE_REGISTERED   },
-  { 's', UMODE_SERVNOTICE   },
-  { 'w', UMODE_WALLOP       },
-  { 'x', UMODE_CLOAK        },
-  { 'y', UMODE_SPY          },
-  { 'z', UMODE_SECURE       },
-  { '\0', 0 }
-};
-
-void
-user_modes_init(void)
-{
-  char *umode_buffer_ptr = umode_buffer;
-
-  for (const struct user_modes *tab = umode_tab; tab->c; ++tab)
-  {
-    umode_map[tab->c] = tab;
-    *umode_buffer_ptr++ = tab->c;
-  }
-
-  *umode_buffer_ptr = '\0';
-}
+#include "user.h"
+#include "user_mode.h"
 
 /* show_lusers()
  *
@@ -111,7 +64,7 @@ user_modes_init(void)
 void
 show_lusers(struct Client *client)
 {
-  if (ConfigServerHide.hide_servers == 0 || HasUMode(client, UMODE_OPER))
+  if (ConfigServerHide.hide_servers == 0 || user_mode_has_flag(client, UMODE_OPER))
     sendto_one_numeric(client, &me, RPL_LUSERCLIENT, (list_length(&global_client_list) - Count.invisi),
                        Count.invisi, list_length(&global_server_list));
   else
@@ -127,7 +80,7 @@ show_lusers(struct Client *client)
   if (list_length(channel_get_list()))
     sendto_one_numeric(client, &me, RPL_LUSERCHANNELS, list_length(channel_get_list()));
 
-  if (ConfigServerHide.hide_servers == 0 || HasUMode(client, UMODE_OPER))
+  if (ConfigServerHide.hide_servers == 0 || user_mode_has_flag(client, UMODE_OPER))
   {
     sendto_one_numeric(client, &me, RPL_LUSERME, list_length(&local_client_list), list_length(&local_server_list));
     sendto_one_numeric(client, &me, RPL_LOCALUSERS, list_length(&local_client_list), Count.max_loc);
@@ -203,18 +156,19 @@ user_introduce(struct Client *client)
 {
   monitor_signon(client);
 
-  sendto_server(client, 0, 0, ":%s UID %s %u %ju %s %s %s %s %s %s %s :%s",
-                client->servptr->id, client->name, client->hopcount + 1,
-                client->tsinfo, user_get_mode_str(client->umodes),
-                client->username, client->host, client->realhost,
-                client->sockhost, client->id, client->account, client->info);
+  sendto_servers(client, 0, 0, ":%s UID %s %u %ju %s %s %s %s %s %s %s :%s",
+                 client->servptr->id, client->name, client->hopcount + 1,
+                 client->tsinfo, user_mode_to_str(client->umodes),
+                 client->username, client->host, client->realhost,
+                 client->sockhost, client->id, client->account, client->info);
 
-  if (!EmptyString(client->tls_certfp))
-    sendto_server(client, 0, 0, ":%s CERTFP %s", client->id, client->tls_certfp);
+  if (!string_is_empty(client->tls_certfp))
+    sendto_servers(client, 0, 0, ":%s CERTFP %s", client->id, client->tls_certfp);
 
-  if (!EmptyString(client->tls_cipher))
-    sendto_server(client, 0, 0, ":%s METADATA client %s cipher :%s",
-                  client->servptr->id, client->id, client->tls_cipher);
+  if (!string_is_empty(client->tls_cipher))
+    sendto_servers(client, 0, 0, ":%s METADATA client %s cipher :%s",
+                   client->servptr->id, client->id, client->tls_cipher);
+  AddFlag(client, FLAGS_INTRODUCED);
 }
 
 /* user_welcome()
@@ -234,24 +188,25 @@ user_welcome(struct Client *client)
 
   if (HasFlag(client, FLAGS_TLS))
   {
-    AddUMode(client, UMODE_SECURE);
+    user_mode_set_flag(client, UMODE_SECURE);
 
     client->tls_cipher = io_strdup(tls_get_cipher(&client->connection->fd->tls));
     sendto_one_notice(client, &me, ":*** Connected securely via %s",
                       client->tls_cipher);
 
-    if (!EmptyString(client->tls_certfp))
+    if (!string_is_empty(client->tls_certfp))
       sendto_one_notice(client, &me, ":*** Your client certificate fingerprint is %s",
                         client->tls_certfp);
   }
 
-  sendto_one_numeric(client, &me, RPL_WELCOME, ConfigServerInfo.network_name,
-                     client->name, client->username, client->realhost);
-  sendto_one_numeric(client, &me, RPL_YOURHOST,
-                     listener_get_name(client->connection->listener), IRCD_VERSION);
+  sendto_one_numeric(client, &me, RPL_WELCOME,
+                     ConfigServerInfo.network_name, client->name, client->username, client->realhost);
+  sendto_one_numeric(client, &me, RPL_YOURHOST, me.name,
+                     listener_get_name(client->connection->listener),
+                     listener_get_port(client->connection->listener), IRCD_VERSION);
   sendto_one_numeric(client, &me, RPL_CREATED, built_date);
-  sendto_one_numeric(client, &me, RPL_MYINFO, me.name, IRCD_VERSION, umode_buffer,
-                     cmode_rpl04[0], cmode_rpl04[1]);
+  sendto_one_numeric(client, &me, RPL_MYINFO,
+                     me.name, IRCD_VERSION, user_mode_get_string(), cmode_rpl04[0], cmode_rpl04[1]);
 
   isupport_show(client);
   show_lusers(client);
@@ -292,23 +247,20 @@ user_register_local(struct Client *client)
   if (conf_check_client(client) == false)
     return;
 
-  const struct MaskItem *const conf = client->connection->confs.head->data;
+  const struct MaskItem *const conf = list_peek_head(&client->connection->confs);
   if (IsNeedIdentd(conf) && !HasFlag(client, FLAGS_GOTID))
   {
     sendto_one_notice(client, &me, ":*** Notice -- You need to install identd to use this server");
-    exit_client(client, "Install identd");
+    client_exit(client, "Install identd");
     ++ServerStats.is_ref;
     return;
   }
 
   if (valid_username(client->username, true) == false)
   {
-    sendto_realops_flags(UMODE_REJ, L_ALL, SEND_NOTICE, "Invalid username: %s (%s@%s)",
-                         client->name, client->username, client->host);
-
-    char buf[sizeof("Invalid username []") + sizeof(client->username)];
-    snprintf(buf, sizeof(buf), "Invalid username [%s]", client->username);
-    exit_client(client, buf);
+    sendto_clients(UMODE_REJ, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "Invalid username: %s (%s@%s)",
+                   client->name, client->username, client->host);
+    client_exit_fmt(client, "Invalid username [%s]", client->username);
     ++ServerStats.is_ref;
     return;
   }
@@ -326,9 +278,9 @@ user_register_local(struct Client *client)
     max_clients += MAX_BUFFER;
   if (list_length(&local_client_list) >= max_clients)
   {
-    sendto_realops_flags(UMODE_REJ, L_ALL, SEND_NOTICE, "Too many clients, rejecting %s[%s].",
-                         client->name, client->host);
-    exit_client(client, "Sorry, server is full - try later");
+    sendto_clients(UMODE_REJ, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "Too many clients, rejecting %s[%s].",
+                   client->name, client->host);
+    client_exit(client, "Sorry, server is full - try later");
     ++ServerStats.is_ref;
     return;
   }
@@ -338,24 +290,21 @@ user_register_local(struct Client *client)
     const struct GecosItem *gecos = gecos_find(client->info, match);
     if (gecos)
     {
-      sendto_realops_flags(UMODE_REJ, L_ALL, SEND_NOTICE,
-                           "X-line Rejecting [%s] [%s], user %s [%s]",
-                           client->info, gecos->reason,
-                           client_get_name(client, HIDE_IP),
-                           client->sockhost);
-      exit_client(client, "Bad user info");
+      sendto_clients(UMODE_REJ, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "X-line Rejecting [%s] [%s], user %s [%s]",
+                     client->info, gecos->reason, client_get_name(client, HIDE_IP), client->sockhost);
+      client_exit(client, "Bad user info");
       ++ServerStats.is_ref;
       return;
     }
   }
 
   /* Password check */
-  if (!EmptyString(conf->passwd))
+  if (!string_is_empty(conf->passwd))
   {
-    if (match_conf_password(client->connection->password, conf) == false)
+    if (conf_match_password(client->connection->password, conf) == false)
     {
       sendto_one_numeric(client, &me, ERR_PASSWDMISMATCH);
-      exit_client(client, "Bad Password");
+      client_exit(client, "Bad Password");
       ++ServerStats.is_ref;
       return;
     }
@@ -371,23 +320,12 @@ user_register_local(struct Client *client)
   strlcpy(client->id, id, sizeof(client->id));
   hash_add_id(client);
 
-  sendto_realops_flags(UMODE_CCONN, L_ALL, SEND_NOTICE,
-                       "Client connecting: %s (%s@%s) [%s] {%s} [%s] <%s>",
-                       client->name, client->username, client->realhost,
-                       client->sockhost,
-                       class_get_name(&client->connection->confs),
-                       client->info, client->id);
-
-  if (ConfigGeneral.invisible_on_connect)
-  {
-    AddUMode(client, UMODE_INVISIBLE);
-    ++Count.invisi;
-  }
+  hook_dispatch(ircd_hook_user_register_local, &(ircd_hook_user_register_ctx){ .client = client });
 
   SetClient(client);
 
   client->servptr = &me;
-  client->connection->last_privmsg = event_base->time.sec_monotonic;
+  client->connection->last_privmsg = io_time_get(IO_TIME_MONOTONIC_SEC);
 
   list_add(client, &client->lnode, &client->servptr->serv->client_list);
   list_add(client, &client->node, &global_client_list);
@@ -398,9 +336,9 @@ user_register_local(struct Client *client)
     Count.max_loc = list_length(&local_client_list);
 
     if (!(Count.max_loc % 10))
-      sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                           "New peak in local client connections: %u",
-                           Count.max_loc);
+      sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                     "New peak in local client connections: %u",
+                     Count.max_loc);
   }
 
   if ((list_length(&local_client_list) + list_length(&local_server_list)) > Count.max_loc_con)
@@ -410,6 +348,9 @@ user_register_local(struct Client *client)
     Count.max_tot = list_length(&global_client_list);
   ++Count.totalrestartcount;
 
+  if (ConfigGeneral.invisible_on_connect)
+    user_mode_set_flag_exec(client, UMODE_INVISIBLE, USER_MODE_SOURCE_REGULAR);
+
   /*
    * Report if user has &^>= etc. and set flags as needed in client
    */
@@ -417,17 +358,10 @@ user_register_local(struct Client *client)
 
   user_welcome(client);
 
-  if (!HasFlag(client, FLAGS_SPOOF))
-  {
-    const char *const cloak = cloak_compute(&client->addr);
-    if (cloak)
-    {
-      user_set_hostmask(client, cloak, false);
-      AddUMode(client, UMODE_CLOAK);
-    }
-  }
+  user_mode_set_flag_exec(client, UMODE_CLOAK, USER_MODE_SOURCE_REGULAR);
+
+  user_mode_send(client, 0, USER_MODE_SEND_CLIENT);
 
-  send_umode(client, 0, true, false);
   user_introduce(client);
 }
 
@@ -460,12 +394,7 @@ user_register_remote(struct Client *client)
   if (list_length(&global_client_list) > Count.max_tot)
     Count.max_tot = list_length(&global_client_list);
 
-  if (HasFlag(client->servptr, FLAGS_EOB))
-    sendto_realops_flags(UMODE_FARCONNECT, L_ALL, SEND_NOTICE,
-                         "Client connecting at %s: %s (%s@%s) [%s] [%s] <%s>",
-                         client->servptr->name,
-                         client->name, client->username, client->realhost,
-                         client->sockhost, client->info, client->id);
+  hook_dispatch(ircd_hook_user_register_remote, &(ircd_hook_user_register_ctx){ .client = client });
 
   user_introduce(client);
 }
@@ -486,7 +415,7 @@ valid_hostname(const char *hostname)
 
   assert(p);
 
-  if (EmptyString(p) || *p == '.' || *p == ':')
+  if (string_is_empty(p) || *p == '.' || *p == ':')
     return false;
 
   for (; *p; ++p)
@@ -569,7 +498,7 @@ valid_nickname(const char *nickname, bool local)
   /*
    * Nicks can't start with a digit or - or be 0 length.
    */
-  if (EmptyString(p) || *p == '-' || (IsDigit(*p) && local))
+  if (string_is_empty(p) || *p == '-' || (IsDigit(*p) && local))
     return false;
 
   for (; *p; ++p)
@@ -579,81 +508,6 @@ valid_nickname(const char *nickname, bool local)
   return p - nickname <= NICKLEN;
 }
 
-/*! \brief Builds a mode change string to buffer pointed by \a buf
- * \param client  Pointer to client
- * \param send    Whether to send a MODE message to client
- * \param old     Old user mode to compare against when building new mode buffer
- */
-void
-send_umode(struct Client *client, unsigned int old, bool send_client, bool send_server)
-{
-  /*
-   * If the client's current user modes match the old modes, indicating no change,
-   * simply return without further processing.
-   */
-  if (client->umodes == old)
-    return;
-
-  char buf[UMODE_MAX_STR];
-  char *m = buf;
-  int what = MODE_NONE;
-
-  /*
-   * Construct a string in the 'buf' array to represent the modifications in the user's mode
-   * between the current modes (client->umodes) and the provided 'old' modes.
-   */
-  for (const struct user_modes *tab = umode_tab; tab->c; ++tab)
-  {
-    if ((tab->flag & old) && !HasUMode(client, tab->flag))
-    {
-      if (what == MODE_DEL)
-        *m++ = tab->c;
-      else
-      {
-        what = MODE_DEL;
-        *m++ = '-';
-        *m++ = tab->c;
-      }
-    }
-    else if (!(tab->flag & old) && HasUMode(client, tab->flag))
-    {
-      if (what == MODE_ADD)
-        *m++ = tab->c;
-      else
-      {
-        what = MODE_ADD;
-        *m++ = '+';
-        *m++ = tab->c;
-      }
-    }
-  }
-
-  *m = '\0';
-  assert(buf[0] != '\0');
-
-  if (send_client)
-    sendto_one(client, ":%s!%s@%s MODE %s :%s",
-               client->name, client->username, client->host, client->name, buf);
-  if (send_server)
-    sendto_server(client, 0, 0, ":%s MODE %s :%s",
-                  client->id, client->id, buf);
-}
-
-const char *
-user_get_mode_str(unsigned int modes)
-{
-  static char buf[UMODE_MAX_STR];
-  char *m = buf;
-
-  *m++ = '+';
-  for (const struct user_modes *tab = umode_tab; tab->c; ++tab)
-    if (modes &tab->flag)
-      *m++ = tab->c;
-  *m = '\0';
-
-  return buf;
-}
-
 void
 user_set_hostmask(struct Client *client, const char *hostname, bool svshost)
 {
@@ -665,14 +519,13 @@ user_set_hostmask(struct Client *client, const char *hostname, bool svshost)
                                  client->name, client->username, client->host);
 
   sendto_common_channels_local(client, true, CAP_CHGHOST, 0, ":%s!%s@%s CHGHOST %s %s",
-                               client->name, client->username,
-                               client->host, client->username, hostname);
+                               client->name, client->username, client->host, client->username, hostname);
 
   strlcpy(client->host, hostname, sizeof(client->host));
 
   if (svshost)
-    sendto_server(client, 0, 0, ":%s SVSHOST %s %ju %s",
-                  client->servptr->id, client->id, client->tsinfo, client->host);
+    sendto_servers(client, 0, 0, ":%s SVSHOST %s %ju %s",
+                   client->servptr->id, client->id, client->tsinfo, client->host);
 
   if (MyConnect(client))
   {
@@ -686,8 +539,8 @@ user_set_hostmask(struct Client *client, const char *hostname, bool svshost)
   list_node_t *node;
   LIST_FOREACH(node, client->channel.head)
   {
-    char modebuf[CMEMBER_STATUS_FLAGS_LEN + 1];
     char nickbuf[CMEMBER_STATUS_FLAGS_LEN * NICKLEN + CMEMBER_STATUS_FLAGS_LEN] = "";
+    char modebuf[CMEMBER_STATUS_FLAGS_LEN + 1];
     char *p = modebuf;
     int len = 0;
     const struct ChannelMember *member = node->data;
@@ -714,7 +567,7 @@ user_set_hostmask(struct Client *client, const char *hostname, bool svshost)
                            client->servptr->name, member->channel->name, modebuf, nickbuf);
   }
 
-  if (client->away[0])
+  if (client->away)
     sendto_common_channels_local(client, false, CAP_AWAY_NOTIFY, CAP_CHGHOST, ":%s!%s@%s AWAY :%s",
                                  client->name, client->username, client->host, client->away);
 }
